@@ -166,7 +166,7 @@ TEST(DOMCamera, Construction)
   cam.SetLensFunction("sin");
   EXPECT_EQ("sin", cam.LensFunction());
 
-  EXPECT_DOUBLE_EQ(IGN_PI_2, cam.LensCutoffAngle().Radian());
+  EXPECT_DOUBLE_EQ(GZ_PI_2, cam.LensCutoffAngle().Radian());
   cam.SetLensCutoffAngle(0.456);
   EXPECT_DOUBLE_EQ(0.456, cam.LensCutoffAngle().Radian());
 
@@ -214,12 +214,11 @@ TEST(DOMCamera, Construction)
   cam.SetLensProjectionTy(2);
   EXPECT_DOUBLE_EQ(2, cam.LensProjectionTy());
 
-  EXPECT_DOUBLE_EQ(1.0, cam.LensIntrinsicsSkew());
+  EXPECT_DOUBLE_EQ(0, cam.LensIntrinsicsSkew());
   cam.SetLensIntrinsicsSkew(2.3);
   EXPECT_DOUBLE_EQ(2.3, cam.LensIntrinsicsSkew());
 
   EXPECT_TRUE(cam.HasLensIntrinsics());
-
   EXPECT_TRUE(cam.HasLensProjection());
 
   EXPECT_EQ(UINT32_MAX, cam.VisibilityMask());
