@@ -14,9 +14,14 @@
  * limitations under the License.
  *
 */
+#include <optional>
 #include <sstream>
+
+#include <gz/math/Material.hh>
+#include <gz/math/Inertial.hh>
 #include "sdf/Capsule.hh"
 #include "sdf/parser.hh"
+#include "Utils.hh"
 
 using namespace sdf;
 
@@ -62,7 +67,7 @@ Errors Capsule::Load(ElementPtr _sdf)
   }
 
   {
-    std::pair<double, bool> pair = _sdf->Get<double>("radius",
+    std::pair<double, bool> pair = _sdf->Get<double>(errors, "radius",
         this->dataPtr->capsule.Radius());
 
     if (!pair.second)
@@ -77,7 +82,7 @@ Errors Capsule::Load(ElementPtr _sdf)
   }
 
   {
-    std::pair<double, bool> pair = _sdf->Get<double>("length",
+    std::pair<double, bool> pair = _sdf->Get<double>(errors, "length",
         this->dataPtr->capsule.Length());
 
     if (!pair.second)
@@ -136,17 +141,54 @@ gz::math::Capsuled &Capsule::Shape()
   return this->dataPtr->capsule;
 }
 
+/////////////////////////////////////////////////
+std::optional<gz::math::Inertiald> Capsule::CalculateInertial(double _density)
+{
+  gz::math::Material material = gz::math::Material(_density);
+  this->dataPtr->capsule.SetMat(material);
+
+  auto capsuleMassMatrix = this->dataPtr->capsule.MassMatrix();
+
+  if(!capsuleMassMatrix)
+  {
+    return std::nullopt;
+  }
+  else
+  {
+    gz::math::Inertiald capsuleInertial;
+    capsuleInertial.SetMassMatrix(capsuleMassMatrix.value());
+    return std::make_optional(capsuleInertial);
+  }
+}
+
+/////////////////////////////////////////////////
+gz::math::AxisAlignedBox Capsule::AxisAlignedBox() const
+{
+  auto halfSize = this->Radius() * gz::math::Vector3d::One +
+    this->Length() / 2 * gz::math::Vector3d::UnitZ;
+  return gz::math::AxisAlignedBox(-halfSize, halfSize);
+}
+
 /////////////////////////////////////////////////
 sdf::ElementPtr Capsule::ToElement() const
+{
+  sdf::Errors errors;
+  auto result = this->ToElement(errors);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Capsule::ToElement(sdf::Errors &_errors) const
 {
   sdf::ElementPtr elem(new sdf::Element);
   sdf::initFile("capsule_shape.sdf", elem);
 
-  sdf::ElementPtr radiusElem = elem->GetElement("radius");
-  radiusElem->Set(this->Radius());
+  sdf::ElementPtr radiusElem = elem->GetElement("radius", _errors);
+  radiusElem->Set(_errors, this->Radius());
 
-  sdf::ElementPtr lengthElem = elem->GetElement("length");
-  lengthElem->Set(this->Length());
+  sdf::ElementPtr lengthElem = elem->GetElement("length", _errors);
+  lengthElem->Set(_errors, this->Length());
 
   return elem;
 }
