@@ -15,23 +15,18 @@
  *
  */
 
+#include <cstdlib>
+#include <memory>
 #include <mutex>
-#include <string.h>
-#include <stdlib.h>
 #include <sstream>
-#include <boost/filesystem.hpp>
-
-/// \todo Remove this diagnositic push/pop in version 5
-#ifndef _WIN32
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-#include "sdf/Types.hh"
-#ifndef _WIN32
-#pragma GCC diagnostic pop
-#endif
+#include <string>
 
 #include "sdf/Console.hh"
+#include "sdf/Filesystem.hh"
+#include "sdf/Types.hh"
+#include "sdf/sdf_config.h"
+
+#include <gz/utils/Environment.hh>
 
 using namespace sdf;
 
@@ -39,56 +34,42 @@ using namespace sdf;
 static std::shared_ptr<Console> myself;
 static std::mutex g_instance_mutex;
 
-/// \todo Output disabled for windows, to allow tests to pass. We should
-/// disable output just for tests on windows.
-#ifndef _WIN32
 static bool g_quiet = false;
-#else
-static bool g_quiet = true;
-#endif
 
-static Console::ConsoleStream g_NullStream(NULL);
+static Console::ConsoleStream g_NullStream(nullptr);
 
 //////////////////////////////////////////////////
 Console::Console()
   : dataPtr(new ConsolePrivate)
 {
+#ifndef SDFORMAT_DISABLE_CONSOLE_LOGFILE
   // Set up the file that we'll log to.
-  try
-  {
 #ifndef _WIN32
-    char *home = getenv("HOME");
+  std::string homeVarName = "HOME";
 #else
-    const char *home = sdf::winGetEnv("HOMEPATH");
+  std::string homeVarName = "HOMEPATH";
 #endif
-    if (!home)
-    {
-      std::cerr << "No HOME defined in the environment. Will not log."
-                << std::endl;
-      return;
-    }
-    boost::filesystem::path logFile(home);
-    logFile /= ".sdformat";
-    logFile /= "sdformat.log";
-    boost::filesystem::path logDir = logFile.parent_path();
-    if (!boost::filesystem::exists(logDir))
-    {
-      boost::filesystem::create_directory(logDir);
-    }
-    else if (!boost::filesystem::is_directory(logDir))
-    {
-      std::cerr << logDir << " exists but is not a directory.  Will not log."
-                << std::endl;
-      return;
-    }
-    this->dataPtr->logFileStream.open(logFile.string().c_str(), std::ios::out);
-  }
-  catch(const boost::filesystem::filesystem_error& e)
+  std::string home;
+  if (!gz::utils::env(homeVarName, home))
   {
-    std::cerr << "Exception while setting up logging: " << e.what()
+    std::cerr << "No HOME defined in the environment. Will not log."
               << std::endl;
     return;
   }
+  std::string logDir = sdf::filesystem::append(home, ".sdformat");
+  if (!sdf::filesystem::exists(logDir))
+  {
+    sdf::filesystem::create_directory(logDir);
+  }
+  else if (!sdf::filesystem::is_directory(logDir))
+  {
+    std::cerr << logDir << " exists but is not a directory.  Will not log."
+              << std::endl;
+    return;
+  }
+  std::string logFile = sdf::filesystem::append(logDir, "sdformat.log");
+  this->dataPtr->logFileStream.open(logFile.c_str(), std::ios::out);
+#endif
 }
 
 //////////////////////////////////////////////////
@@ -101,17 +82,39 @@ ConsolePtr Console::Instance()
 {
   std::lock_guard<std::mutex> lock(g_instance_mutex);
   if (!myself)
+  {
     myself.reset(new Console());
+  }
 
   return myself;
 }
 
+//////////////////////////////////////////////////
+void Console::Clear()
+{
+  std::lock_guard<std::mutex> lock(g_instance_mutex);
+
+  myself = nullptr;
+}
+
 //////////////////////////////////////////////////
 void Console::SetQuiet(bool _quiet)
 {
   g_quiet = _quiet;
 }
 
+//////////////////////////////////////////////////
+sdf::Console::ConsoleStream &Console::GetMsgStream()
+{
+  return this->dataPtr->msgStream;
+}
+
+//////////////////////////////////////////////////
+sdf::Console::ConsoleStream &Console::GetLogStream()
+{
+  return this->dataPtr->logStream;
+}
+
 //////////////////////////////////////////////////
 Console::ConsoleStream &Console::ColorMsg(const std::string &lbl,
                                           const std::string &file,
@@ -139,18 +142,13 @@ Console::ConsoleStream &Console::Log(const std::string &lbl,
 
 //////////////////////////////////////////////////
 void Console::ConsoleStream::Prefix(const std::string &_lbl,
-    const std::string &_file,
-    unsigned int _line,
-    // This is here to prevent a compilation warning about an unused variable.
-#ifndef _WIN32
-    int _color
-#else
-    int /*_color*/
-#endif
-)
+                                    const std::string &_file,
+                                    unsigned int _line,
+                                    int _color)
 {
   size_t index = _file.find_last_of("/") + 1;
 
+  (void)_color;
   if (this->stream)
   {
 #ifndef _WIN32
@@ -169,3 +167,15 @@ void Console::ConsoleStream::Prefix(const std::string &_lbl,
       _file.substr(index , _file.size() - index)<< ":" << _line << "] ";
   }
 }
+
+//////////////////////////////////////////////////
+void Console::ConsoleStream::SetStream(std::ostream *_stream)
+{
+  this->stream = _stream;
+}
+
+//////////////////////////////////////////////////
+std::ostream *Console::ConsoleStream::GetStream()
+{
+  return this->stream;
+}
