@@ -30,6 +30,7 @@
 
 #include "Converter.hh"
 #include "EmbeddedSdf.hh"
+#include "Utils.hh"
 #include "XmlUtils.hh"
 
 using namespace sdf;
@@ -53,7 +54,8 @@ bool IsNotFlattenedElement(const std::string &_elemName)
 // used to update //pose/@relative_to in FindNewModelElements()
 void UpdatePose(tinyxml2::XMLElement *_elem,
                 const size_t &_childNameIdx,
-                const std::string &_modelName)
+                const std::string &_modelName,
+                sdf::Errors &_errors)
 {
   tinyxml2::XMLElement *pose = _elem->FirstChildElement("pose");
   if (pose && pose->Attribute("relative_to"))
@@ -68,15 +70,21 @@ void UpdatePose(tinyxml2::XMLElement *_elem,
   }
 
   if (_elem->FirstChildElement("camera"))
-    UpdatePose(_elem->FirstChildElement("camera"), _childNameIdx, _modelName);
+  {
+    UpdatePose(_elem->FirstChildElement("camera"), _childNameIdx,
+               _modelName, _errors);
+  }
 }
 }
 
 /////////////////////////////////////////////////
-bool Converter::Convert(tinyxml2::XMLDocument *_doc,
+bool Converter::Convert(sdf::Errors &_errors,
+                        tinyxml2::XMLDocument *_doc,
                         const std::string &_toVersion,
+                        const ParserConfig &_config,
                         bool _quiet)
 {
+
   SDF_ASSERT(_doc != nullptr, "SDF XML doc is NULL");
 
   tinyxml2::XMLElement *elem = _doc->FirstChildElement("sdf");
@@ -84,13 +92,15 @@ bool Converter::Convert(tinyxml2::XMLDocument *_doc,
   // Check that the <sdf> element exists
   if (!elem)
   {
-    sdferr << "<sdf> element does not exist.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "<sdf> element does not exist."});
     return false;
   }
 
-  if (!elem || !elem->Attribute("version"))
+  if (!elem->Attribute("version"))
   {
-    sdferr << "  Unable to determine original SDF version\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Unable to determine original SDF version"});
     return false;
   }
 
@@ -143,18 +153,24 @@ bool Converter::Convert(tinyxml2::XMLDocument *_doc,
     xmlDoc.Parse(convertXml);
     if (xmlDoc.Error())
     {
-      sdferr << "Error parsing XML from string: "
-             << xmlDoc.ErrorStr() << '\n';
+      std::stringstream ss;
+      ss << "Error parsing XML from string: "
+         << xmlDoc.ErrorStr();
+      _errors.push_back({ErrorCode::CONVERSION_ERROR, ss.str()});
       return false;
     }
-    ConvertImpl(elem, xmlDoc.FirstChildElement("convert"));
+    ConvertImpl(elem, xmlDoc.FirstChildElement("convert"), _config, _errors);
   }
 
   // Check that we actually converted to the desired final version.
   if (curVersion != _toVersion)
   {
-    sdferr << "Unable to convert from SDF version " << origVersion
-           << " to " << _toVersion << "\n";
+    std::stringstream ss;
+    ss << "Unable to convert from SDF version "
+       << origVersion
+       << " to "
+       << _toVersion;
+    _errors.push_back({ErrorCode::CONVERSION_ERROR, ss.str()});
     return false;
   }
 
@@ -162,18 +178,23 @@ bool Converter::Convert(tinyxml2::XMLDocument *_doc,
 }
 
 /////////////////////////////////////////////////
-void Converter::Convert(tinyxml2::XMLDocument *_doc,
-                        tinyxml2::XMLDocument *_convertDoc)
+void Converter::Convert(sdf::Errors &_errors,
+                        tinyxml2::XMLDocument *_doc,
+                        tinyxml2::XMLDocument *_convertDoc,
+                        const ParserConfig &_config)
 {
   SDF_ASSERT(_doc != NULL, "SDF XML doc is NULL");
   SDF_ASSERT(_convertDoc != NULL, "Convert XML doc is NULL");
 
-  ConvertImpl(_doc->FirstChildElement(), _convertDoc->FirstChildElement());
+  ConvertImpl(_doc->FirstChildElement(), _convertDoc->FirstChildElement(),
+              _config, _errors);
 }
 
 /////////////////////////////////////////////////
 void Converter::ConvertDescendantsImpl(tinyxml2::XMLElement *_e,
-                                       tinyxml2::XMLElement *_c)
+                                       tinyxml2::XMLElement *_c,
+                                       const ParserConfig &_config,
+                                       sdf::Errors &_errors)
 {
   if (!_c->Attribute("descendant_name"))
   {
@@ -195,21 +216,23 @@ void Converter::ConvertDescendantsImpl(tinyxml2::XMLElement *_e,
   {
     if (strcmp(e->Name(), _c->Attribute("descendant_name")) == 0)
     {
-      ConvertImpl(e, _c);
+      ConvertImpl(e, _c, _config, _errors);
     }
-    ConvertDescendantsImpl(e, _c);
+    ConvertDescendantsImpl(e, _c, _config, _errors);
     e = e->NextSiblingElement();
   }
 }
 
 /////////////////////////////////////////////////
 void Converter::ConvertImpl(tinyxml2::XMLElement *_elem,
-                            tinyxml2::XMLElement *_convert)
+                            tinyxml2::XMLElement *_convert,
+                            const ParserConfig &_config,
+                            sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != NULL, "SDF element is NULL");
   SDF_ASSERT(_convert != NULL, "Convert element is NULL");
 
-  CheckDeprecation(_elem, _convert);
+  CheckDeprecation(_elem, _convert, _config, _errors);
 
   for (auto *convertElem = _convert->FirstChildElement("convert");
        convertElem; convertElem = convertElem->NextSiblingElement("convert"))
@@ -220,13 +243,13 @@ void Converter::ConvertImpl(tinyxml2::XMLElement *_elem,
           convertElem->Attribute("name"));
       while (elem)
       {
-        ConvertImpl(elem, convertElem);
+        ConvertImpl(elem, convertElem, _config, _errors);
         elem = elem->NextSiblingElement(convertElem->Attribute("name"));
       }
     }
     if (convertElem->Attribute("descendant_name"))
     {
-      ConvertDescendantsImpl(_elem, convertElem);
+      ConvertDescendantsImpl(_elem, convertElem, _config, _errors);
     }
   }
 
@@ -237,41 +260,49 @@ void Converter::ConvertImpl(tinyxml2::XMLElement *_elem,
 
     if (name == "rename")
     {
-      Rename(_elem, childElem);
+      Rename(_elem, childElem, _errors);
     }
     else if (name == "copy")
     {
-      Move(_elem, childElem, true);
+      Move(_elem, childElem, true, _errors);
     }
     else if (name == "map")
     {
-      Map(_elem, childElem);
+      Map(_elem, childElem, _errors);
     }
     else if (name == "move")
     {
-      Move(_elem, childElem, false);
+      Move(_elem, childElem, false, _errors);
     }
     else if (name == "add")
     {
-      Add(_elem, childElem);
+      Add(_elem, childElem, _errors);
     }
     else if (name == "remove")
     {
-      Remove(_elem, childElem);
+      Remove(_errors, _elem, childElem);
+    }
+    else if (name == "remove_empty")
+    {
+      Remove(_errors, _elem, childElem, true);
     }
     else if (name == "unflatten")
     {
-      Unflatten(_elem);
+      Unflatten(_elem, _errors);
     }
     else if (name != "convert")
     {
-      sdferr << "Unknown convert element[" << name << "]\n";
+      std::stringstream ss;
+      ss << "Unknown convert element["
+         << name
+         << "]";
+      _errors.push_back({ErrorCode::CONVERSION_ERROR, ss.str()});
     }
   }
 }
 
 /////////////////////////////////////////////////
-void Converter::Unflatten(tinyxml2::XMLElement *_elem)
+void Converter::Unflatten(tinyxml2::XMLElement *_elem, sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != nullptr, "SDF element is nullptr");
 
@@ -300,7 +331,7 @@ void Converter::Unflatten(tinyxml2::XMLElement *_elem)
       // recursive unflatten
       if (elemName == "model")
       {
-        Unflatten(elem);
+        Unflatten(elem, _errors);
         break;
       }
 
@@ -311,9 +342,9 @@ void Converter::Unflatten(tinyxml2::XMLElement *_elem)
     tinyxml2::XMLElement *newModel = doc->NewElement("model");
     newModel->SetAttribute("name", newModelName.c_str());
 
-    if (FindNewModelElements(_elem, newModel, found + 2))
+    if (FindNewModelElements(_elem, newModel, found + 2, _errors))
     {
-      Unflatten(newModel);
+      Unflatten(newModel, _errors);
       _elem->InsertEndChild(newModel);
 
       // since newModel is inserted at the end, point back to the top element
@@ -328,7 +359,8 @@ void Converter::Unflatten(tinyxml2::XMLElement *_elem)
 /////////////////////////////////////////////////
 bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
                                     tinyxml2::XMLElement *_newModel,
-                                    const size_t &_childNameIdx)
+                                    const size_t &_childNameIdx,
+                                    sdf::Errors &_errors)
 {
   bool unflattenedNewModel = false;
   std::string newModelName = _newModel->Attribute("name");
@@ -388,7 +420,6 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
       if (elem->Attribute("attached_to"))
       {
         attachedTo = elem->Attribute("attached_to");
-
         SDF_ASSERT(attachedTo.compare(0, newModelNameSize, newModelName) == 0,
           "Error: Frame attribute 'attached_to' does not start with " +
           newModelName);
@@ -416,7 +447,7 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
           e;
           e = e->NextSiblingElement())
       {
-        UpdatePose(e, _childNameIdx, newModelName);
+        UpdatePose(e, _childNameIdx, newModelName, _errors);
       }
     }  // link
 
@@ -431,7 +462,7 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
         eText = e->GetText();
 
         SDF_ASSERT(eText.compare(0, newModelNameSize, newModelName) == 0,
-        "Error: Joint's <parent> value does not start with " + newModelName);
+          "Error: Joint's <parent> value does not start with " + newModelName);
 
         e->SetText(eText.substr(_childNameIdx).c_str());
       }
@@ -443,7 +474,7 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
         eText = e->GetText();
 
         SDF_ASSERT(eText.compare(0, newModelNameSize, newModelName) == 0,
-        "Error: Joint's <child> value does not start with " + newModelName);
+          "Error: Joint's <child> value does not start with " + newModelName);
 
         e->SetText(eText.substr(_childNameIdx).c_str());
       }
@@ -483,7 +514,7 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
           e;
           e = e->NextSiblingElement("sensor"))
       {
-        UpdatePose(e, _childNameIdx, newModelName);
+        UpdatePose(e, _childNameIdx, newModelName, _errors);
       }
     }  // joint
 
@@ -527,8 +558,8 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
         eText = e->GetText();
 
         SDF_ASSERT(eText.compare(0, newModelNameSize, newModelName) == 0,
-        "Error: Gripper's <palm_link> value does not start with "
-        + newModelName);
+          "Error: Gripper's <palm_link> value does not start with "
+          + newModelName);
 
         e->SetText(eText.substr(_childNameIdx).c_str());
       }
@@ -545,7 +576,8 @@ bool Converter::FindNewModelElements(tinyxml2::XMLElement *_elem,
 
 /////////////////////////////////////////////////
 void Converter::Rename(tinyxml2::XMLElement *_elem,
-                       tinyxml2::XMLElement *_renameElem)
+                       tinyxml2::XMLElement *_renameElem,
+                       sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != NULL, "SDF element is NULL");
   SDF_ASSERT(_renameElem != NULL, "Rename element is NULL");
@@ -567,7 +599,8 @@ void Converter::Rename(tinyxml2::XMLElement *_elem,
 
   if (!toElemName)
   {
-    sdferr << "No 'to' element name specified\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+                       "No 'to' element name specified"});
     return;
   }
 
@@ -597,7 +630,9 @@ void Converter::Rename(tinyxml2::XMLElement *_elem,
 }
 
 /////////////////////////////////////////////////
-void Converter::Add(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_addElem)
+void Converter::Add(tinyxml2::XMLElement *_elem,
+                    tinyxml2::XMLElement *_addElem,
+                    sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != NULL, "SDF element is NULL");
   SDF_ASSERT(_addElem != NULL, "Add element is NULL");
@@ -608,8 +643,8 @@ void Converter::Add(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_addElem)
 
   if (!((attributeName == nullptr) ^ (elementName == nullptr)))
   {
-    sdferr << "Exactly one 'element' or 'attribute'"
-           << " must be specified in <add>\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Exactly one 'element' or 'attribute' must be specified in <add>"});
     return;
   }
 
@@ -621,7 +656,8 @@ void Converter::Add(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_addElem)
     }
     else
     {
-      sdferr << "No 'value' specified in <add>\n";
+      _errors.push_back({ErrorCode::CONVERSION_ERROR,
+                         "No 'value' specified in <add>"});
       return;
     }
   }
@@ -639,25 +675,31 @@ void Converter::Add(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_addElem)
 }
 
 /////////////////////////////////////////////////
-void Converter::Remove(tinyxml2::XMLElement *_elem,
-                       tinyxml2::XMLElement *_removeElem)
+void Converter::Remove(sdf::Errors &_errors,
+                       tinyxml2::XMLElement *_elem,
+                       tinyxml2::XMLElement *_removeElem,
+                       bool _removeOnlyEmpty)
 {
   SDF_ASSERT(_elem != NULL, "SDF element is NULL");
-  SDF_ASSERT(_removeElem != NULL, "Move element is NULL");
+  SDF_ASSERT(_removeElem != NULL, "remove element is NULL");
 
   const char *attributeName = _removeElem->Attribute("attribute");
   const char *elementName = _removeElem->Attribute("element");
 
   if (!((attributeName == nullptr) ^ (elementName == nullptr)))
   {
-    sdferr << "Exactly one 'element' or 'attribute'"
-           << " must be specified in <remove>\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Exactly one 'element' or 'attribute' must be specified in <remove>"});
     return;
   }
 
   if (attributeName)
   {
-    _elem->DeleteAttribute(attributeName);
+    const char * attributeValue = _elem->Attribute(attributeName);
+    if (!_removeOnlyEmpty || (attributeValue && strlen(attributeValue) == 0))
+    {
+      _elem->DeleteAttribute(attributeName);
+    }
   }
   else
   {
@@ -665,14 +707,21 @@ void Converter::Remove(tinyxml2::XMLElement *_elem,
 
     while (childElem)
     {
-      _elem->DeleteChild(childElem);
-      childElem = _elem->FirstChildElement(elementName);
+      auto nextSibling = childElem->NextSiblingElement(elementName);
+      if (!_removeOnlyEmpty || (!childElem->FirstAttribute() &&
+          childElem->NoChildren() && !childElem->GetText()))
+      {
+        _elem->DeleteChild(childElem);
+      }
+      childElem = nextSibling;
     }
   }
 }
 
 /////////////////////////////////////////////////
-void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
+void Converter::Map(tinyxml2::XMLElement *_elem,
+                    tinyxml2::XMLElement *_mapElem,
+                    sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != nullptr, "SDF element is nullptr");
   SDF_ASSERT(_mapElem != nullptr, "Map element is nullptr");
@@ -682,26 +731,30 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
 
   if (!fromConvertElem)
   {
-    sdferr << "<map> element requires a <from> child element.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "<map> element requires a <from> child element."});
     return;
   }
   if (!toConvertElem)
   {
-    sdferr << "<map> element requires a <to> child element.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "<map> element requires a <to> child element."});
     return;
   }
 
   const char *fromNameStr = fromConvertElem->Attribute("name");
   const char *toNameStr = toConvertElem->Attribute("name");
 
-  if (!fromNameStr || fromNameStr[0] == '\0')
+  if (!fromNameStr || strlen(fromNameStr) == 0)
   {
-    sdferr << "Map: <from> element requires a non-empty name attribute.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <from> element requires a non-empty name attribute."});
     return;
   }
-  if (!toNameStr || toNameStr[0] == '\0')
+  if (!toNameStr || strlen(toNameStr) == 0)
   {
-    sdferr << "Map: <to> element requires a non-empty name attribute.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <to> element requires a non-empty name attribute."});
     return;
   }
 
@@ -711,22 +764,26 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
   auto *toValueElem = toConvertElem->FirstChildElement("value");
   if (!fromValueElem)
   {
-    sdferr << "Map: <from> element requires at least one <value> element.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <from> element requires at least one <value> element"});
     return;
   }
   if (!toValueElem)
   {
-    sdferr << "Map: <to> element requires at least one <value> element.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <to> element requires at least one <value> element."});
     return;
   }
   if (!fromValueElem->GetText())
   {
-    sdferr << "Map: from value must not be empty.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: from value must not be empty."});
     return;
   }
   if (!toValueElem->GetText())
   {
-    sdferr << "Map: to value must not be empty.\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: to value must not be empty."});
     return;
   }
   valueMap[fromValueElem->GetText()] = toValueElem->GetText();
@@ -739,12 +796,14 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
     }
     if (!fromValueElem->GetText())
     {
-      sdferr << "Map: from value must not be empty.\n";
+      _errors.push_back({ErrorCode::CONVERSION_ERROR,
+          "Map: from value must not be empty."});
       return;
     }
     if (!toValueElem->GetText())
     {
-      sdferr << "Map: to value must not be empty.\n";
+      _errors.push_back({ErrorCode::CONVERSION_ERROR,
+          "Map: to value must not be empty."});
       return;
     }
     valueMap[fromValueElem->GetText()] = toValueElem->GetText();
@@ -777,7 +836,8 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
   if (fromLeaf[0] == '\0' ||
       (fromLeaf[0] == '@' && fromLeaf[1] == '\0'))
   {
-    sdferr << "Map: <from> has invalid name attribute\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <from> has invalid name attribute"});
     return;
   }
   const char *fromValue = nullptr;
@@ -820,7 +880,8 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
   if (toLeaf[0] == '\0' ||
       (toLeaf[0] == '@' && toLeaf[1] == '\0'))
   {
-    sdferr << "Map: <to> has invalid name attribute\n";
+    _errors.push_back({ErrorCode::CONVERSION_ERROR,
+        "Map: <to> has invalid name attribute"});
     return;
   }
   bool toAttribute = toLeaf[0] == '@';
@@ -836,7 +897,8 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
     {
       if (toTokens[newDirIndex].empty())
       {
-        sdferr << "Map: <to> has invalid name attribute\n";
+        _errors.push_back({ErrorCode::CONVERSION_ERROR,
+            "Map: <to> has invalid name attribute"});
         return;
       }
 
@@ -861,7 +923,8 @@ void Converter::Map(tinyxml2::XMLElement *_elem, tinyxml2::XMLElement *_mapElem)
 /////////////////////////////////////////////////
 void Converter::Move(tinyxml2::XMLElement *_elem,
                      tinyxml2::XMLElement *_moveElem,
-                     const bool _copy)
+                     const bool _copy,
+                     sdf::Errors &_errors)
 {
   SDF_ASSERT(_elem != NULL, "SDF element is NULL");
   SDF_ASSERT(_moveElem != NULL, "Move element is NULL");
@@ -962,7 +1025,8 @@ void Converter::Move(tinyxml2::XMLElement *_elem,
 
     if (toElemStr && !toAttrStr)
     {
-      tinyxml2::XMLNode *cloned = DeepClone(moveFrom->GetDocument(), moveFrom);
+      tinyxml2::XMLNode *cloned = DeepClone(_errors, moveFrom->GetDocument(),
+                                            moveFrom);
       tinyxml2::XMLElement *moveTo = static_cast<tinyxml2::XMLElement*>(cloned);
 
       moveTo->SetValue(toName);
@@ -1047,7 +1111,9 @@ const char *Converter::GetValue(const char *_valueElem, const char *_valueAttr,
 
 /////////////////////////////////////////////////
 void Converter::CheckDeprecation(tinyxml2::XMLElement *_elem,
-                                 tinyxml2::XMLElement *_convert)
+                                 tinyxml2::XMLElement *_convert,
+                                 const ParserConfig &_config,
+                                 sdf::Errors &_errors)
 {
   // Process deprecated elements
   for (auto *deprecatedElem = _convert->FirstChildElement("deprecated");
@@ -1083,7 +1149,11 @@ void Converter::CheckDeprecation(tinyxml2::XMLElement *_elem,
       }
     }
 
-    sdfwarn << "Deprecated SDF Values in original file:\n"
-            << stream.str() << "\n\n";
+    std::stringstream ss;
+    ss << "Deprecated SDF Values in original file: \n"
+       << stream.str() << "\n";
+    Error err(ErrorCode::WARNING, ss.str());
+    sdf::enforceConfigurablePolicyCondition(_config.WarningsPolicy(),
+                                            err, _errors);
   }
 }
