@@ -22,6 +22,7 @@
 #include "sdf/Assert.hh"
 #include "sdf/Element.hh"
 #include "sdf/Filesystem.hh"
+#include "Utils.hh"
 
 using namespace sdf;
 
@@ -137,8 +138,21 @@ void Element::AddValue(const std::string &_type,
                        bool _required,
                        const std::string &_description)
 {
+  sdf::Errors errors;
   this->dataPtr->value = this->CreateParam(this->dataPtr->name,
-      _type, _defaultValue, _required, _description);
+      _type, _defaultValue, _required, errors, _description);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::AddValue(const std::string &_type,
+                       const std::string &_defaultValue,
+                       bool _required,
+                       sdf::Errors &_errors,
+                       const std::string &_description)
+{
+  this->dataPtr->value = this->CreateParam(this->dataPtr->name,
+      _type, _defaultValue, _required, _errors, _description);
 }
 
 /////////////////////////////////////////////////
@@ -148,10 +162,26 @@ void Element::AddValue(const std::string &_type,
                        const std::string &_minValue,
                        const std::string &_maxValue,
                        const std::string &_description)
+{
+  sdf::Errors errors;
+  this->AddValue(_type, _defaultValue, _required, _minValue, _maxValue,
+                 errors, _description);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::AddValue(const std::string &_type,
+                       const std::string &_defaultValue,
+                       bool _required,
+                       const std::string &_minValue,
+                       const std::string &_maxValue,
+                       sdf::Errors &_errors,
+                       const std::string &_description)
 {
   this->dataPtr->value =
       std::make_shared<Param>(this->dataPtr->name, _type, _defaultValue,
-                              _required, _minValue, _maxValue, _description);
+                              _required, _minValue, _maxValue, _errors,
+                              _description);
   SDF_ASSERT(this->dataPtr->value->SetParentElement(shared_from_this()),
       "Cannot set parent Element of value to itself.");
 }
@@ -161,10 +191,11 @@ ParamPtr Element::CreateParam(const std::string &_key,
                               const std::string &_type,
                               const std::string &_defaultValue,
                               bool _required,
+                              sdf::Errors &_errors,
                               const std::string &_description)
 {
   ParamPtr param = std::make_shared<Param>(
-      _key, _type, _defaultValue, _required, _description);
+      _key, _type, _defaultValue, _required, _errors, _description);
   SDF_ASSERT(param->SetParentElement(shared_from_this()),
       "Cannot set parent Element of created Param to itself.");
   return param;
@@ -176,13 +207,37 @@ void Element::AddAttribute(const std::string &_key,
                            const std::string &_defaultValue,
                            bool _required,
                            const std::string &_description)
+{
+  sdf::Errors errors;
+  this->AddAttribute(_key, _type, _defaultValue, _required,
+                     errors, _description);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::AddAttribute(const std::string &_key,
+                           const std::string &_type,
+                           const std::string &_defaultValue,
+                           bool _required,
+                           sdf::Errors &_errors,
+                           const std::string &_description)
 {
   this->dataPtr->attributes.push_back(
-      this->CreateParam(_key, _type, _defaultValue, _required, _description));
+      this->CreateParam(_key, _type, _defaultValue,
+                        _required, _errors, _description));
 }
 
 /////////////////////////////////////////////////
 ElementPtr Element::Clone() const
+{
+  sdf::Errors errors;
+  ElementPtr elem = this->Clone(errors);
+  sdf::throwOrPrintErrors(errors);
+  return elem;
+}
+
+/////////////////////////////////////////////////
+ElementPtr Element::Clone(sdf::Errors &_errors) const
 {
   ElementPtr clone(new Element);
   clone->dataPtr->description = this->dataPtr->description;
@@ -211,13 +266,13 @@ ElementPtr Element::Clone() const
   for (eiter = this->dataPtr->elementDescriptions.begin();
       eiter != this->dataPtr->elementDescriptions.end(); ++eiter)
   {
-    clone->dataPtr->elementDescriptions.push_back((*eiter)->Clone());
+    clone->dataPtr->elementDescriptions.push_back((*eiter)->Clone(_errors));
   }
 
   for (eiter = this->dataPtr->elements.begin();
        eiter != this->dataPtr->elements.end(); ++eiter)
   {
-    clone->dataPtr->elements.push_back((*eiter)->Clone());
+    clone->dataPtr->elements.push_back((*eiter)->Clone(_errors));
     clone->dataPtr->elements.back()->SetParent(clone);
   }
 
@@ -230,7 +285,8 @@ ElementPtr Element::Clone() const
 
   if (this->dataPtr->includeElement)
   {
-    clone->dataPtr->includeElement = this->dataPtr->includeElement->Clone();
+    clone->dataPtr->includeElement =
+        this->dataPtr->includeElement->Clone(_errors);
   }
 
   return clone;
@@ -239,6 +295,15 @@ ElementPtr Element::Clone() const
 /////////////////////////////////////////////////
 void Element::Copy(const ElementPtr _elem)
 {
+  sdf::Errors errors;
+  this->Copy(_elem, errors);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::Copy(const ElementPtr _elem, sdf::Errors &_errors)
+{
+
   this->dataPtr->name = _elem->GetName();
   this->dataPtr->description = _elem->GetDescription();
   this->dataPtr->required = _elem->GetRequired();
@@ -282,15 +347,16 @@ void Element::Copy(const ElementPtr _elem)
        _elem->dataPtr->elementDescriptions.begin();
        iter != _elem->dataPtr->elementDescriptions.end(); ++iter)
   {
-    this->dataPtr->elementDescriptions.push_back((*iter)->Clone());
+    this->dataPtr->elementDescriptions.push_back((*iter)->Clone(_errors));
   }
 
   this->dataPtr->elements.clear();
   for (ElementPtr_V::iterator iter = _elem->dataPtr->elements.begin();
        iter != _elem->dataPtr->elements.end(); ++iter)
   {
-    ElementPtr elem = (*iter)->Clone();
-    elem->Copy(*iter);
+    ElementPtr elem;
+    elem = (*iter)->Clone(_errors);
+    elem->Copy(*iter, _errors);
     elem->SetParent(shared_from_this());
     this->dataPtr->elements.push_back(elem);
   }
@@ -299,17 +365,28 @@ void Element::Copy(const ElementPtr _elem)
   {
     if (!this->dataPtr->includeElement)
     {
-      this->dataPtr->includeElement = _elem->dataPtr->includeElement->Clone();
+      this->dataPtr->includeElement =
+          _elem->dataPtr->includeElement->Clone(_errors);
     }
     else
     {
-      this->dataPtr->includeElement->Copy(_elem->dataPtr->includeElement);
+      this->dataPtr->includeElement->Copy(_elem->dataPtr->includeElement,
+                                          _errors);
     }
   }
 }
 
 /////////////////////////////////////////////////
 void Element::PrintDescription(const std::string &_prefix) const
+{
+  sdf::Errors errors;
+  this->PrintDescription(errors, _prefix);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::PrintDescription(sdf::Errors &_errors,
+                               const std::string &_prefix) const
 {
   std::cout << _prefix << "<element name ='" << this->dataPtr->name
             << "' required ='" << this->dataPtr->required << "'";
@@ -318,15 +395,16 @@ void Element::PrintDescription(const std::string &_prefix) const
   {
     std::cout << " type ='" << this->dataPtr->value->GetTypeName()
               << "'"
-              << " default ='" << this->dataPtr->value->GetDefaultAsString()
+              << " default ='"
+              << this->dataPtr->value->GetDefaultAsString(_errors)
               << "'";
-    auto minValue = this->dataPtr->value->GetMinValueAsString();
+    auto minValue = this->dataPtr->value->GetMinValueAsString(_errors);
     if (minValue.has_value())
     {
       std::cout << " min ='" << *minValue << "'";
     }
 
-    auto maxValue = this->dataPtr->value->GetMaxValueAsString();
+    auto maxValue = this->dataPtr->value->GetMaxValueAsString(_errors);
     if (maxValue.has_value())
     {
       std::cout << " max ='" << *maxValue << "'";
@@ -345,7 +423,7 @@ void Element::PrintDescription(const std::string &_prefix) const
   {
     std::cout << _prefix << "  <attribute name ='"
               << (*aiter)->GetKey() << "' type ='" << (*aiter)->GetTypeName()
-              << "' default ='" << (*aiter)->GetDefaultAsString()
+              << "' default ='" << (*aiter)->GetDefaultAsString(_errors)
               << "' required ='" << (*aiter)->GetRequired() << "'>\n";
     std::cout << _prefix << "    <description><![CDATA["
               << (*aiter)->GetDescription()
@@ -370,7 +448,7 @@ void Element::PrintDescription(const std::string &_prefix) const
   for (eiter = this->dataPtr->elementDescriptions.begin();
       eiter != this->dataPtr->elementDescriptions.end(); ++eiter)
   {
-    (*eiter)->PrintDescription(_prefix + "  ");
+    (*eiter)->PrintDescription(_errors, _prefix + "  ");
   }
 
   std::cout << _prefix << "</element>\n";
@@ -500,7 +578,8 @@ void Element::PrintDocLeftPane(std::string &_html, int _spacing,
 }
 
 /////////////////////////////////////////////////
-void Element::PrintValuesImpl(const std::string &_prefix,
+void Element::PrintValuesImpl(sdf::Errors &_errors,
+                              const std::string &_prefix,
                               bool _includeDefaultElements,
                               bool _includeDefaultAttributes,
                               const PrintConfig &_config,
@@ -508,13 +587,14 @@ void Element::PrintValuesImpl(const std::string &_prefix,
 {
   if (_config.PreserveIncludes() && this->GetIncludeElement() != nullptr)
   {
-    _out << this->GetIncludeElement()->ToString(_prefix, _config);
+    _out << this->GetIncludeElement()->ToString(_errors, _prefix, _config);
   }
   else if (this->GetExplicitlySetInFile() || _includeDefaultElements)
   {
     _out << _prefix << "<" << this->dataPtr->name;
 
-    this->dataPtr->PrintAttributes(_includeDefaultAttributes, _config, _out);
+    this->dataPtr->PrintAttributes(
+        _errors, _includeDefaultAttributes, _config, _out);
 
     if (this->dataPtr->elements.size() > 0)
     {
@@ -523,11 +603,12 @@ void Element::PrintValuesImpl(const std::string &_prefix,
       for (eiter = this->dataPtr->elements.begin();
            eiter != this->dataPtr->elements.end(); ++eiter)
       {
-        (*eiter)->ToString(_prefix + "  ",
+        (*eiter)->ToString(_errors,
+                           _out,
+                           _prefix + "  ",
                            _includeDefaultElements,
                            _includeDefaultAttributes,
-                           _config,
-                           _out);
+                           _config);
       }
       _out << _prefix << "</" << this->dataPtr->name << ">\n";
     }
@@ -535,7 +616,7 @@ void Element::PrintValuesImpl(const std::string &_prefix,
     {
       if (this->dataPtr->value)
       {
-        _out << ">" << this->dataPtr->value->GetAsString(_config)
+        _out << ">" << this->dataPtr->value->GetAsString(_errors, _config)
              << "</" << this->dataPtr->name << ">\n";
       }
       else
@@ -550,6 +631,18 @@ void Element::PrintValuesImpl(const std::string &_prefix,
 void ElementPrivate::PrintAttributes(bool _includeDefaultAttributes,
                                      const PrintConfig &_config,
                                      std::ostringstream &_out) const
+{
+  sdf::Errors errors;
+  this->PrintAttributes(errors, _includeDefaultAttributes, _config,
+                        _out);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void ElementPrivate::PrintAttributes(sdf::Errors &_errors,
+                                     bool _includeDefaultAttributes,
+                                     const PrintConfig &_config,
+                                     std::ostringstream &_out) const
 {
   // Attribute exceptions are used in the event of a non-default PrintConfig
   // which modifies the Attributes of this Element that are printed out. The
@@ -578,23 +671,35 @@ void ElementPrivate::PrintAttributes(bool _includeDefaultAttributes,
     // better separation of concerns if the conversion process set the
     // required attributes with their default values.
     if ((*aiter)->GetSet() || (*aiter)->GetRequired() ||
-        _includeDefaultAttributes)
+        _includeDefaultAttributes ||
+        ((*aiter)->GetKey().find(':') != std::string::npos))
     {
       const std::string key = (*aiter)->GetKey();
       const auto it = attributeExceptions.find(key);
       if (it == attributeExceptions.end())
       {
-        _out << " " << key << "='" << (*aiter)->GetAsString(_config) << "'";
+        _out << " " << key << "='"
+             << (*aiter)->GetAsString(_errors, _config) << "'";
       }
     }
   }
 }
 
 /////////////////////////////////////////////////
-void Element::PrintValues(std::string _prefix, const PrintConfig &_config) const
+void Element::PrintValues(std::string _prefix,
+                          const PrintConfig &_config) const
+{
+  sdf::Errors errors;
+  PrintValues(errors, _prefix, true, false, _config);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::PrintValues(sdf::Errors &_errors, std::string _prefix,
+                          const PrintConfig &_config) const
 {
   std::ostringstream ss;
-  PrintValuesImpl(_prefix, true, false, _config, ss);
+  PrintValuesImpl(_errors, _prefix, true, false, _config, ss);
   std::cout << ss.str();
 }
 
@@ -603,9 +708,26 @@ void Element::PrintValues(const std::string &_prefix,
                           bool _includeDefaultElements,
                           bool _includeDefaultAttributes,
                           const PrintConfig &_config) const
+{
+  sdf::Errors errors;
+  PrintValues(errors,
+              _prefix,
+              _includeDefaultElements,
+              _includeDefaultAttributes,
+              _config);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::PrintValues(sdf::Errors &_errors,
+                          const std::string &_prefix,
+                          bool _includeDefaultElements,
+                          bool _includeDefaultAttributes,
+                          const PrintConfig &_config) const
 {
   std::ostringstream ss;
-  PrintValuesImpl(_prefix,
+  PrintValuesImpl(_errors,
+                  _prefix,
                   _includeDefaultElements,
                   _includeDefaultAttributes,
                   _config,
@@ -617,9 +739,18 @@ void Element::PrintValues(const std::string &_prefix,
 std::string Element::ToString(const std::string &_prefix,
                               const PrintConfig &_config) const
 {
-  std::ostringstream out;
-  this->ToString(_prefix, true, false, _config, out);
-  return out.str();
+  sdf::Errors errors;
+  std::string out = this->ToString(errors, _prefix, _config);
+  sdf::throwOrPrintErrors(errors);
+  return out;
+}
+
+/////////////////////////////////////////////////
+std::string Element::ToString(sdf::Errors &_errors,
+                              const std::string &_prefix,
+                              const PrintConfig &_config) const
+{
+  return this->ToString(_errors, _prefix, true, false, _config);
 }
 
 /////////////////////////////////////////////////
@@ -628,23 +759,45 @@ std::string Element::ToString(const std::string &_prefix,
                               bool _includeDefaultAttributes,
                               const PrintConfig &_config) const
 {
+  sdf::Errors errors;
   std::ostringstream out;
-  this->ToString(_prefix,
+  this->ToString(errors,
+                 out,
+                 _prefix,
                  _includeDefaultElements,
                  _includeDefaultAttributes,
-                 _config,
-                 out);
+                 _config);
+  sdf::throwOrPrintErrors(errors);
   return out.str();
 }
 
 /////////////////////////////////////////////////
-void Element::ToString(const std::string &_prefix,
+std::string Element::ToString(sdf::Errors &_errors,
+                              const std::string &_prefix,
+                              bool _includeDefaultElements,
+                              bool _includeDefaultAttributes,
+                              const PrintConfig &_config) const
+{
+  std::ostringstream out;
+  this->ToString(_errors,
+                 out,
+                 _prefix,
+                 _includeDefaultElements,
+                 _includeDefaultAttributes,
+                 _config);
+  return out.str();
+}
+
+/////////////////////////////////////////////////
+void Element::ToString(sdf::Errors &_errors,
+                       std::ostringstream &_out,
+                       const std::string &_prefix,
                        bool _includeDefaultElements,
                        bool _includeDefaultAttributes,
-                       const PrintConfig &_config,
-                       std::ostringstream &_out) const
+                       const PrintConfig &_config) const
 {
-  PrintValuesImpl(_prefix,
+  PrintValuesImpl(_errors,
+                  _prefix,
                   _includeDefaultElements,
                   _includeDefaultAttributes,
                   _config,
@@ -863,12 +1016,33 @@ bool Element::HasUniqueChildNames(const std::string &_type) const
   return this->HasUniqueChildNames(_type, {});
 }
 
+/////////////////////////////////////////////////
+bool Element::HasUniqueChildNames(
+    sdf::Errors &_errors,
+    const std::string &_type) const
+{
+  return this->HasUniqueChildNames(_errors, _type, {});
+}
+
 /////////////////////////////////////////////////
 bool Element::HasUniqueChildNames(
     const std::string &_type,
     const std::vector<std::string> &_ignoreElements) const
 {
-  auto namedElementsCount = this->CountNamedElements(_type, _ignoreElements);
+  sdf::Errors errors;
+  bool result = this->HasUniqueChildNames(errors, _type, _ignoreElements);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+bool Element::HasUniqueChildNames(
+    sdf::Errors &_errors,
+    const std::string &_type,
+    const std::vector<std::string> &_ignoreElements) const
+{
+  auto namedElementsCount = this->CountNamedElements(
+      _errors, _type, _ignoreElements);
   for (auto &iter : namedElementsCount)
   {
     if (iter.second > 1)
@@ -888,6 +1062,26 @@ std::map<std::string, std::size_t> Element::CountNamedElements(
 
 /////////////////////////////////////////////////
 std::map<std::string, std::size_t> Element::CountNamedElements(
+    sdf::Errors &_errors,
+    const std::string &_type) const
+{
+  return this->CountNamedElements(_errors, _type, {});
+}
+
+/////////////////////////////////////////////////
+std::map<std::string, std::size_t> Element::CountNamedElements(
+    const std::string &_type,
+    const std::vector<std::string> &_ignoreElements) const
+{
+  sdf::Errors errors;
+  auto result = this->CountNamedElements(errors, _type, _ignoreElements);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+std::map<std::string, std::size_t> Element::CountNamedElements(
+    sdf::Errors &_errors,
     const std::string &_type,
     const std::vector<std::string> &_ignoreElements) const
 {
@@ -912,7 +1106,8 @@ std::map<std::string, std::size_t> Element::CountNamedElements(
       // Get("name") returns attribute value if it exists before checking
       // for the value of a child element <name>, so it's safe to use
       // here since we've checked HasAttribute("name").
-      std::string childNameAttributeValue = elem->Get<std::string>("name");
+      std::string childNameAttributeValue = elem->Get<std::string>(
+          _errors, "name");
       if (result.find(childNameAttributeValue) == result.end())
       {
         result[childNameAttributeValue] = 1;
@@ -931,11 +1126,20 @@ std::map<std::string, std::size_t> Element::CountNamedElements(
 
 /////////////////////////////////////////////////
 ElementPtr Element::GetElement(const std::string &_name)
+{
+  sdf::Errors errors;
+  ElementPtr result = this->GetElement(_name, errors);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+ElementPtr Element::GetElement(const std::string &_name, sdf::Errors &_errors)
 {
   ElementPtr result = this->GetElementImpl(_name);
   if (result == ElementPtr())
   {
-    result = this->AddElement(_name);
+    result = this->AddElement(_name, _errors);
   }
 
   return result;
@@ -975,6 +1179,15 @@ bool Element::HasElementDescription(const std::string &_name) const
 
 /////////////////////////////////////////////////
 ElementPtr Element::AddElement(const std::string &_name)
+{
+  sdf::Errors errors;
+  ElementPtr elem = this->AddElement(_name, errors);
+  sdf::throwOrPrintErrors(errors);
+  return elem;
+}
+
+/////////////////////////////////////////////////
+ElementPtr Element::AddElement(const std::string &_name, sdf::Errors &_errors)
 {
   // if this element is a reference sdf and does not have any element
   // descriptions then get them from its parent
@@ -986,7 +1199,7 @@ ElementPtr Element::AddElement(const std::string &_name)
     for (unsigned int i = 0; i < parent->GetElementDescriptionCount(); ++i)
     {
       this->dataPtr->elementDescriptions.push_back(
-          parent->GetElementDescription(i)->Clone());
+        parent->GetElementDescription(i)->Clone(_errors));
     }
   }
 
@@ -996,7 +1209,7 @@ ElementPtr Element::AddElement(const std::string &_name)
   {
     if ((*iter)->dataPtr->name == _name)
     {
-      ElementPtr elem = (*iter)->Clone();
+      ElementPtr elem = (*iter)->Clone(_errors);
       elem->SetParent(shared_from_this());
       this->dataPtr->elements.push_back(elem);
 
@@ -1007,15 +1220,15 @@ ElementPtr Element::AddElement(const std::string &_name)
         // Add only required child element
         if ((*iter2)->GetRequired() == "1")
         {
-          elem->AddElement((*iter2)->dataPtr->name);
+          elem->AddElement((*iter2)->dataPtr->name, _errors);
         }
       }
-
       return this->dataPtr->elements.back();
     }
   }
 
-  sdferr << "Missing element description for [" << _name << "]\n";
+  _errors.push_back({ErrorCode::ELEMENT_ERROR,
+      "Missing element description for [" + _name + "]\n"});
   return ElementPtr();
 }
 
@@ -1041,24 +1254,33 @@ void Element::ClearElements()
   this->dataPtr->elements.clear();
 }
 
+
 /////////////////////////////////////////////////
 void Element::Update()
+{
+  sdf::Errors errors;
+  this->Update(errors);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::Update(sdf::Errors &_errors)
 {
   for (sdf::Param_V::iterator iter = this->dataPtr->attributes.begin();
       iter != this->dataPtr->attributes.end(); ++iter)
   {
-    (*iter)->Update();
+    (*iter)->Update(_errors);
   }
 
   for (sdf::ElementPtr_V::iterator iter = this->dataPtr->elements.begin();
       iter != this->dataPtr->elements.end(); ++iter)
   {
-    (*iter)->Update();
+    (*iter)->Update(_errors);
   }
 
   if (this->dataPtr->value)
   {
-    this->dataPtr->value->Update();
+    this->dataPtr->value->Update(_errors);
   }
 }
 
@@ -1190,6 +1412,14 @@ void Element::RemoveFromParent()
 
 /////////////////////////////////////////////////
 void Element::RemoveChild(ElementPtr _child)
+{
+  sdf::Errors errors;
+  RemoveChild(_child, errors);
+  sdf::throwOrPrintErrors(errors);
+}
+
+/////////////////////////////////////////////////
+void Element::RemoveChild(ElementPtr _child, sdf::Errors &)
 {
   SDF_ASSERT(_child, "Cannot remove a nullptr child pointer");
 
@@ -1206,14 +1436,23 @@ void Element::RemoveChild(ElementPtr _child)
 
 /////////////////////////////////////////////////
 std::any Element::GetAny(const std::string &_key) const
+{
+  sdf::Errors errors;
+  std::any result = this->GetAny(errors, _key);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+std::any Element::GetAny(sdf::Errors &_errors, const std::string &_key) const
 {
   std::any result;
   if (_key.empty() && this->dataPtr->value)
   {
-    if (!this->dataPtr->value->GetAny(result))
+    if (!this->dataPtr->value->GetAny(result, _errors))
     {
-      sdferr << "Couldn't get element [" << this->GetName()
-             << "] as std::any\n";
+        _errors.push_back({ErrorCode::ELEMENT_ERROR,
+            "Couldn't get element [" + this->GetName() + "] as std::any\n"});
     }
   }
   else if (!_key.empty())
@@ -1221,9 +1460,10 @@ std::any Element::GetAny(const std::string &_key) const
     ParamPtr param = this->GetAttribute(_key);
     if (param)
     {
-      if (!this->GetAttribute(_key)->GetAny(result))
+      if (!this->GetAttribute(_key)->GetAny(result, _errors))
       {
-        sdferr << "Couldn't get attribute [" << _key << "] as std::any\n";
+        _errors.push_back({ErrorCode::ELEMENT_ERROR,
+            "Couldn't get attribute [" + _key + "] as std::any\n"});
       }
     }
     else
@@ -1231,18 +1471,19 @@ std::any Element::GetAny(const std::string &_key) const
       ElementPtr tmp = this->GetElementImpl(_key);
       if (tmp != ElementPtr())
       {
-        result = tmp->GetAny();
+        result = tmp->GetAny(_errors);
       }
       else
       {
         tmp = this->GetElementDescription(_key);
         if (tmp != ElementPtr())
         {
-          result = tmp->GetAny();
+          result = tmp->GetAny(_errors);
         }
         else
         {
-          sdferr << "Unable to find value for key [" << _key << "]\n";
+          _errors.push_back({ErrorCode::ELEMENT_ERROR,
+              "Unable to find value for key [" + _key + "]\n"});
         }
       }
     }
