@@ -15,8 +15,13 @@
  *
 */
 #include <sstream>
+#include <optional>
+
+#include <gz/math/Inertial.hh>
+#include <gz/math/Material.hh>
 #include "sdf/Ellipsoid.hh"
 #include "sdf/parser.hh"
+#include "Utils.hh"
 
 using namespace sdf;
 
@@ -24,7 +29,7 @@ using namespace sdf;
 class sdf::Ellipsoid::Implementation
 {
   /// \brief An ellipsoid with all three radii of 1 meter
-  public: ignition::math::Ellipsoidd ellipsoid{ignition::math::Vector3d::One};
+  public: gz::math::Ellipsoidd ellipsoid{gz::math::Vector3d::One};
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -32,7 +37,7 @@ class sdf::Ellipsoid::Implementation
 
 /////////////////////////////////////////////////
 Ellipsoid::Ellipsoid()
-  : dataPtr(ignition::utils::MakeImpl<Implementation>())
+  : dataPtr(gz::utils::MakeImpl<Implementation>())
 {
 }
 
@@ -63,9 +68,9 @@ Errors Ellipsoid::Load(ElementPtr _sdf)
 
   if (_sdf->HasElement("radii"))
   {
-    std::pair<ignition::math::Vector3d, bool> pair =
-      _sdf->Get<ignition::math::Vector3d>(
-        "radii", this->dataPtr->ellipsoid.Radii());
+    std::pair<gz::math::Vector3d, bool> pair =
+      _sdf->Get<gz::math::Vector3d>(
+        errors, "radii", this->dataPtr->ellipsoid.Radii());
 
     if (!pair.second)
     {
@@ -86,13 +91,13 @@ Errors Ellipsoid::Load(ElementPtr _sdf)
 }
 
 //////////////////////////////////////////////////
-ignition::math::Vector3d Ellipsoid::Radii() const
+gz::math::Vector3d Ellipsoid::Radii() const
 {
   return this->dataPtr->ellipsoid.Radii();
 }
 
 //////////////////////////////////////////////////
-void Ellipsoid::SetRadii(const ignition::math::Vector3d &_radii)
+void Ellipsoid::SetRadii(const gz::math::Vector3d &_radii)
 {
   this->dataPtr->ellipsoid.SetRadii(_radii);
 }
@@ -104,25 +109,61 @@ sdf::ElementPtr Ellipsoid::Element() const
 }
 
 /////////////////////////////////////////////////
-const ignition::math::Ellipsoidd &Ellipsoid::Shape() const
+const gz::math::Ellipsoidd &Ellipsoid::Shape() const
 {
   return this->dataPtr->ellipsoid;
 }
 
 /////////////////////////////////////////////////
-ignition::math::Ellipsoidd &Ellipsoid::Shape()
+gz::math::Ellipsoidd &Ellipsoid::Shape()
 {
   return this->dataPtr->ellipsoid;
 }
 
+/////////////////////////////////////////////////
+std::optional<gz::math::Inertiald> Ellipsoid::CalculateInertial(double _density)
+{
+  gz::math::Material material = gz::math::Material(_density);
+  this->dataPtr->ellipsoid.SetMat(material);
+
+  auto ellipsoidMassMatrix = this->dataPtr->ellipsoid.MassMatrix();
+
+  if(!ellipsoidMassMatrix)
+  {
+    return std::nullopt;
+  }
+  else
+  {
+    gz::math::Inertiald ellipsoidInertial;
+    ellipsoidInertial.SetMassMatrix(ellipsoidMassMatrix.value());
+    return std::make_optional(ellipsoidInertial);
+  }
+}
+
+/////////////////////////////////////////////////
+gz::math::AxisAlignedBox Ellipsoid::AxisAlignedBox() const
+{
+  auto halfSize = this->Radii();
+  return gz::math::AxisAlignedBox(-halfSize, halfSize);
+}
+
 /////////////////////////////////////////////////
 sdf::ElementPtr Ellipsoid::ToElement() const
+{
+  sdf::Errors errors;
+  auto result = this->ToElement(errors);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Ellipsoid::ToElement(sdf::Errors &_errors) const
 {
   sdf::ElementPtr elem(new sdf::Element);
   sdf::initFile("ellipsoid_shape.sdf", elem);
 
-  sdf::ElementPtr radiiElem = elem->GetElement("radii");
-  radiiElem->Set(this->Radii());
+  sdf::ElementPtr radiiElem = elem->GetElement("radii", _errors);
+  radiiElem->Set(_errors, this->Radii());
 
   return elem;
 }
