@@ -15,8 +15,14 @@
  *
 */
 
+#include <optional>
 #include <gtest/gtest.h>
 #include "sdf/Ellipsoid.hh"
+#include "test_utils.hh"
+#include <gz/math/Vector3.hh>
+#include <gz/math/Inertial.hh>
+#include <gz/math/Pose3.hh>
+#include <gz/math/MassMatrix3.hh>
 
 /////////////////////////////////////////////////
 TEST(DOMEllipsoid, Construction)
@@ -24,7 +30,7 @@ TEST(DOMEllipsoid, Construction)
   sdf::Ellipsoid ellipsoid;
   EXPECT_EQ(nullptr, ellipsoid.Element());
   // A default ellipsoid has all three radii set to 1
-  EXPECT_DOUBLE_EQ(IGN_PI * 4. / 3., ellipsoid.Shape().Volume());
+  EXPECT_DOUBLE_EQ(GZ_PI * 4. / 3., ellipsoid.Shape().Volume());
   EXPECT_EQ(gz::math::Vector3d::One, ellipsoid.Shape().Radii());
 
   const gz::math::Vector3d expectedRadii(1.0, 2.0, 3.0);
@@ -140,6 +146,52 @@ TEST(DOMEllipsoid, Shape)
   EXPECT_EQ(expectedRadii, ellipsoid.Radii());
 }
 
+/////////////////////////////////////////////////
+TEST(DOMEllipsoid, CalculateInertial)
+{
+  sdf::Ellipsoid ellipsoid;
+
+  // density of aluminium
+  const double density = 2170;
+
+  // Invalid dimensions leading to std::nullopt return in
+  // CalculateInertial()
+  ellipsoid.SetRadii(gz::math::Vector3d(-1, 2, 0));
+  auto invalidEllipsoidInertial = ellipsoid.CalculateInertial(density);
+  ASSERT_EQ(std::nullopt, invalidEllipsoidInertial);
+
+  const double a = 1.0;
+  const double b = 10.0;
+  const double c = 100.0;
+
+  ellipsoid.SetRadii(gz::math::Vector3d(a, b, c));
+
+  double expectedMass = ellipsoid.Shape().Volume() * density;
+  double ixx = (expectedMass / 5.0) * (b*b + c*c);
+  double iyy = (expectedMass / 5.0) * (a*a + c*c);
+  double izz = (expectedMass / 5.0) * (a*a + b*b);
+
+  gz::math::MassMatrix3d expectedMassMat(
+    expectedMass,
+    gz::math::Vector3d(ixx, iyy, izz),
+    gz::math::Vector3d::Zero
+  );
+
+  gz::math::Inertiald expectedInertial;
+  expectedInertial.SetMassMatrix(expectedMassMat);
+  expectedInertial.SetPose(gz::math::Pose3d::Zero);
+
+  auto ellipsoidInertial = ellipsoid.CalculateInertial(density);
+  EXPECT_EQ(ellipsoid.Shape().Mat().Density(), density);
+  ASSERT_NE(std::nullopt, ellipsoidInertial);
+  EXPECT_EQ(expectedInertial, *ellipsoidInertial);
+  EXPECT_EQ(expectedInertial.MassMatrix().DiagonalMoments(),
+    ellipsoidInertial->MassMatrix().DiagonalMoments());
+  EXPECT_EQ(expectedInertial.MassMatrix().Mass(),
+    ellipsoidInertial->MassMatrix().Mass());
+  EXPECT_EQ(expectedInertial.Pose(), ellipsoidInertial->Pose());
+}
+
 /////////////////////////////////////////////////
 TEST(DOMEllipsoid, ToElement)
 {
@@ -155,3 +207,50 @@ TEST(DOMEllipsoid, ToElement)
 
   EXPECT_EQ(ellipsoid.Radii(), ellipsoid2.Radii());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMEllipsoid, ToElementErrorOutput)
+{
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+  #ifdef _WIN32
+    sdf::Console::Instance()->SetQuiet(false);
+    sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+  #endif
+
+  sdf::Ellipsoid ellipsoid;
+  sdf::Errors errors;
+
+  ellipsoid.SetRadii(gz::math::Vector3d(0.1, 1.2, 3.4));
+
+  sdf::ElementPtr elem = ellipsoid.ToElement(errors);
+  EXPECT_TRUE(errors.empty());
+  ASSERT_NE(nullptr, elem);
+
+  sdf::Ellipsoid ellipsoid2;
+
+  errors = ellipsoid2.Load(elem);
+  EXPECT_TRUE(errors.empty());
+  EXPECT_EQ(ellipsoid.Radii(), ellipsoid2.Radii());
+
+  // Check nothing has been printed
+  EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+}
+
+/////////////////////////////////////////////////
+TEST(DOMEllipsoid, AxisAlignedBox)
+{
+  sdf::Ellipsoid ellipsoid;
+  ellipsoid.SetRadii(gz::math::Vector3d(1.5, 0.5, 0.25));
+
+  auto aabb = ellipsoid.AxisAlignedBox();
+  EXPECT_EQ(gz::math::Vector3d(3.0, 1.0, 0.5), aabb.Size());
+  EXPECT_EQ(gz::math::Vector3d(-1.5, -0.5, -0.25), aabb.Min());
+  EXPECT_EQ(gz::math::Vector3d(1.5, 0.5, 0.25), aabb.Max());
+}
