@@ -37,11 +37,11 @@ class sdf::Joint::Implementation
   /// \brief Name of the joint.
   public: std::string name = "";
 
-  /// \brief Name of the parent link.
-  public: std::string parentLinkName = "";
+  /// \brief Name of the parent frame.
+  public: std::string parentName = "";
 
-  /// \brief Name of the child link.
-  public: std::string childLinkName = "";
+  /// \brief Name of the child frame.
+  public: std::string childName = "";
 
   /// \brief the joint type.
   public: JointType type = JointType::INVALID;
@@ -52,7 +52,8 @@ class sdf::Joint::Implementation
   /// \brief Frame of the pose.
   public: std::string poseRelativeTo = "";
 
-  /// \brief Thread pitch for screw joints.
+  /// \brief Thread pitch for screw joints in meters / revolution with a
+  /// positive value for right-handed threads.
   public: double threadPitch = 1.0;
 
   /// \brief Joint axis
@@ -117,11 +118,11 @@ Errors Joint::Load(ElementPtr _sdf)
     _sdf->Get<std::string>("parent", "");
   if (parentPair.second)
   {
-    this->dataPtr->parentLinkName = parentPair.first;
-    if (!isValidFrameReference(this->dataPtr->parentLinkName))
+    this->dataPtr->parentName = parentPair.first;
+    if (!isValidFrameReference(this->dataPtr->parentName))
     {
       errors.push_back({ErrorCode::RESERVED_NAME,
-          "The supplied joint parent name [" + this->dataPtr->parentLinkName +
+          "The supplied joint parent name [" + this->dataPtr->parentName +
               "] is not valid."});
     }
   }
@@ -135,11 +136,11 @@ Errors Joint::Load(ElementPtr _sdf)
   std::pair<std::string, bool> childPair = _sdf->Get<std::string>("child", "");
   if (childPair.second)
   {
-    this->dataPtr->childLinkName = childPair.first;
-    if (!isValidFrameReference(this->dataPtr->childLinkName))
+    this->dataPtr->childName = childPair.first;
+    if (!isValidFrameReference(this->dataPtr->childName))
     {
       errors.push_back({ErrorCode::RESERVED_NAME,
-          "The supplied joint child name [" + this->dataPtr->childLinkName +
+          "The supplied joint child name [" + this->dataPtr->childName +
               "] is not valid."});
     }
   }
@@ -149,37 +150,68 @@ Errors Joint::Load(ElementPtr _sdf)
         "The child element is missing."});
   }
 
-  if (this->dataPtr->childLinkName == "world")
+  if (this->dataPtr->childName == "world")
   {
     errors.push_back({ErrorCode::JOINT_CHILD_LINK_INVALID,
         "Joint with name[" + this->dataPtr->name +
         "] specified invalid child link [world]."});
   }
 
-  if (this->dataPtr->childLinkName == this->dataPtr->parentLinkName)
+  if (this->dataPtr->childName == this->dataPtr->parentName)
   {
     errors.push_back({ErrorCode::JOINT_PARENT_SAME_AS_CHILD,
         "Joint with name[" + this->dataPtr->name +
         "] must specify different frame names for "
-        "parent and child, while [" + this->dataPtr->childLinkName +
+        "parent and child, while [" + this->dataPtr->childName +
         "] was specified for both."});
   }
 
-  if (_sdf->HasElement("axis"))
+  // Map of follower axis name to dataPtr->axis[] array index
+  // The keys of this map are also the only valid leader axis names
+  const std::map<std::string, std::size_t>
+      followerAxisNames = {{"axis", 0}, {"axis2", 1}};
+  for (const auto &[followerAxis, i] : followerAxisNames)
   {
-    this->dataPtr->axis[0].emplace();
-    Errors axisErrors = this->dataPtr->axis[0]->Load(_sdf->GetElement("axis"));
-    errors.insert(errors.end(), axisErrors.begin(), axisErrors.end());
+    if (_sdf->HasElement(followerAxis))
+    {
+      auto &axis = this->dataPtr->axis[i].emplace();
+      Errors axisErrors = axis.Load(_sdf->GetElement(followerAxis));
+      errors.insert(errors.end(), axisErrors.begin(), axisErrors.end());
+
+      if (axis.Mimic())
+      {
+        const auto leaderAxis = axis.Mimic()->Axis();
+        if (axis.Mimic()->Joint() == this->Name() &&
+            leaderAxis == followerAxis)
+        {
+          errors.push_back({ErrorCode::JOINT_AXIS_MIMIC_INVALID,
+            "Axis with name [" + followerAxis + "] in " +
+            "joint with name [" + this->dataPtr->name +
+            "] cannot mimic itself."});
+        }
+        if (followerAxisNames.find(leaderAxis) == followerAxisNames.end())
+        {
+          errors.push_back({ErrorCode::JOINT_AXIS_MIMIC_INVALID,
+            "Axis with name [" + followerAxis + "] in " +
+            "joint with name [" + this->dataPtr->name +
+            "] specified an invalid leader axis name [" + leaderAxis + "]."});
+        }
+      }
+    }
   }
 
-  if (_sdf->HasElement("axis2"))
+  if (_sdf->HasElement("screw_thread_pitch"))
   {
-    this->dataPtr->axis[1].emplace();
-    Errors axisErrors = this->dataPtr->axis[1]->Load(_sdf->GetElement("axis2"));
-    errors.insert(errors.end(), axisErrors.begin(), axisErrors.end());
+    // Prefer the screw_thread_pitch parameter if available.
+    this->dataPtr->threadPitch = _sdf->Get<double>("screw_thread_pitch");
   }
-
-  this->dataPtr->threadPitch = _sdf->Get<double>("thread_pitch", 1.0).first;
+  else if (_sdf->HasElement("thread_pitch"))
+  {
+    // If thread_pitch is available, convert to meters / revolution
+    // and fix sign.
+    this->dataPtr->threadPitch = -2*GZ_PI / _sdf->Get<double>("thread_pitch");
+  }
+  // Otherwise the default value of threadPitch will be used
 
   // Read the type
   std::pair<std::string, bool> typePair = _sdf->Get<std::string>("type", "");
@@ -250,28 +282,52 @@ void Joint::SetType(const JointType _jointType)
   this->dataPtr->type = _jointType;
 }
 
+/////////////////////////////////////////////////
+const std::string &Joint::ParentName() const
+{
+  return this->dataPtr->parentName;
+}
+
+/////////////////////////////////////////////////
+void Joint::SetParentName(const std::string &_name)
+{
+  this->dataPtr->parentName = _name;
+}
+
+/////////////////////////////////////////////////
+const std::string &Joint::ChildName() const
+{
+  return this->dataPtr->childName;
+}
+
+/////////////////////////////////////////////////
+void Joint::SetChildName(const std::string &_name)
+{
+  this->dataPtr->childName = _name;
+}
+
 /////////////////////////////////////////////////
 const std::string &Joint::ParentLinkName() const
 {
-  return this->dataPtr->parentLinkName;
+  return this->ParentName();
 }
 
 /////////////////////////////////////////////////
 void Joint::SetParentLinkName(const std::string &_name)
 {
-  this->dataPtr->parentLinkName = _name;
+  this->SetParentName(_name);
 }
 
 /////////////////////////////////////////////////
 const std::string &Joint::ChildLinkName() const
 {
-  return this->dataPtr->childLinkName;
+  return this->ChildName();
 }
 
 /////////////////////////////////////////////////
 void Joint::SetChildLinkName(const std::string &_name)
 {
-  this->dataPtr->childLinkName = _name;
+  this->SetChildName(_name);
 }
 
 /////////////////////////////////////////////////
@@ -352,7 +408,7 @@ Errors Joint::ResolveChildLink(std::string &_link) const
   }
 
   std::string link;
-  errors = resolveFrameAttachedToBody(link, graph, this->ChildLinkName());
+  errors = resolveFrameAttachedToBody(link, graph, this->ChildName());
   if (errors.empty())
   {
     _link = link;
@@ -367,7 +423,7 @@ Errors Joint::ResolveParentLink(std::string &_link) const
 
   // special case for world, return without resolving since it's not in a
   // model's FrameAttachedToGraph
-  if ("world" == this->ParentLinkName())
+  if ("world" == this->ParentName())
   {
     _link = "world";
     return errors;
@@ -382,7 +438,7 @@ Errors Joint::ResolveParentLink(std::string &_link) const
   }
 
   std::string link;
-  errors = resolveFrameAttachedToBody(link, graph, this->ParentLinkName());
+  errors = resolveFrameAttachedToBody(link, graph, this->ParentName());
   if (errors.empty())
   {
     _link = link;
@@ -397,22 +453,34 @@ sdf::SemanticPose Joint::SemanticPose() const
       this->dataPtr->name,
       this->dataPtr->pose,
       this->dataPtr->poseRelativeTo,
-      this->ChildLinkName(),
+      this->ChildName(),
       this->dataPtr->poseRelativeToGraph);
 }
 
 /////////////////////////////////////////////////
-double Joint::ThreadPitch() const
+double Joint::ScrewThreadPitch() const
 {
   return this->dataPtr->threadPitch;
 }
 
 /////////////////////////////////////////////////
-void Joint::SetThreadPitch(double _threadPitch)
+void Joint::SetScrewThreadPitch(double _threadPitch)
 {
   this->dataPtr->threadPitch = _threadPitch;
 }
 
+/////////////////////////////////////////////////
+double Joint::ThreadPitch() const
+{
+  return -2*GZ_PI / this->dataPtr->threadPitch;
+}
+
+/////////////////////////////////////////////////
+void Joint::SetThreadPitch(double _threadPitch)
+{
+  this->dataPtr->threadPitch = -2*GZ_PI / _threadPitch;
+}
+
 /////////////////////////////////////////////////
 sdf::ElementPtr Joint::Element() const
 {
@@ -516,8 +584,8 @@ sdf::ElementPtr Joint::ToElement() const
   }
 
   elem->GetAttribute("type")->Set<std::string>(jointType);
-  elem->GetElement("parent")->Set<std::string>(this->ParentLinkName());
-  elem->GetElement("child")->Set<std::string>(this->ChildLinkName());
+  elem->GetElement("parent")->Set<std::string>(this->ParentName());
+  elem->GetElement("child")->Set<std::string>(this->ChildName());
   for (unsigned int i = 0u; i < 2u; ++i)
   {
     const JointAxis *axis =  this->Axis(i);
