@@ -27,6 +27,7 @@
 #include "sdf/InterfaceModel.hh"
 #include "sdf/InterfaceModelPoseGraph.hh"
 #include "sdf/Joint.hh"
+#include "sdf/JointAxis.hh"
 #include "sdf/Link.hh"
 #include "sdf/Model.hh"
 #include "sdf/ParserConfig.hh"
@@ -126,6 +127,10 @@ class sdf::Model::Implementation
   /// ToElement() function to output only the plugin specified in the
   /// <include> tag when the ToElementUseIncludeTag policy is true..
   public: std::vector<Plugin> includePlugins;
+
+  /// \brief Whether the model was merge-included and needs to be processed to
+  /// carry out the merge.
+  public: bool isMerged{false};
 };
 
 /////////////////////////////////////////////////
@@ -183,6 +188,11 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
     }
   }
 
+  // Note: this attribute is not defined in the spec. It is used internally for
+  // implementing merge-includes when custom parsers are present.
+  this->dataPtr->isMerged =
+    _sdf->Get<bool>("__merge__", this->dataPtr->isMerged).first;
+
   this->dataPtr->placementFrameName = _sdf->Get<std::string>("placement_frame",
                              this->dataPtr->placementFrameName).first;
 
@@ -203,34 +213,40 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   {
     if (size > 1)
     {
-      sdfwarn << "Non-unique name[" << name << "] detected " << size
-              << " times in XML children of model with name[" << this->Name()
-              << "].\n";
+      std::stringstream ss;
+      ss << "Non-unique name[" << name << "] detected " << size
+         << " times in XML children of model with name[" << this->Name()
+         << "].";
+      Error err(ErrorCode::WARNING, ss.str());
+      enforceConfigurablePolicyCondition(
+          _config.WarningsPolicy(), err, errors);
     }
   }
+  std::unordered_set<std::string> nestedModelNames;
+  std::unordered_set<std::string> linkNames;
+  std::unordered_set<std::string> jointNames;
+  std::unordered_set<std::string> explicitFrameNames;
+  auto recordUniqueName = [&errors](std::unordered_set<std::string>& nameList,
+                                      const std::string &_elementName,
+                                      const std::string &_name)
+  {
+    if (nameList.count(_name) > 0)
+    {
+      errors.emplace_back(ErrorCode::DUPLICATE_NAME,
+          _elementName + " with name[" + _name + "] already exists.");
+      return false;
+    }
+    nameList.insert(_name);
+    return true;
+  };
 
   // Set of implicit and explicit frame names in this model for tracking
-  // name collisions
-  std::unordered_set<std::string> frameNames;
-
-  // Load nested models.
-  Errors nestedModelLoadErrors = loadUniqueRepeated<Model>(_sdf, "model",
-    this->dataPtr->models, _config);
-  errors.insert(errors.end(),
-                nestedModelLoadErrors.begin(),
-                nestedModelLoadErrors.end());
-
-  // Nested models are loaded first, and loadUniqueRepeated ensures there are no
-  // duplicate names, so these names can be added to frameNames without
-  // checking uniqueness.
-  for (const auto &model : this->dataPtr->models)
-  {
-    frameNames.insert(model.Name());
-  }
-
+  // name collisions. This is used to handle name clashes in old versions of
+  // SDFormat where sibling elements were allowed to have duplicate names.
+  std::unordered_set<std::string> implicitFrameNames;
   // Load InterfaceModels into a temporary container so we can have special
   // handling for merged InterfaceModels.
-  std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
+  std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelConstPtr>>
       tmpInterfaceModels;
   // Load included models via the interface API
   Errors interfaceModelLoadErrors = loadIncludedInterfaceModels(
@@ -242,7 +258,7 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   {
     if (!ifaceInclude.IsMerge().value_or(false))
     {
-      frameNames.insert(ifaceModel->Name());
+      implicitFrameNames.insert(ifaceModel->Name());
       this->dataPtr->interfaceModels.emplace_back(ifaceInclude, ifaceModel);
     }
     else
@@ -271,49 +287,205 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
         nestedInclude.SetIncludePoseRelativeTo(proxyModelFrameName);
         nestedInclude.SetIncludeRawPose(
             ifaceNestedModel->ModelFramePoseInParentFrame());
+        implicitFrameNames.insert(ifaceNestedModel->Name());
         this->dataPtr->interfaceModels.emplace_back(nestedInclude,
                                                     ifaceNestedModel);
       }
     }
   }
 
-  // Load all the links.
-  Errors linkLoadErrors = loadUniqueRepeated<Link>(_sdf, "link",
-    this->dataPtr->links);
-  errors.insert(errors.end(), linkLoadErrors.begin(), linkLoadErrors.end());
-
-  // Check links for name collisions and modify and warn if so.
-  for (auto &link : this->dataPtr->links)
+  for (auto elem = _sdf->GetFirstElement(); elem; elem = elem->GetNextElement())
   {
-    std::string linkName = link.Name();
-    if (frameNames.count(linkName) > 0)
+    const std::string &elementName = elem->GetName();
+    if (elementName == "model")
     {
-      // This link has a name collision
-      if (sdfVersion < gz::math::SemanticVersion(1, 7))
+      auto model = loadSingle<Model>(errors, elem, _config);
+      if (!recordUniqueName(nestedModelNames, elementName, model.Name()))
       {
-        // This came from an old file, so try to workaround by renaming link
-        linkName += "_link";
-        int i = 0;
-        while (frameNames.count(linkName) > 0)
+        continue;
+      }
+      if (model.IsMerged())
+      {
+        sdf::Frame proxyFrame = model.PrepareForMerge(errors, "__model__");
+        this->AddFrame(proxyFrame);
+
+        auto moveElements = [](const auto &_src, auto &_dest)
         {
-          linkName = link.Name() + "_link" + std::to_string(i++);
+          std::move(_src.begin(), _src.end(), std::back_inserter(_dest));
+        };
+        moveElements(model.dataPtr->links, this->dataPtr->links);
+        moveElements(model.dataPtr->frames, this->dataPtr->frames);
+        moveElements(model.dataPtr->joints, this->dataPtr->joints);
+        for (const auto &nestedModel : model.dataPtr->models)
+        {
+          implicitFrameNames.insert(nestedModel.Name());
         }
-        sdfwarn << "Link with name [" << link.Name() << "] "
-                << "in model with name [" << this->Name() << "] "
-                << "has a name collision, changing link name to ["
-                << linkName << "].\n";
-        link.SetName(linkName);
+        for (const auto &nestedInterfaceModel : model.dataPtr->interfaceModels)
+        {
+          implicitFrameNames.insert(nestedInterfaceModel.second->Name());
+        }
+        for (const auto &nestedMergedInterfaceModel :
+             model.dataPtr->mergedInterfaceModels)
+        {
+          for (const auto &nestedModel :
+               nestedMergedInterfaceModel.second->NestedModels())
+          {
+            implicitFrameNames.insert(nestedModel->Name());
+          }
+        }
+
+        moveElements(model.dataPtr->models, this->dataPtr->models);
+        moveElements(model.dataPtr->interfaceModels,
+                     this->dataPtr->interfaceModels);
+        moveElements(model.dataPtr->mergedInterfaceModels,
+                     this->dataPtr->mergedInterfaceModels);
       }
       else
       {
-        sdferr << "Link with name [" << link.Name() << "] "
-               << "in model with name [" << this->Name() << "] "
-               << "has a name collision. Please rename this link.\n";
+        implicitFrameNames.insert(model.Name());
+        this->dataPtr->models.push_back(std::move(model));
       }
     }
-    frameNames.insert(linkName);
+    else if (elementName == "link")
+    {
+      auto link = loadSingle<Link>(errors, elem, _config);
+      std::string linkName = link.Name();
+      if (!recordUniqueName(linkNames, elementName, linkName))
+      {
+        continue;
+      }
+
+      // Check links for name collisions and modify and warn if so.
+      if (implicitFrameNames.count(linkName) > 0)
+      {
+        // This link has a name collision
+        if (sdfVersion < gz::math::SemanticVersion(1, 7))
+        {
+          // This came from an old file, so try to workaround by renaming link
+          linkName += "_link";
+          int i = 0;
+          while (implicitFrameNames.count(linkName) > 0)
+          {
+            linkName = link.Name() + "_link" + std::to_string(i++);
+          }
+          std::stringstream ss;
+          ss << "Link with name [" << link.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision, changing link name to [" << linkName
+             << "].";
+          Error err(ErrorCode::WARNING, ss.str());
+          enforceConfigurablePolicyCondition(_config.WarningsPolicy(), err,
+                                             errors);
+          link.SetName(linkName);
+        }
+        else
+        {
+          std::stringstream ss;
+          ss << "Link with name [" << link.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision. Please rename this link.";
+          errors.push_back({ErrorCode::DUPLICATE_NAME, ss.str()});
+        }
+      }
+      implicitFrameNames.insert(linkName);
+      this->dataPtr->links.push_back(std::move(link));
+    }
+    else if (elementName == "joint")
+    {
+      auto joint = loadSingle<Joint>(errors, elem);
+      std::string jointName = joint.Name();
+      if (!recordUniqueName(jointNames, elementName, jointName))
+      {
+        continue;
+      }
+      // Check joints for name collisions and modify and warn if so.
+      if (implicitFrameNames.count(jointName) > 0)
+      {
+        // This joint has a name collision
+        if (sdfVersion < gz::math::SemanticVersion(1, 7))
+        {
+          // This came from an old file, so try to workaround by renaming joint
+          jointName += "_joint";
+          int i = 0;
+          while (implicitFrameNames.count(jointName) > 0)
+          {
+            jointName = joint.Name() + "_joint" + std::to_string(i++);
+          }
+          std::stringstream ss;
+          ss << "Joint with name [" << joint.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision, changing joint name to [" << jointName
+             << "].";
+          Error err(ErrorCode::WARNING, ss.str());
+          enforceConfigurablePolicyCondition(_config.WarningsPolicy(), err,
+                                             errors);
+
+          joint.SetName(jointName);
+        }
+        else
+        {
+          std::stringstream ss;
+          ss << "Joint with name [" << joint.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision. Please rename this joint.";
+          errors.push_back({ErrorCode::DUPLICATE_NAME, ss.str()});
+        }
+      }
+
+      implicitFrameNames.insert(jointName);
+      this->dataPtr->joints.push_back(std::move(joint));
+    }
+    else if (elementName == "frame")
+    {
+      auto frame = loadSingle<Frame>(errors, elem);
+      std::string frameName = frame.Name();
+      if (!recordUniqueName(explicitFrameNames, elementName, frameName))
+      {
+        continue;
+      }
+      // Check frames for name collisions and modify and warn if so.
+      if (implicitFrameNames.count(frameName) > 0)
+      {
+        // This frame has a name collision
+        if (sdfVersion < gz::math::SemanticVersion(1, 7))
+        {
+          // This came from an old file, so try to workaround by renaming frame
+          frameName += "_frame";
+          int i = 0;
+          while (implicitFrameNames.count(frameName) > 0)
+          {
+            frameName = frame.Name() + "_frame" + std::to_string(i++);
+          }
+          std::stringstream ss;
+          ss << "Frame with name [" << frame.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision, changing frame name to [" << frameName
+             << "].";
+          Error err(ErrorCode::WARNING, ss.str());
+          enforceConfigurablePolicyCondition(_config.WarningsPolicy(), err,
+                                             errors);
+
+          frame.SetName(frameName);
+        }
+        else
+        {
+          std::stringstream ss;
+          ss << "Frame with name [" << frame.Name() << "] "
+             << "in model with name [" << this->Name() << "] "
+             << "has a name collision. Please rename this frame.";
+          errors.push_back({ErrorCode::DUPLICATE_NAME, ss.str()});
+        }
+      }
+      implicitFrameNames.insert(frameName);
+      this->dataPtr->frames.push_back(std::move(frame));
+    }
   }
 
+  // Load the model plugins
+  Errors pluginErrors = loadRepeated<Plugin>(_sdf, "plugin",
+    this->dataPtr->plugins);
+  errors.insert(errors.end(), pluginErrors.begin(), pluginErrors.end());
+
   // If the model is not static and has no nested models:
   // Require at least one (interface) link so the implicit model frame can be
   // attached to something.
@@ -325,85 +497,6 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
                      "A model must have at least one link."});
   }
 
-  // Load all the joints.
-  Errors jointLoadErrors = loadUniqueRepeated<Joint>(_sdf, "joint",
-    this->dataPtr->joints);
-  errors.insert(errors.end(), jointLoadErrors.begin(), jointLoadErrors.end());
-
-  // Check joints for name collisions and modify and warn if so.
-  for (auto &joint : this->dataPtr->joints)
-  {
-    std::string jointName = joint.Name();
-    if (frameNames.count(jointName) > 0)
-    {
-      // This joint has a name collision
-      if (sdfVersion < gz::math::SemanticVersion(1, 7))
-      {
-        // This came from an old file, so try to workaround by renaming joint
-        jointName += "_joint";
-        int i = 0;
-        while (frameNames.count(jointName) > 0)
-        {
-          jointName = joint.Name() + "_joint" + std::to_string(i++);
-        }
-        sdfwarn << "Joint with name [" << joint.Name() << "] "
-                << "in model with name [" << this->Name() << "] "
-                << "has a name collision, changing joint name to ["
-                << jointName << "].\n";
-        joint.SetName(jointName);
-      }
-      else
-      {
-        sdferr << "Joint with name [" << joint.Name() << "] "
-               << "in model with name [" << this->Name() << "] "
-               << "has a name collision. Please rename this joint.\n";
-      }
-    }
-    frameNames.insert(jointName);
-  }
-
-  // Load all the frames.
-  Errors frameLoadErrors = loadUniqueRepeated<Frame>(_sdf, "frame",
-    this->dataPtr->frames);
-  errors.insert(errors.end(), frameLoadErrors.begin(), frameLoadErrors.end());
-
-  // Check frames for name collisions and modify and warn if so.
-  for (auto &frame : this->dataPtr->frames)
-  {
-    std::string frameName = frame.Name();
-    if (frameNames.count(frameName) > 0)
-    {
-      // This frame has a name collision
-      if (sdfVersion < gz::math::SemanticVersion(1, 7))
-      {
-        // This came from an old file, so try to workaround by renaming frame
-        frameName += "_frame";
-        int i = 0;
-        while (frameNames.count(frameName) > 0)
-        {
-          frameName = frame.Name() + "_frame" + std::to_string(i++);
-        }
-        sdfwarn << "Frame with name [" << frame.Name() << "] "
-                << "in model with name [" << this->Name() << "] "
-                << "has a name collision, changing frame name to ["
-                << frameName << "].\n";
-        frame.SetName(frameName);
-      }
-      else
-      {
-        sdferr << "Frame with name [" << frame.Name() << "] "
-               << "in model with name [" << this->Name() << "] "
-               << "has a name collision. Please rename this frame.\n";
-      }
-    }
-    frameNames.insert(frameName);
-  }
-
-  // Load the model plugins
-  Errors pluginErrors = loadRepeated<Plugin>(_sdf, "plugin",
-    this->dataPtr->plugins);
-  errors.insert(errors.end(), pluginErrors.begin(), pluginErrors.end());
-
   // Check whether the model was loaded from an <include> tag. If so, set
   // the URI and capture the plugins.
   if (_sdf->GetIncludeElement() && _sdf->GetIncludeElement()->HasElement("uri"))
@@ -731,7 +824,7 @@ std::pair<const Link*, std::string> Model::CanonicalLinkAndRelativeName() const
       auto firstModel = this->ModelByIndex(0);
       auto canonicalLinkAndName = firstModel->CanonicalLinkAndRelativeName();
       // Prepend firstModelName if a valid link is found.
-      if (nullptr != canonicalLinkAndName.first)
+      if (canonicalLinkAndName.second != "")
       {
         canonicalLinkAndName.second =
             firstModel->Name() + "::" + canonicalLinkAndName.second;
@@ -812,6 +905,23 @@ void Model::SetPoseRelativeTo(const std::string &_frame)
   this->dataPtr->poseRelativeTo = _frame;
 }
 
+/////////////////////////////////////////////////
+void Model::ResolveAutoInertials(sdf::Errors &_errors,
+                              const ParserConfig &_config)
+{
+  // Loop through all the nested models, if there are any
+  for (sdf::Model &model : this->dataPtr->models)
+  {
+    model.ResolveAutoInertials(_errors, _config);
+  }
+
+  // Calculate and set inertials for all the links in the model
+  for (sdf::Link &link : this->dataPtr->links)
+  {
+    link.ResolveAutoInertials(_errors, _config);
+  }
+}
+
 /////////////////////////////////////////////////
 void Model::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
@@ -1049,6 +1159,10 @@ sdf::ElementPtr Model::ToElement(const OutputConfig &_config) const
   for (const Plugin &plugin : this->dataPtr->plugins)
     elem->InsertElement(plugin.ToElement(), true);
 
+  // Frames
+  for (const sdf::Frame &frame : this->dataPtr->frames)
+    elem->InsertElement(frame.ToElement(), true);
+
   return elem;
 }
 
@@ -1145,3 +1259,158 @@ void Model::AddPlugin(const Plugin &_plugin)
 {
   this->dataPtr->plugins.push_back(_plugin);
 }
+
+/////////////////////////////////////////////////
+bool Model::IsMerged() const
+{
+  return this->dataPtr->isMerged;
+}
+
+/////////////////////////////////////////////////
+sdf::Frame Model::PrepareForMerge(sdf::Errors &_errors,
+                                  const std::string &_parentOfProxyFrame)
+{
+  // Build the pose graph of the model so we can adjust the pose of the proxy
+  // frame to take the placement frame into account.
+  auto poseGraph = std::make_shared<sdf::PoseRelativeToGraph>();
+  sdf::ScopedGraph<sdf::PoseRelativeToGraph> scopedPoseGraph(poseGraph);
+  sdf::Errors poseGraphErrors =
+      sdf::buildPoseRelativeToGraph(scopedPoseGraph, this);
+  _errors.insert(_errors.end(), poseGraphErrors.begin(), poseGraphErrors.end());
+  sdf::Errors poseValidationErrors =
+      validatePoseRelativeToGraph(scopedPoseGraph);
+  _errors.insert(_errors.end(), poseValidationErrors.begin(),
+                 poseValidationErrors.end());
+
+  auto frameGraph = std::make_shared<sdf::FrameAttachedToGraph>();
+  sdf::ScopedGraph<sdf::FrameAttachedToGraph> scopedFrameGraph(frameGraph);
+  sdf::Errors frameGraphErrors =
+      sdf::buildFrameAttachedToGraph(scopedFrameGraph, this);
+  _errors.insert(_errors.end(), frameGraphErrors.begin(),
+                 frameGraphErrors.end());
+  sdf::Errors frameValidationErrors =
+      validateFrameAttachedToGraph(scopedFrameGraph);
+  _errors.insert(_errors.end(), frameValidationErrors.begin(),
+                 frameValidationErrors.end());
+
+  this->SetPoseRelativeToGraph(scopedPoseGraph);
+  this->SetFrameAttachedToGraph(scopedFrameGraph);
+
+  const std::string proxyModelFrameName =
+      computeMergedModelProxyFrameName(this->Name());
+
+  sdf::Frame proxyFrame;
+  proxyFrame.SetName(proxyModelFrameName);
+  proxyFrame.SetAttachedTo(this->CanonicalLinkAndRelativeName().second);
+  gz::math::Pose3d modelPose = this->RawPose();
+  if (!this->PlacementFrameName().empty())
+  {
+    // Build the pose graph of the model so we can adjust the pose of the proxy
+    // frame to take the placement frame into account.
+    if (poseGraphErrors.empty())
+    {
+      // M - model frame (__model__)
+      // R - The `relative_to` frame of the placement frame's //pose element.
+      // See resolveModelPoseWithPlacementFrame in FrameSemantics.cc for
+      // notation and documentation
+      // Note, when the frame graph is built for a model with placement frame,
+      // it is built with an pose offset to take into account the placement
+      // frame. Therefore, we only need to resolve the pose of the model
+      // relative to `R` here. We could manually compute it as
+      // X_RM = X_RL * X_LM; where `L` is the link specified in the placement
+      // frame attribute.
+      gz::math::Pose3d X_RM = this->RawPose();
+      sdf::Errors resolveErrors = this->SemanticPose().Resolve(X_RM);
+      _errors.insert(_errors.end(), resolveErrors.begin(), resolveErrors.end());
+      modelPose = X_RM;
+    }
+  }
+
+  proxyFrame.SetRawPose(modelPose);
+  proxyFrame.SetPoseRelativeTo(this->PoseRelativeTo().empty()
+                                   ? _parentOfProxyFrame
+                                   : this->PoseRelativeTo());
+
+  auto isEmptyOrModelFrame = [](const std::string &_attr)
+  {
+    return _attr.empty() || _attr == "__model__";
+  };
+
+  // Merge links, frames, joints, and nested models.
+  for (auto &link : this->dataPtr->links)
+  {
+    if (isEmptyOrModelFrame(link.PoseRelativeTo()))
+    {
+      link.SetPoseRelativeTo(proxyModelFrameName);
+    }
+  }
+
+  for (auto &frame : this->dataPtr->frames)
+  {
+    if (isEmptyOrModelFrame(frame.AttachedTo()))
+    {
+      frame.SetAttachedTo(proxyModelFrameName);
+    }
+    if (frame.PoseRelativeTo() == "__model__")
+    {
+      frame.SetPoseRelativeTo(proxyModelFrameName);
+    }
+  }
+
+  for (auto &joint : this->dataPtr->joints)
+  {
+    if (joint.PoseRelativeTo() == "__model__")
+    {
+      joint.SetPoseRelativeTo(proxyModelFrameName);
+    }
+    if (joint.ParentName() == "__model__")
+    {
+      joint.SetParentName(proxyModelFrameName);
+    }
+    if (joint.ChildName() == "__model__")
+    {
+      joint.SetChildName(proxyModelFrameName);
+    }
+    for (unsigned int ai : {0, 1})
+    {
+      const sdf::JointAxis *axis = joint.Axis(ai);
+      if (axis && axis->XyzExpressedIn() == "__model__")
+      {
+        sdf::JointAxis axisCopy = *axis;
+        axisCopy.SetXyzExpressedIn(proxyModelFrameName);
+        joint.SetAxis(ai, axisCopy);
+      }
+    }
+  }
+
+  for (auto &nestedModel : this->dataPtr->models)
+  {
+    if (isEmptyOrModelFrame(nestedModel.PoseRelativeTo()))
+    {
+      nestedModel.SetPoseRelativeTo(proxyModelFrameName);
+    }
+  }
+  // Note: Since Model::Load is called recursively, all merge-include nested
+  // models would already have been merged by this point, so there is no need
+  // to call PrepareForMerge recursively here.
+
+  for (auto &ifaceModel : this->dataPtr->interfaceModels)
+  {
+    if (isEmptyOrModelFrame(
+            ifaceModel.first->IncludePoseRelativeTo().value_or("")))
+    {
+      ifaceModel.first->SetIncludePoseRelativeTo(proxyModelFrameName);
+    }
+  }
+
+  for (auto &ifaceModel : this->dataPtr->mergedInterfaceModels)
+  {
+    if (isEmptyOrModelFrame(
+            ifaceModel.first->IncludePoseRelativeTo().value_or("")))
+    {
+      ifaceModel.first->SetIncludePoseRelativeTo(proxyModelFrameName);
+    }
+  }
+
+  return proxyFrame;
+}
