@@ -24,6 +24,7 @@
 
 #include "ParamPassing.hh"
 #include "parser_private.hh"
+#include "Utils.hh"
 #include "XmlUtils.hh"
 
 namespace sdf
@@ -51,7 +52,7 @@ void updateParams(const ParserConfig &_config,
       _errors.push_back({ErrorCode::ATTRIBUTE_MISSING,
         "Element identifier requires an element_id attribute, but the "
         "element_id is not set. Skipping element alteration:\n"
-        + ElementToString(childElemXml)
+        + ElementToString(_errors, childElemXml)
       });
       continue;
     }
@@ -66,7 +67,7 @@ void updateParams(const ParserConfig &_config,
       _errors.push_back({ErrorCode::ATTRIBUTE_INVALID,
         "Missing name after double colons in element identifier. "
         "Skipping element alteration:\n"
-        + ElementToString(childElemXml)
+        + ElementToString(_errors, childElemXml)
       });
       continue;
     }
@@ -82,7 +83,7 @@ void updateParams(const ParserConfig &_config,
       {
         _errors.push_back({ErrorCode::ATTRIBUTE_INVALID,
           "Action [" + actionStr + "] is not a valid action. Skipping "
-          "element alteration:\n" + ElementToString(childElemXml)
+          "element alteration:\n" + ElementToString(_errors, childElemXml)
         });
         continue;
       }
@@ -101,7 +102,7 @@ void updateParams(const ParserConfig &_config,
         _errors.push_back({ErrorCode::ATTRIBUTE_MISSING,
           "Element to be added is missing a 'name' attribute. "
           "Skipping element addition:\n"
-          + ElementToString(childElemXml)
+          + ElementToString(_errors, childElemXml)
         });
         continue;
       }
@@ -111,7 +112,7 @@ void updateParams(const ParserConfig &_config,
       {
         _errors.push_back({ErrorCode::ATTRIBUTE_INVALID,
           "The 'name' attribute can not be empty. Skipping element addition:\n"
-          + ElementToString(childElemXml)
+          + ElementToString(_errors, childElemXml)
         });
         continue;
       }
@@ -128,7 +129,7 @@ void updateParams(const ParserConfig &_config,
           + " element_id='" + childElemXml->Attribute("element_id")
           + "'> because element already exists in included model. "
           + "Skipping element addition:\n"
-          + ElementToString(childElemXml)
+          + ElementToString(_errors, childElemXml)
         });
         continue;
       }
@@ -156,7 +157,8 @@ void updateParams(const ParserConfig &_config,
       _errors.push_back({ErrorCode::ELEMENT_MISSING,
         "Could not find element <" + std::string(childElemXml->Name())
         + " element_id='" + childElemXml->Attribute("element_id") + "'>. " +
-        "Skipping element modification:\n" + ElementToString(childElemXml)
+        "Skipping element modification:\n" +
+        ElementToString(_errors, childElemXml)
       });
       continue;
     }
@@ -176,11 +178,11 @@ void updateParams(const ParserConfig &_config,
     }
     else if (actionStr == "modify")
     {
-      modify(childElemXml, elem, _errors);
+      modify(childElemXml, _config, elem, _errors);
     }
     else if (actionStr == "remove")
     {
-      remove(childElemXml, elem, _errors);
+      remove(childElemXml, _config, elem, _errors);
     }
     else if (actionStr == "replace")
     {
@@ -193,7 +195,7 @@ void updateParams(const ParserConfig &_config,
       {
         _errors.push_back({ErrorCode::ELEMENT_INVALID,
           "Unable to convert XML to SDF. Skipping element replacement:\n"
-          + ElementToString(childElemXml)
+          + ElementToString(_errors, childElemXml)
         });
         continue;
       }
@@ -292,6 +294,8 @@ bool isValidAction(const std::string &_action)
 //////////////////////////////////////////////////
 ElementPtr getElementByName(const ElementPtr _elem,
                             const tinyxml2::XMLElement *_xml,
+                            const sdf::ParserConfig _config,
+                            sdf::Errors &_errors,
                             const bool _isModifyAction)
 {
   std::string elemName = _xml->Name();
@@ -324,10 +328,14 @@ ElementPtr getElementByName(const ElementPtr _elem,
   else if (elem->HasAttribute("name")
             && elem->GetAttribute("name")->GetRequired())
   {
-    sdfwarn << "The original element [" << elemName << "] contains the "
-            << "attribute 'name' but none was provided in the element modifier."
-            << " The assumed element to be modified is: <" << elemName
-            << " name='" << elem->Get<std::string>("name") << "'>\n";
+    std::stringstream ss;
+    ss << "The original element [" << elemName << "] contains the "
+       << "attribute 'name' but none was provided in the element modifier."
+       << " The assumed element to be modified is: <" << elemName
+       << " name='" << elem->Get<std::string>("name") << "'>";
+    Error err(ErrorCode::WARNING, ss.str());
+    enforceConfigurablePolicyCondition(
+        _config.WarningsPolicy(), err, _errors);
   }
 
   return elem;
@@ -348,7 +356,7 @@ ElementPtr initElementDescription(const tinyxml2::XMLElement *_xml,
     _errors.push_back({ErrorCode::ELEMENT_INVALID,
       "Element [" + std::string(_xml->Name()) + "] is not a defined "
       "SDF element. Skipping element alteration\n: "
-      + ElementToString(_xml)
+      + ElementToString(_errors, _xml)
     });
     return nullptr;
   }
@@ -377,7 +385,7 @@ void handleIndividualChildActions(const ParserConfig &_config,
         "Missing an action attribute. Skipping child element modification "
         "with parent <" + std::string(_childrenXml->Name()) + " element_id='"
         + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-        + ElementToString(xmlChild)
+        + ElementToString(_errors, xmlChild)
       });
       continue;
     }
@@ -390,45 +398,45 @@ void handleIndividualChildActions(const ParserConfig &_config,
         "child element modification with parent <"
         + std::string(_childrenXml->Name()) + " element_id='"
         + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-        + ElementToString(xmlChild)
+        + ElementToString(_errors, xmlChild)
       });
       continue;
     }
 
     if (actionStr == "remove")
     {
-      ElementPtr e = getElementByName(_elem, xmlChild);
+      ElementPtr e = getElementByName(_elem, xmlChild, _config, _errors);
       if (e == nullptr)
       {
         _errors.push_back({ErrorCode::ELEMENT_MISSING,
           "Could not find element. Skipping child element removal "
           "with parent <" + std::string(_childrenXml->Name()) + " element_id='"
           + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-          + ElementToString(xmlChild)
+          + ElementToString(_errors, xmlChild)
         });
       }
       else
       {
-        remove(xmlChild, e, _errors);
+        remove(xmlChild, _config, e, _errors);
       }
 
       continue;
     }
     else if (actionStr == "modify")
     {
-      ElementPtr e = getElementByName(_elem, xmlChild, true);
+      ElementPtr e = getElementByName(_elem, xmlChild, _config, _errors, true);
       if (e == nullptr)
       {
         _errors.push_back({ErrorCode::ELEMENT_MISSING,
           "Could not find element. Skipping child element modification "
           "with parent <" + std::string(_childrenXml->Name()) + " element_id='"
           + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-          + ElementToString(xmlChild)
+          + ElementToString(_errors, xmlChild)
         });
       }
       else
       {
-        modify(xmlChild, e, _errors);
+        modify(xmlChild, _config, e, _errors);
       }
 
       continue;
@@ -448,7 +456,7 @@ void handleIndividualChildActions(const ParserConfig &_config,
         "child element modification with parent <"
         + std::string(_childrenXml->Name()) + " element_id='"
         + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-        + ElementToString(xmlChild)
+        + ElementToString(_errors, xmlChild)
       });
       continue;
     }
@@ -461,7 +469,7 @@ void handleIndividualChildActions(const ParserConfig &_config,
         "Unable to convert XML to SDF. Skipping child element alteration "
         "with parent <" + std::string(_childrenXml->Name()) + " element_id='"
         + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-        + ElementToString(xmlChild)
+        + ElementToString(_errors, xmlChild)
       });
       continue;
     }
@@ -472,14 +480,14 @@ void handleIndividualChildActions(const ParserConfig &_config,
     }
     else if (actionStr == "replace")
     {
-      ElementPtr e = getElementByName(_elem, xmlChild);
+      ElementPtr e = getElementByName(_elem, xmlChild, _config, _errors);
       if (e == nullptr)
       {
         _errors.push_back({ErrorCode::ELEMENT_MISSING,
           "Could not find element. Skipping child element replacement "
           "with parent <" + std::string(_childrenXml->Name()) + " element_id='"
           + std::string(_childrenXml->Attribute("element_id")) + "'>:\n"
-          + ElementToString(xmlChild)
+          + ElementToString(_errors, xmlChild)
         });
         continue;
       }
@@ -491,7 +499,7 @@ void handleIndividualChildActions(const ParserConfig &_config,
           "Replacement element is missing a 'name' attribute. "
           "Skipping element replacement <" + std::string(_childrenXml->Name())
           + " element_id='" + std::string(_childrenXml->Attribute("element_id"))
-          + "'>:\n" + ElementToString(xmlChild)
+          + "'>:\n" + ElementToString(_errors, xmlChild)
         });
         continue;
       }
@@ -518,7 +526,7 @@ void add(const ParserConfig &_config, const std::string &_source,
   {
     _errors.push_back({ErrorCode::ELEMENT_INVALID,
       "Unable to convert XML to SDF. Skipping element addition:\n"
-      + ElementToString(_childXml)
+      + ElementToString(_errors, _childXml)
     });
   }
 }
@@ -550,7 +558,8 @@ void modifyAttributes(tinyxml2::XMLElement *_xml,
       {
         _errors.push_back({ErrorCode::ATTRIBUTE_INVALID,
           "Attribute [" + attrName + "] is invalid. "
-          "Skipping attribute modification in:\n" + ElementToString(_xml)
+          "Skipping attribute modification in:\n" +
+          ElementToString(_errors, _xml)
         });
         continue;
       }
@@ -562,7 +571,8 @@ void modifyAttributes(tinyxml2::XMLElement *_xml,
 
 //////////////////////////////////////////////////
 void modifyChildren(tinyxml2::XMLElement *_xml,
-                    ElementPtr _elem, Errors &_errors)
+                    const sdf::ParserConfig &_config, ElementPtr _elem,
+                    Errors &_errors)
 {
   for (tinyxml2::XMLElement *xmlChild = _xml->FirstChildElement();
        xmlChild;
@@ -574,12 +584,13 @@ void modifyChildren(tinyxml2::XMLElement *_xml,
     {
       _errors.push_back({ErrorCode::ELEMENT_MISSING,
         "Could not find element [" + elemName + "]. "
-        "Skipping modification for:\n" + ElementToString(_xml)
+        "Skipping modification for:\n" + ElementToString(_errors, _xml)
       });
       continue;
     }
 
-    ElementPtr elemChild = getElementByName(_elem, xmlChild, true);
+    ElementPtr elemChild = getElementByName(_elem, xmlChild,
+                                            _config, _errors, true);
     ParamPtr paramChild = elemChild->GetValue();
 
     if (xmlChild->GetText())
@@ -590,7 +601,7 @@ void modifyChildren(tinyxml2::XMLElement *_xml,
         _errors.push_back({ErrorCode::ELEMENT_INVALID,
           "Value [" + std::string(xmlChild->GetText()) + "] for element ["
           + elemName + "] is invalid. Skipping modification for:\n"
-          + ElementToString(_xml)
+          + ElementToString(_errors, _xml)
         });
         continue;
       }
@@ -609,21 +620,26 @@ void modifyChildren(tinyxml2::XMLElement *_xml,
       else
       {
         // sdf has child elements but no children were specified in xml
-        sdfwarn << "No modifications for element "
-                << ElementToString(xmlChild)
-                << " provided, skipping modification for:\n"
-                << ElementToString(_xml) << "\n";
+        std::stringstream ss;
+        ss << "No modifications for element "
+           << ElementToString(_errors, xmlChild)
+           << " provided, skipping modification for:\n"
+           << ElementToString(_errors, _xml);
+        Error err(ErrorCode::WARNING, ss.str());
+        enforceConfigurablePolicyCondition(
+            _config.WarningsPolicy(), err, _errors);
       }
     }
     else
     {
-      modify(xmlChild, elemChild, _errors);
+      modify(xmlChild, _config, elemChild, _errors);
     }
   }
 }
 
 //////////////////////////////////////////////////
-void modify(tinyxml2::XMLElement *_xml, ElementPtr _elem, Errors &_errors)
+void modify(tinyxml2::XMLElement *_xml,  const sdf::ParserConfig &_config,
+            ElementPtr _elem, Errors &_errors)
 {
   modifyAttributes(_xml, _elem, _errors);
 
@@ -636,19 +652,20 @@ void modify(tinyxml2::XMLElement *_xml, ElementPtr _elem, Errors &_errors)
       _errors.push_back({ErrorCode::ELEMENT_INVALID,
         "Value [" + std::string(_xml->GetText()) + "] for element [" +
         std::string(_xml->Name()) + "] is invalid. Skipping modification for:\n"
-        + ElementToString(_xml)
+        + ElementToString(_errors, _xml)
       });
     }
   }
   else
   {
     // modify children elements
-    modifyChildren(_xml, _elem, _errors);
+    modifyChildren(_xml, _config, _elem, _errors);
   }
 }
 
 //////////////////////////////////////////////////
-void remove(const tinyxml2::XMLElement *_xml, ElementPtr _elem, Errors &_errors)
+void remove(const tinyxml2::XMLElement *_xml, const sdf::ParserConfig &_config,
+            ElementPtr _elem, Errors &_errors)
 {
   if (_xml->NoChildren())
   {
@@ -663,7 +680,7 @@ void remove(const tinyxml2::XMLElement *_xml, ElementPtr _elem, Errors &_errors)
          xmlChild;
          xmlChild = xmlChild->NextSiblingElement())
     {
-      elemChild = getElementByName(_elem, xmlChild);
+      elemChild = getElementByName(_elem, xmlChild, _config, _errors);
       if (elemChild == nullptr)
       {
         const tinyxml2::XMLElement *xmlParent = _xml->Parent()->ToElement();
@@ -673,7 +690,7 @@ void remove(const tinyxml2::XMLElement *_xml, ElementPtr _elem, Errors &_errors)
           + std::string(xmlParent->Name()) + " element_id='"
           + std::string(xmlParent->Attribute("element_id")) + "'> with parent <"
           + std::string(_xml->Name()) + ">:\n"
-          + ElementToString(xmlChild)
+          + ElementToString(_errors, xmlChild)
         });
         continue;
       }
