@@ -46,45 +46,45 @@ TEST(Param, Bool)
 {
   sdf::Param boolParam("key", "bool", "true", false, "description");
   bool value = true;
-  boolParam.Get<bool>(value);
+  EXPECT_TRUE(boolParam.Get<bool>(value));
   EXPECT_TRUE(value);
 
-  boolParam.Set(false);
-  boolParam.Get<bool>(value);
+  EXPECT_TRUE(boolParam.Set(false));
+  EXPECT_TRUE(boolParam.Get<bool>(value));
   EXPECT_FALSE(value);
 
   // String parameter that represents a boolean.
   sdf::Param strParam("key", "string", "true", false, "description");
 
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_TRUE(value);
 
-  strParam.Set("false");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("false"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_FALSE(value);
 
-  strParam.Set("1");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("1"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_TRUE(value);
 
-  strParam.Set("0");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("0"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_FALSE(value);
 
-  strParam.Set("True");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("True"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_TRUE(value);
 
-  strParam.Set("TRUE");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("TRUE"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_TRUE(value);
 
   // Anything other than 1 or true is treated as a false value
-  strParam.Set("%");
-  strParam.Get<bool>(value);
+  EXPECT_TRUE(strParam.Set("%"));
+  EXPECT_TRUE(strParam.Get<bool>(value));
   EXPECT_FALSE(value);
 
-  boolParam.Set(true);
+  EXPECT_TRUE(boolParam.Set(true));
   std::any anyValue;
   EXPECT_TRUE(boolParam.GetAny(anyValue));
   try
@@ -98,6 +98,89 @@ TEST(Param, Bool)
   EXPECT_TRUE(value);
 }
 
+////////////////////////////////////////////////////
+/// Test getting and setting a floating point value
+TEST(Param, Double)
+{
+  sdf::Param doubleParam("key", "double", "0.0", false, "description");
+  double value = -1.0;
+  EXPECT_TRUE(doubleParam.Get<double>(value));
+  EXPECT_DOUBLE_EQ(0.0, value);
+
+  doubleParam.Set(1.0);
+  EXPECT_TRUE(doubleParam.Get<double>(value));
+  EXPECT_DOUBLE_EQ(1.0, value);
+
+  const double sixteenDigits = 12345678.87654321;
+  doubleParam.Set(sixteenDigits);
+  EXPECT_TRUE(doubleParam.Get<double>(value));
+  EXPECT_DOUBLE_EQ(sixteenDigits, value);
+}
+
+////////////////////////////////////////////////////
+/// Test getting and setting a Vector3d
+TEST(Param, Vector3d)
+{
+  sdf::Param vectorParam("key", "vector3", "0 0 0", false, "description");
+  gz::math::Vector3d value{1.0, 2.0, 3.0};
+  EXPECT_TRUE(vectorParam.Get<gz::math::Vector3d>(value));
+  EXPECT_DOUBLE_EQ(0.0, value.X());
+  EXPECT_DOUBLE_EQ(0.0, value.Y());
+  EXPECT_DOUBLE_EQ(0.0, value.Z());
+
+  vectorParam.Set(gz::math::Vector3d::One);
+  EXPECT_TRUE(vectorParam.Get<gz::math::Vector3d>(value));
+  EXPECT_DOUBLE_EQ(1.0, value.X());
+  EXPECT_DOUBLE_EQ(1.0, value.Y());
+  EXPECT_DOUBLE_EQ(1.0, value.Z());
+
+  const double fifteenDigits = 0.123456789012345;
+  const gz::math::Vector3d manyDigits{
+      fifteenDigits, 1.0 + fifteenDigits, 2.0 + fifteenDigits};
+  vectorParam.Set(manyDigits);
+  EXPECT_TRUE(vectorParam.Get<gz::math::Vector3d>(value));
+  EXPECT_DOUBLE_EQ(0.0 + fifteenDigits, value.X());
+  EXPECT_DOUBLE_EQ(1.0 + fifteenDigits, value.Y());
+  EXPECT_DOUBLE_EQ(2.0 + fifteenDigits, value.Z());
+}
+
+////////////////////////////////////////////////////
+/// Test getting and setting a Pose3d
+TEST(Param, Pose3d)
+{
+  sdf::Param poseParam("key", "pose", "0 0 0 0 0 0", false, "description");
+  gz::math::Pose3d value{1.0, 2.0, 3.0, 0.1, 0.2, 0.3};
+  EXPECT_TRUE(poseParam.Get<gz::math::Pose3d>(value));
+  EXPECT_DOUBLE_EQ(0.0, value.Pos().X());
+  EXPECT_DOUBLE_EQ(0.0, value.Pos().Y());
+  EXPECT_DOUBLE_EQ(0.0, value.Pos().Z());
+  EXPECT_DOUBLE_EQ(0.0, value.Rot().Euler().X());
+  EXPECT_DOUBLE_EQ(0.0, value.Rot().Euler().Y());
+  EXPECT_DOUBLE_EQ(0.0, value.Rot().Euler().Z());
+
+  poseParam.Set(gz::math::Pose3d(1, 2, 3, 0.1, 0.2, 0.3));
+  EXPECT_TRUE(poseParam.Get<gz::math::Pose3d>(value));
+  EXPECT_DOUBLE_EQ(1.0, value.Pos().X());
+  EXPECT_DOUBLE_EQ(2.0, value.Pos().Y());
+  EXPECT_DOUBLE_EQ(3.0, value.Pos().Z());
+  EXPECT_DOUBLE_EQ(0.1, value.Rot().Euler().X());
+  EXPECT_DOUBLE_EQ(0.2, value.Rot().Euler().Y());
+  EXPECT_DOUBLE_EQ(0.3, value.Rot().Euler().Z());
+
+  const double fifteenDigits = 0.123456789012345;
+  const gz::math::Pose3d manyDigits{
+      fifteenDigits, 1.0 + fifteenDigits, 2.0 + fifteenDigits,
+      0.5 * fifteenDigits, fifteenDigits, 2.0 * fifteenDigits};
+  poseParam.Set(manyDigits);
+  EXPECT_TRUE(poseParam.Get<gz::math::Pose3d>(value));
+  EXPECT_DOUBLE_EQ(0.0 + fifteenDigits, value.Pos().X());
+  EXPECT_DOUBLE_EQ(1.0 + fifteenDigits, value.Pos().Y());
+  EXPECT_DOUBLE_EQ(2.0 + fifteenDigits, value.Pos().Z());
+  EXPECT_DOUBLE_EQ(0.5 * fifteenDigits, value.Rot().Euler().X());
+  EXPECT_DOUBLE_EQ(1.0 * fifteenDigits, value.Rot().Euler().Y());
+  EXPECT_DOUBLE_EQ(2.0 * fifteenDigits, value.Rot().Euler().Z());
+}
+
 ////////////////////////////////////////////////////
 /// Test decimal number
 TEST(SetFromString, Decimals)
@@ -715,7 +798,7 @@ TEST(Param, ReparsingAfterSetFromStringPose)
   // After reparse, rotation values will be parsed as degrees.
   EXPECT_TRUE(poseParam.Reparse());
   EXPECT_TRUE(poseParam.Get<Pose>(value));
-  EXPECT_EQ(Pose(2, 3, 4, IGN_DTOR(0.5), IGN_DTOR(0.6), IGN_DTOR(0.7)), value);
+  EXPECT_EQ(Pose(2, 3, 4, GZ_DTOR(0.5), GZ_DTOR(0.6), GZ_DTOR(0.7)), value);
 
   // Changing parent @rotation_format to euler_rpy, value remains the same
   sdf::ParamPtr rotationFormatAttrib =
@@ -724,21 +807,21 @@ TEST(Param, ReparsingAfterSetFromStringPose)
   ASSERT_TRUE(rotationFormatAttrib->Set<std::string>("euler_rpy"));
   EXPECT_TRUE(poseParam.Reparse());
   EXPECT_TRUE(poseParam.Get<Pose>(value));
-  EXPECT_EQ(Pose(2, 3, 4, IGN_DTOR(0.5), IGN_DTOR(0.6), IGN_DTOR(0.7)), value);
+  EXPECT_EQ(Pose(2, 3, 4, GZ_DTOR(0.5), GZ_DTOR(0.6), GZ_DTOR(0.7)), value);
 
   // Changing parent @rotation_format to quat_xyzw, reparse will fail, value
   // remains the same as before
   ASSERT_TRUE(rotationFormatAttrib->Set<std::string>("quat_xyzw"));
   EXPECT_FALSE(poseParam.Reparse());
   EXPECT_TRUE(poseParam.Get<Pose>(value));
-  EXPECT_EQ(Pose(2, 3, 4, IGN_DTOR(0.5), IGN_DTOR(0.6), IGN_DTOR(0.7)), value);
+  EXPECT_EQ(Pose(2, 3, 4, GZ_DTOR(0.5), GZ_DTOR(0.6), GZ_DTOR(0.7)), value);
 
   // Changing parent @rotation_format to something invalid, reparse will fail,
   // value remains the same as before
   ASSERT_TRUE(rotationFormatAttrib->Set<std::string>("invalid_format"));
   EXPECT_FALSE(poseParam.Reparse());
   EXPECT_TRUE(poseParam.Get<Pose>(value));
-  EXPECT_EQ(Pose(2, 3, 4, IGN_DTOR(0.5), IGN_DTOR(0.6), IGN_DTOR(0.7)), value);
+  EXPECT_EQ(Pose(2, 3, 4, GZ_DTOR(0.5), GZ_DTOR(0.6), GZ_DTOR(0.7)), value);
 }
 
 /////////////////////////////////////////////////
@@ -900,11 +983,3 @@ TEST(Param, PoseWithDefaultValue)
   EXPECT_EQ(Pose(1, 0, 0, 0, 0, 0), poseElemClone->Get<Pose>());
   EXPECT_STREQ(expectedString.c_str(), poseElemClone->ToString("").c_str());
 }
-
-/////////////////////////////////////////////////
-/// Main
-int main(int argc, char **argv)
-{
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
