@@ -14,9 +14,11 @@
  * limitations under the License.
  *
 */
+#include <limits>
 
 #include "sdf/PrintConfig.hh"
 #include "sdf/Console.hh"
+#include "Utils.hh"
 
 using namespace sdf;
 
@@ -36,6 +38,9 @@ class PrintConfig::Implementation
 
   /// \brief True to preserve <include> tags, false to expand.
   public: bool preserveIncludes = false;
+
+  /// \brief The output stream's precision. By default, uses maximum precision.
+  public: int outPrecision = std::numeric_limits<int>::max();
 };
 
 /////////////////////////////////////////////////
@@ -71,19 +76,36 @@ bool PrintConfig::PreserveIncludes() const
 /////////////////////////////////////////////////
 bool PrintConfig::SetRotationSnapToDegrees(unsigned int _interval,
                                            double _tolerance)
+{
+  sdf::Errors errors;
+  bool result = this->SetRotationSnapToDegrees(_interval,
+                                               _tolerance,
+                                               errors);
+  throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+bool PrintConfig::SetRotationSnapToDegrees(unsigned int _interval,
+                                           double _tolerance,
+                                           sdf::Errors &_errors)
 {
   if (_interval == 0 || _interval > 360)
   {
-    sdferr << "Interval value to snap to must be larger than 0, and less than "
-           << "or equal to 360.\n";
+    std::stringstream ss;
+    ss << "Interval value to snap to must be larger than 0, and less than "
+       << "or equal to 360.";
+    _errors.push_back({ErrorCode::ROTATION_SNAP_CONFIG_ERROR, ss.str()});
     return false;
   }
 
   if (_tolerance <= 0 || _tolerance > 360 ||
       _tolerance >= static_cast<double>(_interval))
   {
-    sdferr << "Tolerance must be larger than 0, less than or equal to "
-           << "360, and less than the provided interval.\n";
+    std::stringstream ss;
+    ss << "Tolerance must be larger than 0, less than or equal to "
+       << "360, and less than the provided interval.";
+    _errors.push_back({ErrorCode::ROTATION_SNAP_CONFIG_ERROR, ss.str()});
     return false;
   }
 
@@ -104,13 +126,26 @@ std::optional<double> PrintConfig::RotationSnapTolerance() const
   return this->dataPtr->rotationSnapTolerance;
 }
 
+/////////////////////////////////////////////////
+void PrintConfig::SetOutPrecision(int _precision)
+{
+  this->dataPtr->outPrecision = _precision;
+}
+
+/////////////////////////////////////////////////
+int PrintConfig::OutPrecision() const
+{
+  return this->dataPtr->outPrecision;
+}
+
 /////////////////////////////////////////////////
 bool PrintConfig::operator==(const PrintConfig &_config) const
 {
   if (this->RotationInDegrees() == _config.RotationInDegrees() &&
       this->RotationSnapToDegrees() == _config.RotationSnapToDegrees() &&
       this->RotationSnapTolerance() == _config.RotationSnapTolerance() &&
-      this->PreserveIncludes() == _config.PreserveIncludes())
+      this->PreserveIncludes() == _config.PreserveIncludes() &&
+      this->OutPrecision() == _config.OutPrecision())
   {
     return true;
   }
