@@ -16,8 +16,16 @@
 */
 
 #include <gtest/gtest.h>
+
 #include <any>
+#include <filesystem>
+
 #include <gz/math.hh>
+#include <gz/utils/Environment.hh>
+#include <gz/utils/SuppressWarning.hh>
+
+#include "test_config.hh"
+#include "test_utils.hh"
 
 #include "sdf/sdf.hh"
 
@@ -272,6 +280,26 @@ TEST(SDF, ElementRemoveChild)
   EXPECT_FALSE(elem);
 }
 
+////////////////////////////////////////////////////
+TEST(SDF, SetRoot)
+{
+  sdf::SDF s;
+
+  sdf::ElementPtr elem;
+  elem.reset(new sdf::Element());
+  elem->AddValue("bool", "true", false, "description");
+  s.SetRoot(elem);
+  EXPECT_EQ(elem, s.Root());
+
+  // Test deprecated setter, remove in libsdformat14
+  s.SetRoot(nullptr);
+  EXPECT_EQ(nullptr, s.Root());
+  GZ_UTILS_WARN_IGNORE__DEPRECATED_DECLARATION
+  s.Root(elem);
+  GZ_UTILS_WARN_RESUME__DEPRECATED_DECLARATION
+  EXPECT_EQ(elem, s.Root());
+}
+
 ////////////////////////////////////////////////////
 /// Ensure that getting empty values with empty keys returns correct values.
 TEST(SDF, EmptyValues)
@@ -281,56 +309,56 @@ TEST(SDF, EmptyValues)
 
   elem.reset(new sdf::Element());
   EXPECT_FALSE(elem->Get<bool>(emptyString));
-  elem->AddValue("bool", "true", "0", "description");
+  elem->AddValue("bool", "true", false, "description");
   EXPECT_TRUE(elem->Get<bool>(emptyString));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<int>(emptyString), 0);
-  elem->AddValue("int", "12", "0", "description");
+  elem->AddValue("int", "12", false, "description");
   EXPECT_EQ(elem->Get<int>(emptyString), 12);
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<unsigned int>(emptyString),
       static_cast<unsigned int>(0));
-  elem->AddValue("unsigned int", "123", "0", "description");
+  elem->AddValue("unsigned int", "123", false, "description");
   EXPECT_EQ(elem->Get<unsigned int>(emptyString),
       static_cast<unsigned int>(123));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<char>(emptyString), '\0');
-  elem->AddValue("char", "a", "0", "description");
+  elem->AddValue("char", "a", false, "description");
   EXPECT_EQ(elem->Get<char>(emptyString), 'a');
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<std::string>(emptyString), "");
-  elem->AddValue("string", "hello", "0", "description");
+  elem->AddValue("string", "hello", false, "description");
   EXPECT_EQ(elem->Get<std::string>(emptyString), "hello");
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<gz::math::Vector2d>(emptyString),
       gz::math::Vector2d());
-  elem->AddValue("vector2d", "1 2", "0", "description");
+  elem->AddValue("vector2d", "1 2", false, "description");
   EXPECT_EQ(elem->Get<gz::math::Vector2d>(emptyString),
       gz::math::Vector2d(1, 2));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<gz::math::Vector3d>(emptyString),
       gz::math::Vector3d());
-  elem->AddValue("vector3", "1 2 3", "0", "description");
+  elem->AddValue("vector3", "1 2 3", false, "description");
   EXPECT_EQ(elem->Get<gz::math::Vector3d>(emptyString),
       gz::math::Vector3d(1, 2, 3));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<gz::math::Quaterniond>(emptyString),
             gz::math::Quaterniond());
-  elem->AddValue("quaternion", "1 2 3", "0", "description");
+  elem->AddValue("quaternion", "1 2 3", false, "description");
   EXPECT_EQ(elem->Get<gz::math::Quaterniond>(emptyString),
             gz::math::Quaterniond(-2.14159, 1.14159, -0.141593));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<gz::math::Pose3d>(emptyString),
       gz::math::Pose3d());
-  elem->AddValue("pose", "1.0 2.0 3.0 4.0 5.0 6.0", "0", "description");
+  elem->AddValue("pose", "1.0 2.0 3.0 4.0 5.0 6.0", false, "description");
   EXPECT_EQ(elem->Get<gz::math::Pose3d>(emptyString).Pos(),
       gz::math::Pose3d(1, 2, 3, 4, 5, 6).Pos());
   EXPECT_EQ(elem->Get<gz::math::Pose3d>(emptyString).Rot().Euler(),
@@ -339,23 +367,23 @@ TEST(SDF, EmptyValues)
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<gz::math::Color>(emptyString),
       gz::math::Color());
-  elem->AddValue("color", ".1 .2 .3 1.0", "0", "description");
+  elem->AddValue("color", ".1 .2 .3 1.0", false, "description");
   EXPECT_EQ(elem->Get<gz::math::Color>(emptyString),
             gz::math::Color(.1f, .2f, .3f, 1.0f));
 
   elem.reset(new sdf::Element());
   EXPECT_EQ(elem->Get<sdf::Time>(emptyString), sdf::Time());
-  elem->AddValue("time", "1 2", "0", "description");
+  elem->AddValue("time", "1 2", false, "description");
   EXPECT_EQ(elem->Get<sdf::Time>(emptyString), sdf::Time(1, 2));
 
   elem.reset(new sdf::Element());
   EXPECT_NEAR(elem->Get<float>(emptyString), 0.0, 1e-6);
-  elem->AddValue("float", "12.34", "0", "description");
+  elem->AddValue("float", "12.34", false, "description");
   EXPECT_NEAR(elem->Get<float>(emptyString), 12.34, 1e-6);
 
   elem.reset(new sdf::Element());
   EXPECT_NEAR(elem->Get<double>(emptyString), 0.0, 1e-6);
-  elem->AddValue("double", "12.34", "0", "description");
+  elem->AddValue("double", "12.34", false, "description");
   EXPECT_NEAR(elem->Get<double>(emptyString), 12.34, 1e-6);
 }
 
@@ -537,6 +565,59 @@ TEST(SDF, Version)
   EXPECT_STREQ(SDF_VERSION, sdf::SDF::Version().c_str());
 }
 
+/////////////////////////////////////////////////
+TEST(SDF, EmbeddedSpec)
+{
+  std::string result;
+
+  result = sdf::SDF::EmbeddedSpec("actor.sdf", false);
+  EXPECT_NE(result.find("<!-- Actor -->"), std::string::npos);
+  EXPECT_NE(result.find("<element name=\"actor\" required=\"*\">"),
+      std::string::npos);
+  result = sdf::SDF::EmbeddedSpec("actor.sdf", true);
+  EXPECT_NE(result.find("<!-- Actor -->"), std::string::npos);
+  EXPECT_NE(result.find("<element name=\"actor\" required=\"*\">"),
+      std::string::npos);
+
+  result = sdf::SDF::EmbeddedSpec("root.sdf", false);
+  EXPECT_NE(result.find("SDFormat base element"), std::string::npos);
+  EXPECT_NE(result.find("name=\"version\" type=\"string\""), std::string::npos);
+  result = sdf::SDF::EmbeddedSpec("root.sdf", true);
+  EXPECT_NE(result.find("SDFormat base element"), std::string::npos);
+  EXPECT_NE(result.find("name=\"version\" type=\"string\""), std::string::npos);
+}
+
+TEST(SDF, EmbeddedSpecNonExistent)
+{
+  std::string result;
+
+  // Capture sdferr output
+  std::stringstream stderr_buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &stderr_buffer);
+#ifdef _WIN32
+  sdf::Console::Instance()->SetQuiet(false);
+  sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+      sdf::Console::Instance()->SetQuiet(true);
+      });
+#endif
+
+  result = sdf::SDF::EmbeddedSpec("unavailable.sdf", false);
+  EXPECT_NE(stderr_buffer.str().find("Unable to find SDF filename"),
+      std::string::npos);
+  EXPECT_NE(stderr_buffer.str().find("with version"), std::string::npos);
+  EXPECT_TRUE(result.empty());
+
+  // clear the contents of the buffer
+  stderr_buffer.str("");
+
+  result = sdf::SDF::EmbeddedSpec("unavailable.sdf", true);
+  EXPECT_TRUE(stderr_buffer.str().empty());
+  EXPECT_TRUE(result.empty());
+}
+
 /////////////////////////////////////////////////
 TEST(SDF, FilePath)
 {
@@ -643,14 +724,9 @@ TEST(SDF, ToStringAndParse)
 bool create_new_temp_dir(std::string &_new_temp_path)
 {
   std::string tmppath;
-  const char *tmp = getenv("TMPDIR");
-  if (tmp)
+  if(!gz::utils::env("TMPDIR", tmppath))
   {
-    tmppath = std::string(tmp);
-  }
-  else
-  {
-    tmppath = std::string("/tmp");
+    tmppath = "/tmp";
   }
 
   tmppath += "/XXXXXX";
@@ -666,6 +742,46 @@ bool create_new_temp_dir(std::string &_new_temp_path)
   return true;
 }
 
+/////////////////////////////////////////////////
+TEST(SDF, ErrorOutput)
+{
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+  sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+  #ifdef _WIN32
+    sdf::Console::Instance()->SetQuiet(false);
+    sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+  #endif
+  sdf::Errors errors;
+
+  // Test findFile
+  EXPECT_EQ(sdf::findFile(errors, "adfjialkas31", false, true), "");
+  EXPECT_EQ(errors.size(), 1) << errors;
+  EXPECT_NE(std::string::npos,
+    errors[0].Message().find("Tried to use callback in sdf::findFile(), "
+                             "but the callback is empty.  Did you call "
+                             "sdf::setFindCallback()?"))
+      << errors[0].Message();
+  errors.clear();
+
+  sdf::SDF sdf;
+  sdf.SetFromString(errors, "banana");
+  EXPECT_EQ(errors.size(), 2) << errors;
+  EXPECT_NE(std::string::npos,
+    errors[0].Message().find("Error parsing XML from string"))
+      << errors[0].Message();
+  EXPECT_NE(std::string::npos,
+    errors[1].Message().find("Unable to parse sdf string[banana]"))
+      << errors[1].Message();
+  // Check nothing has been printed
+  EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+}
+
 /////////////////////////////////////////////////
 bool g_findFileCbCalled = false;
 std::string findFileCb(const std::string &)
@@ -715,12 +831,58 @@ TEST(SDF, WriteURIPath)
   ASSERT_EQ(std::remove(tempFile.c_str()), 0);
   ASSERT_EQ(rmdir(tempDir.c_str()), 0);
 }
-#endif  // _WIN32
-
 /////////////////////////////////////////////////
-/// Main
-int main(int argc, char **argv)
+TEST(SDF, FindFileModelSDFCurrDir)
 {
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
+  // Change to a temporary directory before running test
+  auto prevPath = std::filesystem::current_path();
+  std::string tmpDir;
+  ASSERT_TRUE(sdf::testing::TestTmpPath(tmpDir));
+  std::filesystem::current_path(tmpDir);
+
+  // A file named model.sdf in current directory
+  auto tempFile = tmpDir + "/model.sdf";
+
+  sdf::SDF tempSDF;
+  tempSDF.Write(tempFile);
+
+  // Check file was created
+  auto fp = fopen(tempFile.c_str(), "r");
+  ASSERT_NE(nullptr, fp);
+
+  // Get path of the file returned from findFile()
+  std::string foundFile = sdf::findFile("model.sdf", true, false);
+
+  // Check that the returned file is model.sdf from curr directory
+  ASSERT_EQ(tempFile, foundFile);
+
+  // Cleanup
+  ASSERT_EQ(std::remove(tempFile.c_str()), 0);
+  std::filesystem::current_path(prevPath);
 }
+
+///////////////////////////////////////////////////
+// Test the copy constructor provides a deep copy
+TEST(SDF, CopyConstructor)
+{
+  // Set up a simple sdf model file
+  std::ostringstream stream;
+  stream << "<sdf version='1.3'>"
+         << "  <model name='test_model'>"
+         << "    <pose>0 1 2  0 0 0</pose>"
+         << "    <static>false</static>"
+         << "  </model>"
+         << "</sdf>";
+  sdf::SDF sdfParsed;
+  sdfParsed.SetFromString(stream.str());
+
+  sdf::SDF copy1sdfParsed(sdfParsed);
+  sdf::SDF copy2sdfParsed = sdfParsed;
+
+  sdfParsed.Root()->SetName("Test");
+  ASSERT_EQ(sdfParsed.Root()->GetName(), "Test");
+  // Since a deep copy occurs no changes should be reflected;
+  ASSERT_NE(copy1sdfParsed.Root()->GetName(), "Test");
+  ASSERT_NE(copy2sdfParsed.Root()->GetName(), "Test");
+}
+#endif  // _WIN32
