@@ -55,6 +55,12 @@ Scene::Scene()
 
 /////////////////////////////////////////////////
 Errors Scene::Load(ElementPtr _sdf)
+{
+  return this->Load(_sdf, ParserConfig::GlobalConfig());
+}
+
+/////////////////////////////////////////////////
+Errors Scene::Load(ElementPtr _sdf, const ParserConfig &_config)
 {
   Errors errors;
 
@@ -71,30 +77,31 @@ Errors Scene::Load(ElementPtr _sdf)
   }
 
   // Get the ambient property
-  this->dataPtr->ambient = _sdf->Get<gz::math::Color>("ambient",
+  this->dataPtr->ambient = _sdf->Get<gz::math::Color>(errors, "ambient",
       this->dataPtr->ambient).first;
 
   // Get the background color property
-  this->dataPtr->background = _sdf->Get<gz::math::Color>("background",
+  this->dataPtr->background = _sdf->Get<gz::math::Color>(errors, "background",
       this->dataPtr->background).first;
 
   // Get the grid property
-  this->dataPtr->grid = _sdf->Get<bool>("grid",
+  this->dataPtr->grid = _sdf->Get<bool>(errors, "grid",
       this->dataPtr->grid).first;
 
   // Get the shadows property
-  this->dataPtr->shadows = _sdf->Get<bool>("shadows",
+  this->dataPtr->shadows = _sdf->Get<bool>(errors, "shadows",
       this->dataPtr->shadows).first;
 
   // Get the origin_visual property
-  this->dataPtr->originVisual = _sdf->Get<bool>("origin_visual",
+  this->dataPtr->originVisual = _sdf->Get<bool>(errors, "origin_visual",
       this->dataPtr->originVisual).first;
 
   // load sky
   if (_sdf->HasElement("sky"))
   {
     this->dataPtr->sky.emplace();
-    Errors err = this->dataPtr->sky->Load(_sdf->GetElement("sky"));
+    Errors err = this->dataPtr->sky->Load(_sdf->GetElement(
+                                            "sky", errors), _config);
     errors.insert(errors.end(), err.begin(), err.end());
   }
 
@@ -180,18 +187,28 @@ sdf::ElementPtr Scene::Element() const
 
 /////////////////////////////////////////////////
 sdf::ElementPtr Scene::ToElement() const
+{
+  sdf::Errors errors;
+  auto result = this->ToElement(errors);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Scene::ToElement(sdf::Errors &_errors) const
 {
   sdf::ElementPtr elem(new sdf::Element);
   sdf::initFile("scene.sdf", elem);
 
-  elem->GetElement("ambient")->Set(this->Ambient());
-  elem->GetElement("background")->Set(this->Background());
-  elem->GetElement("grid")->Set(this->Grid());
-  elem->GetElement("origin_visual")->Set(this->OriginVisual());
-  elem->GetElement("shadows")->Set(this->Shadows());
+  elem->GetElement("ambient", _errors)->Set(_errors, this->Ambient());
+  elem->GetElement("background", _errors)->Set(_errors, this->Background());
+  elem->GetElement("grid", _errors)->Set(_errors, this->Grid());
+  elem->GetElement("origin_visual", _errors)->Set(
+      _errors, this->OriginVisual());
+  elem->GetElement("shadows", _errors)->Set(_errors, this->Shadows());
 
   if (this->dataPtr->sky)
-    elem->InsertElement(this->dataPtr->sky->ToElement(), true);
+    elem->InsertElement(this->dataPtr->sky->ToElement(_errors), true);
 
   return elem;
 }
