@@ -14,12 +14,15 @@
  * limitations under the License.
  *
 */
+#include <array>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <optional>
 #include <vector>
 #include <gz/math/Pose3.hh>
 #include "sdf/AirPressure.hh"
+#include "sdf/AirSpeed.hh"
 #include "sdf/Altimeter.hh"
 #include "sdf/Camera.hh"
 #include "sdf/Error.hh"
@@ -38,8 +41,9 @@
 using namespace sdf;
 
 /// Sensor type strings. These should match the data in
-/// `enum class SensorType` located in Sensor.hh.
-const std::vector<std::string> sensorTypeStrs =
+/// `enum class SensorType` located in Sensor.hh, and the size template
+/// parameter should match the number of elements as well.
+constexpr std::array<const std::string_view, 27> kSensorTypeStrs =
 {
   "none",
   "altimeter",
@@ -66,7 +70,8 @@ const std::vector<std::string> sensorTypeStrs =
   "segmentation_camera",
   "boundingbox_camera",
   "custom",
-  "wide_angle_camera"
+  "wide_angle_camera",
+  "air_speed"
 };
 
 class sdf::Sensor::Implementation
@@ -110,6 +115,9 @@ class sdf::Sensor::Implementation
   /// \brief Optional air pressure sensor.
   public: std::optional<AirPressure> airPressure;
 
+  /// \brief Optional air pressure sensor.
+  public: std::optional<AirSpeed> airSpeed;
+
   /// \brief Optional camera.
   public: std::optional<Camera> camera;
 
@@ -164,6 +172,8 @@ bool Sensor::operator==(const Sensor &_sensor) const
       return *(this->dataPtr->magnetometer) == *(_sensor.dataPtr->magnetometer);
     case SensorType::AIR_PRESSURE:
       return *(this->dataPtr->airPressure) == *(_sensor.dataPtr->airPressure);
+    case SensorType::AIR_SPEED:
+      return *(this->dataPtr->airSpeed) == *(_sensor.dataPtr->airSpeed);
     case SensorType::FORCE_TORQUE:
       return *(this->dataPtr->forceTorque) == *(_sensor.dataPtr->forceTorque);
     case SensorType::IMU:
@@ -251,6 +261,14 @@ Errors Sensor::Load(ElementPtr _sdf)
         _sdf->GetElement("air_pressure"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
+  else if (type == "air_speed")
+  {
+    this->dataPtr->type = SensorType::AIR_SPEED;
+    this->dataPtr->airSpeed.emplace();
+    Errors err = this->dataPtr->airSpeed->Load(
+        _sdf->GetElement("air_speed"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
   else if (type == "altimeter")
   {
     this->dataPtr->type = SensorType::ALTIMETER;
@@ -513,9 +531,9 @@ void Sensor::SetType(const SensorType _type)
 /////////////////////////////////////////////////
 bool Sensor::SetType(const std::string &_typeStr)
 {
-  for (size_t i = 0; i < sensorTypeStrs.size(); ++i)
+  for (size_t i = 0; i < kSensorTypeStrs.size(); ++i)
   {
-    if (_typeStr == sensorTypeStrs[i])
+    if (_typeStr == kSensorTypeStrs[i])
     {
       this->dataPtr->type = static_cast<SensorType>(i);
       return true;
@@ -578,6 +596,24 @@ void Sensor::SetAirPressureSensor(const AirPressure &_air)
   this->dataPtr->airPressure = _air;
 }
 
+/////////////////////////////////////////////////
+const AirSpeed *Sensor::AirSpeedSensor() const
+{
+  return optionalToPointer(this->dataPtr->airSpeed);
+}
+
+/////////////////////////////////////////////////
+AirSpeed *Sensor::AirSpeedSensor()
+{
+  return optionalToPointer(this->dataPtr->airSpeed);
+}
+
+/////////////////////////////////////////////////
+void Sensor::SetAirSpeedSensor(const AirSpeed &_air)
+{
+  this->dataPtr->airSpeed = _air;
+}
+
 /////////////////////////////////////////////////
 const Lidar *Sensor::LidarSensor() const
 {
@@ -612,8 +648,8 @@ void Sensor::SetUpdateRate(double _hz)
 std::string Sensor::TypeStr() const
 {
   size_t index = static_cast<int>(this->dataPtr->type);
-  if (index > 0 && index < sensorTypeStrs.size())
-    return sensorTypeStrs[index];
+  if (index > 0 && index < kSensorTypeStrs.size())
+    return std::string(kSensorTypeStrs[index]);
   return "none";
 }
 
@@ -691,13 +727,22 @@ Imu *Sensor::ImuSensor()
 
 /////////////////////////////////////////////////
 sdf::ElementPtr Sensor::ToElement() const
+{
+  sdf::Errors errors;
+  auto result = this->ToElement(errors);
+  sdf::throwOrPrintErrors(errors);
+  return result;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Sensor::ToElement(sdf::Errors &_errors) const
 {
   sdf::ElementPtr elem(new sdf::Element);
   sdf::initFile("sensor.sdf", elem);
 
-  elem->GetAttribute("type")->Set<std::string>(this->TypeStr());
-  elem->GetAttribute("name")->Set<std::string>(this->Name());
-  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  elem->GetAttribute("type")->Set<std::string>(this->TypeStr(), _errors);
+  elem->GetAttribute("name")->Set<std::string>(this->Name(), _errors);
+  sdf::ElementPtr poseElem = elem->GetElement("pose", _errors);
   if (!this->dataPtr->poseRelativeTo.empty())
   {
     poseElem->GetAttribute("relative_to")->Set<std::string>(
@@ -716,6 +761,13 @@ sdf::ElementPtr Sensor::ToElement() const
     sdf::ElementPtr airPressureElem = elem->GetElement("air_pressure");
     airPressureElem->Copy(this->dataPtr->airPressure->ToElement());
   }
+  // air speed
+  else if (this->Type() == sdf::SensorType::AIR_SPEED &&
+      this->dataPtr->airSpeed)
+  {
+    sdf::ElementPtr airSpeedElem = elem->GetElement("air_speed");
+    airSpeedElem->Copy(this->dataPtr->airSpeed->ToElement());
+  }
   // altimeter
   else if (this->Type() == sdf::SensorType::ALTIMETER &&
       this->dataPtr->altimeter)
@@ -760,8 +812,10 @@ sdf::ElementPtr Sensor::ToElement() const
   }
   else
   {
-    std::cout << "Conversion of sensor type: [" << this->TypeStr() << "] from "
-      << "SDF DOM to Element is not supported yet." << std::endl;
+    std::stringstream ss;
+    ss << "Conversion of sensor type: [" << this->TypeStr() << "] from SDF "
+       << "DOM to Element is not supported yet." << this->Name();
+    _errors.push_back({ErrorCode::ELEMENT_INVALID, ss.str()});
   }
 
   // Add in the plugins
