@@ -18,6 +18,7 @@
 #include <unordered_set>
 #include <vector>
 #include <optional>
+#include <gz/math/SemanticVersion.hh>
 #include <gz/math/Vector3.hh>
 
 #include "sdf/Actor.hh"
@@ -25,6 +26,8 @@
 #include "sdf/InterfaceElements.hh"
 #include "sdf/InterfaceModel.hh"
 #include "sdf/InterfaceModelPoseGraph.hh"
+#include "sdf/Joint.hh"
+#include "sdf/JointAxis.hh"
 #include "sdf/Light.hh"
 #include "sdf/Model.hh"
 #include "sdf/ParserConfig.hh"
@@ -46,8 +49,8 @@ class sdf::World::Implementation
   /// \return Errors, if any.
   public: Errors LoadSphericalCoordinates(sdf::ElementPtr _elem);
 
-  /// \brief Optional atmosphere model.
-  public: std::optional<sdf::Atmosphere> atmosphere;
+  /// \brief Required atmosphere model.
+  public: sdf::Atmosphere atmosphere;
 
   /// \brief Audio device name
   public: std::string audioDevice = "default";
@@ -59,12 +62,15 @@ class sdf::World::Implementation
   /// \brief Optional Gui parameters.
   public: std::optional<sdf::Gui> gui;
 
-  /// \brief Optional Scene parameters.
-  public: std::optional<sdf::Scene> scene;
+  /// \brief Required scene parameters.
+  public: sdf::Scene scene;
 
   /// \brief The frames specified in this world.
   public: std::vector<Frame> frames;
 
+  /// \brief The joints specified in this world.
+  public: std::vector<Joint> joints;
+
   /// \brief The lights specified in this world.
   public: std::vector<Light> lights;
 
@@ -83,8 +89,8 @@ class sdf::World::Implementation
   public: std::vector<Model> models;
 
   /// \brief The interface models specified in this world.
-  public: std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
-      interfaceModels;
+  public: std::vector<std::pair<sdf::NestedInclude,
+          sdf::InterfaceModelConstPtr>> interfaceModels;
 
   /// \brief Name of the world.
   public: std::string name = "";
@@ -130,6 +136,7 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   Errors errors;
 
   this->dataPtr->sdf = _sdf;
+  gz::math::SemanticVersion sdfVersion(_sdf->OriginalVersion());
 
   // Check that the provided SDF element is a <world>
   // This is an error that cannot be recovered, so return an error.
@@ -176,9 +183,8 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   // Read the atmosphere element
   if (_sdf->HasElement("atmosphere"))
   {
-    this->dataPtr->atmosphere.emplace();
     Errors atmosphereLoadErrors =
-      this->dataPtr->atmosphere->Load(_sdf->GetElement("atmosphere"));
+      this->dataPtr->atmosphere.Load(_sdf->GetElement("atmosphere"));
     errors.insert(errors.end(), atmosphereLoadErrors.begin(),
         atmosphereLoadErrors.end());
   }
@@ -206,39 +212,37 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   {
     if (size > 1)
     {
-      sdfwarn << "Non-unique name[" << name << "] detected " << size
-              << " times in XML children of world with name[" << this->Name()
-              << "].\n";
+      std::stringstream ss;
+      ss << "Non-unique name[" << name << "] detected " << size
+         << " times in XML children of world with name[" << this->Name()
+         << "].";
+      Error err(ErrorCode::WARNING, ss.str());
+      enforceConfigurablePolicyCondition(
+          _config.WarningsPolicy(), err, errors);
     }
   }
 
-  for (const auto &[name, size] :
-       _sdf->CountNamedElements("", Element::NameUniquenessExceptions()))
+  std::unordered_set<std::string> modelNames;
+  std::unordered_set<std::string> jointNames;
+  std::unordered_set<std::string> explicitFrameNames;
+  auto recordUniqueName = [&errors](std::unordered_set<std::string>& nameList,
+                                      const std::string &_elementName,
+                                      const std::string &_name)
   {
-    if (size > 1)
+    if (nameList.count(_name) > 0)
     {
-      sdfwarn << "Non-unique name[" << name << "] detected " << size
-              << " times in XML children of world with name[" << this->Name()
-              << "].\n";
+      errors.emplace_back(ErrorCode::DUPLICATE_NAME,
+          _elementName + " with name[" + _name + "] already exists.");
+      return false;
     }
-  }
+    nameList.insert(_name);
+    return true;
+  };
 
-  // Set of implicit and explicit frame names in this model for tracking
-  // name collisions
-  std::unordered_set<std::string> frameNames;
-
-  // Load all the models.
-  Errors modelLoadErrors =
-      loadUniqueRepeated<Model>(_sdf, "model", this->dataPtr->models, _config);
-  errors.insert(errors.end(), modelLoadErrors.begin(), modelLoadErrors.end());
-
-  // Models are loaded first, and loadUniqueRepeated ensures there are no
-  // duplicate names, so these names can be added to frameNames without
-  // checking uniqueness.
-  for (const auto &model : this->dataPtr->models)
-  {
-    frameNames.insert(model.Name());
-  }
+  // Set of implicit and explicit frame names in this world for tracking
+  // name collisions. This is used to handle name clashes in old versions of
+  // SDFormat where sibling elements were allowed to have duplicate names.
+  std::unordered_set<std::string> implicitFrameNames;
 
   // Load included models via the interface API
   Errors interfaceModelLoadErrors = loadIncludedInterfaceModels(
@@ -248,7 +252,114 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
 
   for (const auto &ifaceModelPair : this->dataPtr->interfaceModels)
   {
-    frameNames.insert(ifaceModelPair.second->Name());
+    implicitFrameNames.insert(ifaceModelPair.second->Name());
+  }
+
+  for (auto elem = _sdf->GetFirstElement(); elem; elem = elem->GetNextElement())
+  {
+    const std::string elementName = elem->GetName();
+    if (elementName == "model")
+    {
+      auto model = loadSingle<Model>(errors, elem, _config);
+      if (!recordUniqueName(modelNames, elementName, model.Name()))
+      {
+        continue;
+      }
+      implicitFrameNames.insert(model.Name());
+      if (model.IsMerged())
+      {
+        sdf::Frame proxyFrame = model.PrepareForMerge(errors, "world");
+        this->AddFrame(proxyFrame);
+
+        for (uint64_t fi = 0; fi < model.FrameCount(); ++fi)
+        {
+          this->dataPtr->frames.push_back(std::move(*model.FrameByIndex(fi)));
+        }
+
+        for (uint64_t ji = 0; ji < model.JointCount(); ++ji)
+        {
+          this->dataPtr->joints.push_back(std::move(*model.JointByIndex(ji)));
+        }
+
+        for (uint64_t mi = 0; mi < model.ModelCount(); ++mi)
+        {
+          this->dataPtr->models.push_back(std::move(*model.ModelByIndex(mi)));
+        }
+
+        for (uint64_t imi = 0; imi < model.InterfaceModelCount(); ++imi)
+        {
+          InterfaceModelConstPtr ifaceModel = model.InterfaceModelByIndex(imi);
+          NestedInclude nestedInclude =
+              *model.InterfaceModelNestedIncludeByIndex(imi);
+          this->dataPtr->interfaceModels.emplace_back(std::move(nestedInclude),
+                                                      ifaceModel);
+        }
+
+        // TODO(azeey) Support Merge-included interface models when `World`
+        // supports them.
+      }
+      else
+      {
+        this->dataPtr->models.push_back(std::move(model));
+      }
+    }
+    else if (elementName == "joint")
+    {
+      auto joint = loadSingle<Joint>(errors, elem);
+      if (!recordUniqueName(jointNames, elementName, joint.Name()))
+      {
+        continue;
+      }
+      this->dataPtr->joints.push_back(std::move(joint));
+    }
+    else if (elementName == "frame")
+    {
+      auto frame = loadSingle<Frame>(errors, elem);
+      if (!recordUniqueName(explicitFrameNames, elementName, frame.Name()))
+      {
+        continue;
+      }
+      this->dataPtr->frames.push_back(std::move(frame));
+    }
+  }
+
+  // Check frames for name collisions and modify and warn if so.
+  for (auto &frame : this->dataPtr->frames)
+  {
+    std::string frameName = frame.Name();
+    if (implicitFrameNames.count(frameName) > 0)
+    {
+      // This frame has a name collision
+      if (sdfVersion < gz::math::SemanticVersion(1, 7))
+      {
+        // This came from an old file, so try to workaround by renaming frame
+        frameName += "_frame";
+        int i = 0;
+        while (implicitFrameNames.count(frameName) > 0)
+        {
+          frameName = frame.Name() + "_frame" + std::to_string(i++);
+        }
+        std::stringstream ss;
+        ss << "Frame with name [" << frame.Name() << "] "
+           << "in world with name [" << this->Name() << "] "
+           << "has a name collision, changing frame name to ["
+           << frameName << "].";
+        Error err(ErrorCode::WARNING, ss.str());
+        enforceConfigurablePolicyCondition(
+            _config.WarningsPolicy(), err, errors);
+
+        frame.SetName(frameName);
+      }
+      else
+      {
+        std::stringstream ss;
+        ss << "Frame with name [" << frame.Name() << "] "
+           << "in world with name [" << this->Name() << "] "
+           << "has a name collision. Please rename this frame.";
+        errors.push_back({ErrorCode::DUPLICATE_NAME, ss.str()});
+      }
+    }
+    implicitFrameNames.insert(frameName);
   }
 
   // Load all the physics.
@@ -271,31 +382,6 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
       this->dataPtr->lights);
   errors.insert(errors.end(), lightLoadErrors.begin(), lightLoadErrors.end());
 
-  // Load all the frames.
-  Errors frameLoadErrors = loadUniqueRepeated<Frame>(_sdf, "frame",
-      this->dataPtr->frames);
-  errors.insert(errors.end(), frameLoadErrors.begin(), frameLoadErrors.end());
-
-  // Check frames for name collisions and modify and warn if so.
-  for (auto &frame : this->dataPtr->frames)
-  {
-    std::string frameName = frame.Name();
-    if (frameNames.count(frameName) > 0)
-    {
-      frameName += "_frame";
-      int i = 0;
-      while (frameNames.count(frameName) > 0)
-      {
-        frameName = frame.Name() + "_frame" + std::to_string(i++);
-      }
-      sdfwarn << "Frame with name [" << frame.Name() << "] "
-              << "in world with name [" << this->Name() << "] "
-              << "has a name collision, changing frame name to ["
-              << frameName << "].\n";
-      frame.SetName(frameName);
-    }
-    frameNames.insert(frameName);
-  }
 
   // Load the Gui
   if (_sdf->HasElement("gui"))
@@ -308,9 +394,8 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   // Load the Scene
   if (_sdf->HasElement("scene"))
   {
-    this->dataPtr->scene.emplace();
     Errors sceneLoadErrors =
-        this->dataPtr->scene->Load(_sdf->GetElement("scene"));
+        this->dataPtr->scene.Load(_sdf->GetElement("scene"), _config);
     errors.insert(errors.end(), sceneLoadErrors.begin(), sceneLoadErrors.end());
   }
 
@@ -453,7 +538,7 @@ Model *World::ModelByName(const std::string &_name)
 /////////////////////////////////////////////////
 const sdf::Atmosphere *World::Atmosphere() const
 {
-  return optionalToPointer(this->dataPtr->atmosphere);
+  return &(this->dataPtr->atmosphere);
 }
 
 /////////////////////////////////////////////////
@@ -491,7 +576,7 @@ void World::SetGui(const sdf::Gui &_gui)
 /////////////////////////////////////////////////
 const sdf::Scene *World::Scene() const
 {
-  return optionalToPointer(this->dataPtr->scene);
+  return &(this->dataPtr->scene);
 }
 
 /////////////////////////////////////////////////
@@ -569,6 +654,67 @@ Frame *World::FrameByName(const std::string &_name)
       static_cast<const World*>(this)->FrameByName(_name));
 }
 
+/////////////////////////////////////////////////
+uint64_t World::JointCount() const
+{
+  return this->dataPtr->joints.size();
+}
+
+/////////////////////////////////////////////////
+const Joint *World::JointByIndex(const uint64_t _index) const
+{
+  if (_index < this->dataPtr->joints.size())
+    return &this->dataPtr->joints[_index];
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+Joint *World::JointByIndex(uint64_t _index)
+{
+  return const_cast<Joint*>(
+      static_cast<const World*>(this)->JointByIndex(_index));
+}
+
+/////////////////////////////////////////////////
+const Joint *World::JointByName(const std::string &_name) const
+{
+  auto index = _name.rfind("::");
+  if (index != std::string::npos)
+  {
+    const Model *model = this->ModelByName(_name.substr(0, index));
+    if (nullptr != model)
+    {
+      return model->JointByName(_name.substr(index + 2));
+    }
+
+    // The nested model name preceding the last "::" could not be found.
+    // Since "::" are reserved and not allowed in names, return a nullptr.
+    return nullptr;
+  }
+
+  for (auto const &f : this->dataPtr->joints)
+  {
+    if (f.Name() == _name)
+    {
+      return &f;
+    }
+  }
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+Joint *World::JointByName(const std::string &_name)
+{
+  return const_cast<Joint*>(
+      static_cast<const World*>(this)->JointByName(_name));
+}
+
+/////////////////////////////////////////////////
+bool World::JointNameExists(const std::string &_name) const
+{
+  return nullptr != this->JointByName(_name);
+}
+
 /////////////////////////////////////////////////
 uint64_t World::LightCount() const
 {
@@ -627,14 +773,27 @@ Actor *World::ActorByIndex(uint64_t _index)
 /////////////////////////////////////////////////
 bool World::ActorNameExists(const std::string &_name) const
 {
-  for (auto const &a : this->dataPtr->actors)
+  return nullptr != this->ActorByName(_name);
+}
+
+/////////////////////////////////////////////////
+const Actor *World::ActorByName(const std::string &_name) const
+{
+  for (const Actor &a : this->dataPtr->actors)
   {
     if (a.Name() == _name)
     {
-      return true;
+      return &a;
     }
   }
-  return false;
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+Actor *World::ActorByName(const std::string &_name)
+{
+  return const_cast<Actor*>(
+      static_cast<const World*>(this)->ActorByName(_name));
 }
 
 //////////////////////////////////////////////////
@@ -713,6 +872,17 @@ const NestedInclude *World::InterfaceModelNestedIncludeByIndex(
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+void World::ResolveAutoInertials(sdf::Errors &_errors,
+                              const ParserConfig &_config)
+{
+  // Call ResolveAutoInertials() function for all the models
+  for (sdf::Model &model : this->dataPtr->models)
+  {
+    model.ResolveAutoInertials(_errors, _config);
+  }
+}
+
 /////////////////////////////////////////////////
 void World::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
@@ -731,6 +901,10 @@ void World::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
   {
     frame.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
   }
+  for (auto &joint : this->dataPtr->joints)
+  {
+    joint.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
+  }
   for (auto &light : this->dataPtr->lights)
   {
     light.SetXmlParentName("world");
@@ -748,6 +922,10 @@ void World::SetFrameAttachedToGraph(
   {
     frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
   }
+  for (auto &joint : this->dataPtr->joints)
+  {
+    joint.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
+  }
   for (auto &model : this->dataPtr->models)
   {
     model.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
@@ -779,7 +957,9 @@ Errors World::Implementation::LoadSphericalCoordinates(
   else
   {
     auto surfaceModelStr = _elem->Get<std::string>("surface_model");
-    if (surfaceModelStr != "EARTH_WGS84")
+    if ((surfaceModelStr != "EARTH_WGS84") &&
+        (surfaceModelStr != "MOON_SCS") &&
+        (surfaceModelStr != "CUSTOM_SURFACE"))
     {
       errors.push_back({ErrorCode::ELEMENT_INVALID,
           "The supplied <surface_model> [" + surfaceModelStr +
@@ -789,6 +969,33 @@ Errors World::Implementation::LoadSphericalCoordinates(
         surfaceModelStr);
   }
 
+  // Read ellipsoidal parameters for custom surfaces.
+  double axisEquatorial = 0;
+  double axisPolar = 0;
+
+  if (surfaceModel == gz::math::SphericalCoordinates::CUSTOM_SURFACE)
+  {
+    if (!_elem->HasElement("surface_axis_equatorial"))
+    {
+      errors.push_back({ErrorCode::ELEMENT_MISSING,
+          "Missing required element <surface_axis_equatorial>"});
+    }
+    else
+    {
+      axisEquatorial = _elem->Get<double>("surface_axis_equatorial");
+    }
+
+    if (!_elem->HasElement("surface_axis_polar"))
+    {
+      errors.push_back({ErrorCode::ELEMENT_MISSING,
+          "Missing required element <surface_axis_polar>"});
+    }
+    else
+    {
+      axisPolar = _elem->Get<double>("surface_axis_polar");
+    }
+  }
+
   std::string worldFrameOrientation{"ENU"};
   if (_elem->HasElement("world_frame_orientation"))
   {
@@ -848,9 +1055,23 @@ Errors World::Implementation::LoadSphericalCoordinates(
 
   // Create coordinates
   this->sphericalCoordinates.emplace();
-  this->sphericalCoordinates =
-      gz::math::SphericalCoordinates(surfaceModel, latitude, longitude,
-      elevation, heading);
+  if (surfaceModel != gz::math::SphericalCoordinates::CUSTOM_SURFACE)
+  {
+    this->sphericalCoordinates =
+        gz::math::SphericalCoordinates(surfaceModel, latitude, longitude,
+        elevation, heading);
+  }
+  else
+  {
+    this->sphericalCoordinates =
+      gz::math::SphericalCoordinates(surfaceModel,
+          axisEquatorial, axisPolar);
+
+    this->sphericalCoordinates->SetLatitudeReference(latitude);
+    this->sphericalCoordinates->SetLongitudeReference(longitude);
+    this->sphericalCoordinates->SetElevationReference(elevation);
+    this->sphericalCoordinates->SetHeadingOffset(heading);
+  }
 
   return errors;
 }
@@ -880,10 +1101,18 @@ sdf::ElementPtr World::ToElement(const OutputConfig &_config) const
   for (const sdf::Actor &actor : this->dataPtr->actors)
     elem->InsertElement(actor.ToElement(), true);
 
+  // Joints
+  for (const sdf::Joint &joint : this->dataPtr->joints)
+    elem->InsertElement(joint.ToElement(), true);
+
   // Lights
   for (const sdf::Light &light : this->dataPtr->lights)
     elem->InsertElement(light.ToElement(), true);
 
+  // Frames
+  for (const sdf::Frame &frame : this->dataPtr->frames)
+    elem->InsertElement(frame.ToElement(), true);
+
   // Spherical coordinates.
   if (this->dataPtr->sphericalCoordinates)
   {
@@ -900,19 +1129,21 @@ sdf::ElementPtr World::ToElement(const OutputConfig &_config) const
         this->dataPtr->sphericalCoordinates->ElevationReference());
     sphericalElem->GetElement("heading_deg")->Set(
         this->dataPtr->sphericalCoordinates->HeadingOffset().Degree());
+    sphericalElem->GetElement("surface_axis_equatorial")->Set(
+        this->dataPtr->sphericalCoordinates->SurfaceAxisEquatorial());
+    sphericalElem->GetElement("surface_axis_polar")->Set(
+        this->dataPtr->sphericalCoordinates->SurfaceAxisPolar());
   }
 
   // Atmosphere
-  if (this->dataPtr->atmosphere)
-    elem->InsertElement(this->dataPtr->atmosphere->ToElement(), true);
+  elem->InsertElement(this->dataPtr->atmosphere.ToElement(), true);
 
   // Gui
   if (this->dataPtr->gui)
     elem->InsertElement(this->dataPtr->gui->ToElement(), true);
 
   // Scene
-  if (this->dataPtr->scene)
-    elem->InsertElement(this->dataPtr->scene->ToElement(), true);
+  elem->InsertElement(this->dataPtr->scene.ToElement(), true);
 
   // Audio
   if (this->dataPtr->audioDevice != "default")
@@ -937,6 +1168,12 @@ void World::ClearActors()
   this->dataPtr->actors.clear();
 }
 
+/////////////////////////////////////////////////
+void World::ClearJoints()
+{
+  this->dataPtr->joints.clear();
+}
+
 /////////////////////////////////////////////////
 void World::ClearLights()
 {
@@ -955,6 +1192,16 @@ void World::ClearFrames()
   this->dataPtr->frames.clear();
 }
 
+/////////////////////////////////////////////////
+bool World::NameExistsInFrameAttachedToGraph(const std::string &_name) const
+{
+  if (!this->dataPtr->frameAttachedToGraph)
+    return false;
+
+  return this->dataPtr->frameAttachedToGraph.VertexIdByName(_name)
+      !=  gz::math::graph::kNullId;
+}
+
 /////////////////////////////////////////////////
 bool World::AddModel(const Model &_model)
 {
@@ -974,6 +1221,16 @@ bool World::AddActor(const Actor &_actor)
   return true;
 }
 
+/////////////////////////////////////////////////
+bool World::AddJoint(const Joint &_joint)
+{
+  if (this->JointNameExists(_joint.Name()))
+    return false;
+  this->dataPtr->joints.push_back(_joint);
+
+  return true;
+}
+
 /////////////////////////////////////////////////
 bool World::AddLight(const Light &_light)
 {
