@@ -17,13 +17,21 @@
 
 #include <gtest/gtest.h>
 #include <gz/math/Color.hh>
+#include <gz/math/SphericalCoordinates.hh>
 #include <gz/math/Vector3.hh>
+#include <gz/math/MassMatrix3.hh>
+#include <gz/math/Inertial.hh>
+#include <gz/math/Pose3.hh>
 #include "sdf/Frame.hh"
 #include "sdf/Light.hh"
 #include "sdf/Actor.hh"
 #include "sdf/Model.hh"
+#include "sdf/Link.hh"
+#include "sdf/Types.hh"
 #include "sdf/Physics.hh"
+#include "sdf/Root.hh"
 #include "sdf/World.hh"
+#include "sdf/Collision.hh"
 
 /////////////////////////////////////////////////
 TEST(DOMWorld, Construction)
@@ -37,6 +45,11 @@ TEST(DOMWorld, Construction)
   EXPECT_STREQ("default", world.AudioDevice().c_str());
   EXPECT_EQ(gz::math::Vector3d::Zero, world.WindLinearVelocity());
 
+  // The scene and atmosphere are requred, per the SDF spec. Make sure
+  // that they have been created.
+  EXPECT_TRUE(world.Scene());
+  EXPECT_TRUE(world.Atmosphere());
+
   EXPECT_EQ(0u, world.ModelCount());
   EXPECT_EQ(nullptr, world.ModelByIndex(0));
   EXPECT_EQ(nullptr, world.ModelByIndex(1));
@@ -51,17 +64,36 @@ TEST(DOMWorld, Construction)
   EXPECT_EQ(nullptr, world.ModelByName("a::b::c"));
   EXPECT_EQ(nullptr, world.ModelByName("::::"));
 
-  EXPECT_EQ(0u, world.FrameCount());
-  EXPECT_EQ(nullptr, world.FrameByIndex(0));
-  EXPECT_EQ(nullptr, world.FrameByIndex(1));
-  EXPECT_FALSE(world.FrameNameExists(""));
-  EXPECT_FALSE(world.FrameNameExists("default"));
+  EXPECT_EQ(nullptr, world.ActorByName(""));
+  EXPECT_EQ(nullptr, world.ActorByName("default"));
 
   EXPECT_EQ(0u, world.FrameCount());
   EXPECT_EQ(nullptr, world.FrameByIndex(0));
   EXPECT_EQ(nullptr, world.FrameByIndex(1));
   EXPECT_FALSE(world.FrameNameExists(""));
   EXPECT_FALSE(world.FrameNameExists("default"));
+  EXPECT_FALSE(world.FrameNameExists("a::b"));
+  EXPECT_FALSE(world.FrameNameExists("a::b::c"));
+  EXPECT_FALSE(world.FrameNameExists("::::"));
+  EXPECT_EQ(nullptr, world.FrameByName(""));
+  EXPECT_EQ(nullptr, world.FrameByName("default"));
+  EXPECT_EQ(nullptr, world.FrameByName("a::b"));
+  EXPECT_EQ(nullptr, world.FrameByName("a::b::c"));
+  EXPECT_EQ(nullptr, world.FrameByName("::::"));
+
+  EXPECT_EQ(0u, world.JointCount());
+  EXPECT_EQ(nullptr, world.JointByIndex(0));
+  EXPECT_EQ(nullptr, world.JointByIndex(1));
+  EXPECT_FALSE(world.JointNameExists(""));
+  EXPECT_FALSE(world.JointNameExists("default"));
+  EXPECT_FALSE(world.JointNameExists("a::b"));
+  EXPECT_FALSE(world.JointNameExists("a::b::c"));
+  EXPECT_FALSE(world.JointNameExists("::::"));
+  EXPECT_EQ(nullptr, world.JointByName(""));
+  EXPECT_EQ(nullptr, world.JointByName("default"));
+  EXPECT_EQ(nullptr, world.JointByName("a::b"));
+  EXPECT_EQ(nullptr, world.JointByName("a::b::c"));
+  EXPECT_EQ(nullptr, world.JointByName("::::"));
 
   EXPECT_EQ(1u, world.PhysicsCount());
 
@@ -75,6 +107,10 @@ TEST(DOMWorld, Construction)
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
       "PoseRelativeToGraph error: scope does not point to a valid graph"));
+
+  // world doesn't have graphs, so no names should exist in graphs
+  EXPECT_FALSE(world.NameExistsInFrameAttachedToGraph(""));
+  EXPECT_FALSE(world.NameExistsInFrameAttachedToGraph("model1"));
 }
 
 /////////////////////////////////////////////////
@@ -336,7 +372,7 @@ TEST(DOMWorld, SetGui)
 TEST(DOMWorld, SetScene)
 {
   sdf::World world;
-  EXPECT_EQ(nullptr, world.Scene());
+  EXPECT_NE(nullptr, world.Scene());
 
   sdf::Scene scene;
   scene.SetAmbient(gz::math::Color::Blue);
@@ -366,6 +402,9 @@ TEST(DOMWorld, AddModel)
   EXPECT_EQ(1u, world.ModelCount());
   EXPECT_FALSE(world.AddModel(model));
   EXPECT_EQ(1u, world.ModelCount());
+  // world doesn't have graphs, so no names should exist in graphs
+  EXPECT_FALSE(world.NameExistsInFrameAttachedToGraph(""));
+  EXPECT_FALSE(world.NameExistsInFrameAttachedToGraph("model1"));
 
   world.ClearModels();
   EXPECT_EQ(0u, world.ModelCount());
@@ -422,15 +461,50 @@ TEST(DOMWorld, AddActor)
   EXPECT_EQ(1u, world.ActorCount());
   EXPECT_FALSE(world.AddActor(actor));
   EXPECT_EQ(1u, world.ActorCount());
+  EXPECT_NE(nullptr, world.ActorByName("actor1"));
 
   world.ClearActors();
   EXPECT_EQ(0u, world.ActorCount());
+  EXPECT_EQ(nullptr, world.ActorByName("actor1"));
 
   EXPECT_TRUE(world.AddActor(actor));
   EXPECT_EQ(1u, world.ActorCount());
   const sdf::Actor *actorFromWorld = world.ActorByIndex(0);
   ASSERT_NE(nullptr, actorFromWorld);
   EXPECT_EQ(actorFromWorld->Name(), actor.Name());
+
+  const sdf::Actor *actorFromWorldByName = world.ActorByName("actor1");
+  ASSERT_NE(nullptr, actorFromWorldByName);
+  EXPECT_EQ(actorFromWorldByName->Name(), actor.Name());
+
+  sdf::Actor *mutableActorFromWorldByName = world.ActorByName("actor1");
+  ASSERT_NE(nullptr, mutableActorFromWorldByName);
+  EXPECT_EQ(mutableActorFromWorldByName->Name(), actor.Name());
+  mutableActorFromWorldByName->SetName("new_name");
+  EXPECT_NE(mutableActorFromWorldByName->Name(), actor.Name());
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, AddJoint)
+{
+  sdf::World world;
+  EXPECT_EQ(0u, world.JointCount());
+
+  sdf::Joint joint;
+  joint.SetName("joint1");
+  EXPECT_TRUE(world.AddJoint(joint));
+  EXPECT_EQ(1u, world.JointCount());
+  EXPECT_FALSE(world.AddJoint(joint));
+  EXPECT_EQ(1u, world.JointCount());
+
+  world.ClearJoints();
+  EXPECT_EQ(0u, world.JointCount());
+
+  EXPECT_TRUE(world.AddJoint(joint));
+  EXPECT_EQ(1u, world.JointCount());
+  const sdf::Joint *jointFromWorld = world.JointByIndex(0);
+  ASSERT_NE(nullptr, jointFromWorld);
+  EXPECT_EQ(jointFromWorld->Name(), joint.Name());
 }
 
 /////////////////////////////////////////////////
@@ -456,6 +530,63 @@ TEST(DOMWorld, AddLight)
   EXPECT_EQ(lightFromWorld->Name(), light.Name());
 }
 
+/////////////////////////////////////////////////
+TEST(DOMWorld, ResolveAutoInertials)
+{
+  std::string sdf = "<?xml version=\"1.0\"?>"
+  " <sdf version=\"1.11\">"
+  "  <world name='inertial_test_world'>"
+  "   <model name='shapes'>"
+  "     <link name='link'>"
+  "       <inertial auto='true' />"
+  "       <collision name='box_col'>"
+  "         <density>1240.0</density>"
+  "         <geometry>"
+  "           <box>"
+  "             <size>2 2 2</size>"
+  "           </box>"
+  "         </geometry>"
+  "       </collision>"
+  "     </link>"
+  "   </model>"
+  "  </world>"
+  " </sdf>";
+
+  sdf::Root root;
+  const sdf::ParserConfig sdfParserConfig;
+  sdf::Errors errors = root.LoadSdfString(sdf, sdfParserConfig);
+  EXPECT_TRUE(errors.empty());
+  EXPECT_NE(nullptr, root.Element());
+
+  const sdf::World *world = root.WorldByIndex(0);
+  const sdf::Model *model = world->ModelByIndex(0);
+  const sdf::Link *link = model->LinkByIndex(0);
+
+  root.ResolveAutoInertials(errors, sdfParserConfig);
+
+  const double l = 2;
+  const double w = 2;
+  const double h = 2;
+
+  double expectedMass = l*w*h * 1240.0;
+  double ixx = (1.0/12.0) * expectedMass * (w*w + h*h);
+  double iyy = (1.0/12.0) * expectedMass * (l*l + h*h);
+  double izz = (1.0/12.0) * expectedMass * (l*l + w*w);
+
+  gz::math::MassMatrix3d expectedMassMat(
+    expectedMass,
+    gz::math::Vector3d(ixx, iyy, izz),
+    gz::math::Vector3d::Zero
+  );
+
+  gz::math::Inertiald expectedInertial;
+  expectedInertial.SetMassMatrix(expectedMassMat);
+  expectedInertial.SetPose(gz::math::Pose3d::Zero);
+
+  ASSERT_TRUE(errors.empty());
+  EXPECT_EQ(expectedInertial, link->Inertial());
+}
+
 /////////////////////////////////////////////////
 TEST(DOMWorld, ToElement)
 {
@@ -468,6 +599,16 @@ TEST(DOMWorld, ToElement)
   world.SetMagneticField(gz::math::Vector3d(2.0, 0.1, 0.5));
   world.SetSphericalCoordinates(gz::math::SphericalCoordinates());
 
+  sdf::Frame frame1, frame2;
+  frame1.SetName("my-frame1");
+  frame1.SetRawPose(gz::math::Pose3d(1, 2, 3, 0.1, 0.2, 0.3));
+  world.AddFrame(frame1);
+  frame2.SetName("my-frame2");
+  frame2.SetAttachedTo("my-frame1");
+  frame2.SetRawPose(gz::math::Pose3d(-1, 20, 34, 0.1, 0.2, 0.3));
+  world.AddFrame(frame2);
+  EXPECT_EQ(2u, world.FrameCount());
+
   sdf::Atmosphere atmosphere;
   world.SetAtmosphere(atmosphere);
 
@@ -503,6 +644,19 @@ TEST(DOMWorld, ToElement)
       world.ClearActors();
   }
 
+  for (int j = 0; j <= 1; ++j)
+  {
+    for (int i = 0; i < 4; ++i)
+    {
+      sdf::Joint joint;
+      joint.SetName("joint" + std::to_string(i));
+      EXPECT_TRUE(world.AddJoint(joint));
+      EXPECT_FALSE(world.AddJoint(joint));
+    }
+    if (j == 0)
+      world.ClearJoints();
+  }
+
   for (int j = 0; j <= 1; ++j)
   {
     for (int i = 0; i < 4; ++i)
@@ -560,6 +714,10 @@ TEST(DOMWorld, ToElement)
   for (uint64_t i = 0; i < world2.ModelCount(); ++i)
     EXPECT_NE(nullptr, world2.ModelByIndex(i));
 
+  EXPECT_EQ(world.JointCount(), world2.JointCount());
+  for (uint64_t i = 0; i < world2.JointCount(); ++i)
+    EXPECT_NE(nullptr, world2.JointByIndex(i));
+
   EXPECT_EQ(world.LightCount(), world2.LightCount());
   for (uint64_t i = 0; i < world2.LightCount(); ++i)
     EXPECT_NE(nullptr, world2.LightByIndex(i));
@@ -575,6 +733,32 @@ TEST(DOMWorld, ToElement)
   ASSERT_EQ(1u, world2.Plugins().size());
   EXPECT_EQ("name1", world2.Plugins()[0].Name());
   EXPECT_EQ("filename1", world2.Plugins()[0].Filename());
+
+  std::string sphericalCoordsSdf =
+    "<spherical_coordinates>\n"
+    "  <surface_model>EARTH_WGS84</surface_model>\n"
+    "  <latitude_deg>0</latitude_deg>\n"
+    "  <longitude_deg>0</longitude_deg>\n"
+    "  <elevation>0</elevation>\n"
+    "  <heading_deg>0</heading_deg>\n"
+    "  <world_frame_orientation>ENU</world_frame_orientation>\n"
+    "  <surface_axis_equatorial>6378137</surface_axis_equatorial>\n"
+    "  <surface_axis_polar>6356752.3142449996</surface_axis_polar>\n"
+    "</spherical_coordinates>\n";
+
+  EXPECT_EQ(sphericalCoordsSdf,
+      elem->GetElement("spherical_coordinates")->ToString(""));
+
+  ASSERT_EQ(2u, world2.FrameCount());
+  const sdf::Frame *world2Frame1 = world2.FrameByIndex(0);
+  EXPECT_EQ(frame1.Name(), world2Frame1->Name());
+  EXPECT_EQ(frame1.AttachedTo(), world2Frame1->AttachedTo());
+  EXPECT_EQ(frame1.RawPose(), world2Frame1->RawPose());
+
+  const sdf::Frame *world2Frame2 = world2.FrameByIndex(1);
+  EXPECT_EQ(frame2.Name(), world2Frame2->Name());
+  EXPECT_EQ(frame2.AttachedTo(), world2Frame2->AttachedTo());
+  EXPECT_EQ(frame2.RawPose(), world2Frame2->RawPose());
 }
 
 /////////////////////////////////////////////////
@@ -590,6 +774,10 @@ TEST(DOMWorld, MutableByIndex)
   actor.SetName("actor1");
   EXPECT_TRUE(world.AddActor(actor));
 
+  sdf::Joint joint;
+  joint.SetName("joint1");
+  EXPECT_TRUE(world.AddJoint(joint));
+
   sdf::Light light;
   light.SetName("light1");
   EXPECT_TRUE(world.AddLight(light));
@@ -616,6 +804,13 @@ TEST(DOMWorld, MutableByIndex)
   a->SetName("actor2");
   EXPECT_EQ("actor2", world.ActorByIndex(0)->Name());
 
+  // Modify the joint
+  sdf::Joint *j = world.JointByIndex(0);
+  ASSERT_NE(nullptr, j);
+  EXPECT_EQ("joint1", j->Name());
+  j->SetName("joint2");
+  EXPECT_EQ("joint2", world.JointByIndex(0)->Name());
+
   // Modify the light
   sdf::Light *l = world.LightByIndex(0);
   ASSERT_NE(nullptr, l);
@@ -651,6 +846,10 @@ TEST(DOMWorld, MutableByName)
   frame.SetName("frame1");
   EXPECT_TRUE(world.AddFrame(frame));
 
+  sdf::Joint joint;
+  joint.SetName("joint1");
+  EXPECT_TRUE(world.AddJoint(joint));
+
   // Modify the model
   sdf::Model *m = world.ModelByName("model1");
   ASSERT_NE(nullptr, m);
@@ -666,6 +865,14 @@ TEST(DOMWorld, MutableByName)
   f->SetName("frame2");
   EXPECT_FALSE(world.FrameByName("frame1"));
   EXPECT_TRUE(world.FrameByName("frame2"));
+
+  // Modify the joint
+  sdf::Joint *j = world.JointByName("joint1");
+  ASSERT_NE(nullptr, j);
+  EXPECT_EQ("joint1", j->Name());
+  j->SetName("joint2");
+  EXPECT_FALSE(world.JointByName("joint1"));
+  EXPECT_TRUE(world.JointByName("joint2"));
 }
 
 /////////////////////////////////////////////////
@@ -691,3 +898,171 @@ TEST(DOMWorld, Plugins)
   world.ClearPlugins();
   EXPECT_TRUE(world.Plugins().empty());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, LoadEarthSurface)
+{
+  std::string sdf =
+    "<?xml version='1.0'?>"
+    "<sdf version='1.10'>"
+    "<world name='spherical coordinates'>"
+    "<spherical_coordinates>"
+    " <surface_model>EARTH_WGS84</surface_model>"
+    "  <world_frame_orientation>ENU</world_frame_orientation>"
+    "  <latitude_deg>-22.9</latitude_deg>"
+    "  <longitude_deg>-43.2</longitude_deg>"
+    "  <elevation>0</elevation>"
+    "  <heading_deg>0</heading_deg>"
+    "</spherical_coordinates>"
+    "</world>"
+    "</sdf>";
+
+  sdf::Root root;
+  auto sdfErrors = root.LoadSdfString(sdf);
+  EXPECT_NE(nullptr, root.Element());
+  EXPECT_TRUE(sdfErrors.empty());
+
+  auto world = root.WorldByIndex(0);
+  auto sc = world->SphericalCoordinates();
+  EXPECT_EQ(sc->Surface(),
+      gz::math::SphericalCoordinates::EARTH_WGS84);
+  EXPECT_NEAR(sc->LatitudeReference().Degree(), -22.9, 1e-6);
+  EXPECT_NEAR(sc->LongitudeReference().Degree(), -43.2, 1e-6);
+  EXPECT_NEAR(sc->HeadingOffset().Degree(), 0, 1e-6);
+  EXPECT_NEAR(sc->ElevationReference(), 0, 1e-6);
+  EXPECT_NEAR(sc->SurfaceRadius(),
+      6371000.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisEquatorial(),
+      6378137.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisPolar(),
+      6356752.314245, 1e-3);
+  EXPECT_NEAR(sc->SurfaceFlattening(),
+      1.0/298.257223563, 1e-5);
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, LoadMoonSurface)
+{
+  std::string sdf =
+    "<?xml version='1.0'?>"
+    "<sdf version='1.10'>"
+    "<world name='spherical coordinates'>"
+    "<spherical_coordinates>"
+    " <surface_model>MOON_SCS</surface_model>"
+    "  <world_frame_orientation>ENU</world_frame_orientation>"
+    "  <latitude_deg>-22.9</latitude_deg>"
+    "  <longitude_deg>-43.2</longitude_deg>"
+    "  <elevation>0</elevation>"
+    "  <heading_deg>0</heading_deg>"
+    "</spherical_coordinates>"
+    "</world>"
+    "</sdf>";
+
+  sdf::Root root;
+  auto sdfErrors = root.LoadSdfString(sdf);
+  EXPECT_NE(nullptr, root.Element());
+  EXPECT_TRUE(sdfErrors.empty());
+
+  auto world = root.WorldByIndex(0);
+  auto sc = world->SphericalCoordinates();
+  EXPECT_EQ(sc->Surface(),
+      gz::math::SphericalCoordinates::MOON_SCS);
+  EXPECT_NEAR(sc->LatitudeReference().Degree(), -22.9, 1e-6);
+  EXPECT_NEAR(sc->LongitudeReference().Degree(), -43.2, 1e-6);
+  EXPECT_NEAR(sc->HeadingOffset().Degree(), 0, 1e-6);
+  EXPECT_NEAR(sc->ElevationReference(), 0, 1e-6);
+  EXPECT_NEAR(sc->SurfaceRadius(),
+      1737400.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisEquatorial(),
+      1738100.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisPolar(),
+      1736000.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceFlattening(),
+      0.0012, 1e-5);
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, LoadCustomSurface)
+{
+  std::string sdf =
+    "<?xml version='1.0'?>"
+    "<sdf version='1.10'>"
+    "<world name='spherical coordinates'>"
+    "<spherical_coordinates>"
+    " <surface_model>CUSTOM_SURFACE</surface_model>"
+    "  <surface_axis_equatorial>12000</surface_axis_equatorial>"
+    "  <surface_axis_polar>10000</surface_axis_polar>"
+    "  <world_frame_orientation>ENU</world_frame_orientation>"
+    "  <latitude_deg>-22.9</latitude_deg>"
+    "  <longitude_deg>-43.2</longitude_deg>"
+    "  <elevation>0</elevation>"
+    "  <heading_deg>0</heading_deg>"
+    "</spherical_coordinates>"
+    "</world>"
+    "</sdf>";
+
+  sdf::Root root;
+  auto sdfErrors = root.LoadSdfString(sdf);
+  EXPECT_NE(nullptr, root.Element());
+  EXPECT_TRUE(sdfErrors.empty());
+
+  auto world = root.WorldByIndex(0);
+  auto sc = world->SphericalCoordinates();
+  EXPECT_EQ(sc->Surface(),
+      gz::math::SphericalCoordinates::CUSTOM_SURFACE);
+  EXPECT_NEAR(sc->LatitudeReference().Degree(), -22.9, 1e-6);
+  EXPECT_NEAR(sc->LongitudeReference().Degree(), -43.2, 1e-6);
+  EXPECT_NEAR(sc->HeadingOffset().Degree(), 0, 1e-6);
+  EXPECT_NEAR(sc->ElevationReference(), 0, 1e-6);
+  EXPECT_NEAR(sc->SurfaceRadius(),
+      11333.333, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisEquatorial(),
+      12000.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisPolar(),
+      10000.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceFlattening(),
+      0.1666666, 1e-5);
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, LoadCustomSurfaceWithErrors)
+{
+  std::string sdf =
+    "<?xml version='1.0'?>"
+    "<sdf version='1.10'>"
+    "<world name='spherical coordinates'>"
+    "<spherical_coordinates>"
+    " <surface_model>CUSTOM_SURFACE</surface_model>"
+    "  <world_frame_orientation>ENU</world_frame_orientation>"
+    "  <latitude_deg>-22.9</latitude_deg>"
+    "  <longitude_deg>-43.2</longitude_deg>"
+    "  <elevation>0</elevation>"
+    "  <heading_deg>0</heading_deg>"
+    "</spherical_coordinates>"
+    "</world>"
+    "</sdf>";
+
+  sdf::Root root;
+  auto sdfErrors = root.LoadSdfString(sdf);
+  EXPECT_NE(nullptr, root.Element());
+  EXPECT_EQ(sdfErrors.size(), 3);
+
+  // Since equatorial and polar axes were not passed
+  // in the sdf, the measurements default to Earth's values.
+  auto world = root.WorldByIndex(0);
+  auto sc = world->SphericalCoordinates();
+  EXPECT_EQ(sc->Surface(),
+      gz::math::SphericalCoordinates::CUSTOM_SURFACE);
+  EXPECT_NEAR(sc->LatitudeReference().Degree(), -22.9, 1e-6);
+  EXPECT_NEAR(sc->LongitudeReference().Degree(), -43.2, 1e-6);
+  EXPECT_NEAR(sc->HeadingOffset().Degree(), 0, 1e-6);
+  EXPECT_NEAR(sc->ElevationReference(), 0, 1e-6);
+  EXPECT_NEAR(sc->SurfaceRadius(),
+      6371000.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisEquatorial(),
+      6378137.0, 1e-3);
+  EXPECT_NEAR(sc->SurfaceAxisPolar(),
+      6356752.314245, 1e-3);
+  EXPECT_NEAR(sc->SurfaceFlattening(),
+      1.0/298.257223563, 1e-5);
+}
