@@ -23,7 +23,10 @@
 #include "sdf/Element.hh"
 #include "sdf/Console.hh"
 #include "sdf/Filesystem.hh"
-#include "test_config.h"
+
+#include <gz/utils/Environment.hh>
+
+#include "test_config.hh"
 #include "test_utils.hh"
 
 /////////////////////////////////////////////////
@@ -76,13 +79,14 @@ TEST(Parser, CustomUnknownElements)
   EXPECT_TRUE(sdf::readFile(path, sdf));
 
 #ifndef _WIN32
-  char *homeDir = getenv("HOME");
+  std::string homeVarName = "HOME";
 #else
-  char *homeDir;
-  size_t sz = 0;
-  _dupenv_s(&homeDir, &sz, "HOMEPATH");
+  std::string homeVarName = "HOMEPATH";
 #endif
 
+  std::string homeDir;
+  ASSERT_TRUE(gz::utils::env(homeVarName, homeDir));
+
   std::string pathLog =
     sdf::filesystem::append(homeDir, ".sdformat", "sdformat.log");
 
@@ -147,8 +151,8 @@ TEST(Parser, readFileConversions)
 /////////////////////////////////////////////////
 TEST(Parser, NameUniqueness)
 {
-  // These tests are copies of the ones in ign_TEST.cc but use direct calls to
-  // name uniqueness validator functions instead of going through ign.
+  // These tests are copies of the ones in gz_TEST.cc but use direct calls to
+  // name uniqueness validator functions instead of going through gz.
 
   // Check an SDF file with sibling elements of the same type (world)
   // that have duplicate names.
@@ -311,7 +315,7 @@ TEST(Parser, SyntaxErrorInValues)
 
     sdf::readFile(path, sdf);
     EXPECT_PRED2(sdf::testing::contains, buffer.str(),
-                 "Unable to set value [bad ] for key[linear]");
+                 "Unable to set value [bad] for key[linear]");
     EXPECT_PRED2(sdf::testing::contains, buffer.str(),
                  "bad_syntax_double.sdf:L7");
     EXPECT_PRED2(sdf::testing::contains, buffer.str(),
@@ -415,10 +419,18 @@ TEST(Parser, IncludesErrors)
 
     const auto path =
         sdf::testing::TestFile("sdf", "includes_missing_model.sdf");
+    sdf::setFindCallback([&](const std::string &/*_file*/)
+        {
+          return "";
+        });
+
     sdf::SDFPtr sdf(new sdf::SDF());
     sdf::init(sdf);
 
     sdf::readFile(path, sdf);
+
+    std::cout << buffer.str() << std::endl;
+
     EXPECT_PRED2(sdf::testing::contains, buffer.str(),
                  "Error Code " +
                  std::to_string(
@@ -523,8 +535,14 @@ TEST(Parser, DoubleColonNameAttrError)
 
   sdf::Errors errors;
   EXPECT_FALSE(sdf::readString(stream.str(), sdf, errors));
-  ASSERT_EQ(errors.size(), 1u);
+  ASSERT_EQ(errors.size(), 2u);
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::RESERVED_NAME);
+  EXPECT_NE(std::string::npos, errors[0].Message().find(
+      "Detected delimiter '::' in element name in<link name='A::B'/>"));
+  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::RESERVED_NAME);
+  EXPECT_NE(std::string::npos, errors[1].Message().find(
+      "Delimiter '::' found in attribute names of element <sdf>,"
+      " which is not allowed in SDFormat >= 1.8"));
 
   sdf = InitSDF();
   stream.str("");
@@ -832,13 +850,110 @@ TEST(Parser, ElementRemovedAfterDeprecation)
   EXPECT_EQ(errors[0].LineNumber().value(), 10);
 }
 
+/////////////////////////////////////////////////
+/// Check for non SDF non URDF error on a string
+TEST(Parser, ReadStringErrorNotSDForURDF)
+{
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+  #ifdef _WIN32
+    sdf::Console::Instance()->SetQuiet(false);
+    sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+  #endif
+
+  sdf::SDFPtr sdf = InitSDF();
+  const std::string testString = R"(<?xml version="1.0" ?>
+    <foo>
+    </foo>)";
+  sdf::Errors errors;
+  EXPECT_FALSE(sdf::readString(testString, sdf, errors));
+  ASSERT_EQ(errors.size(), 1u);
+  EXPECT_NE(std::string::npos, errors[0].Message().find(
+      "XML does not seem to be an SDFormat or an URDF string."));
+
+  // Check nothing has been printed
+  EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+}
+
+/////////////////////////////////////////////////
+/// Check for non SDF non URDF error on a file
+TEST(Parser, ReadFileErrorNotSDForURDF)
+{
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+  #ifdef _WIN32
+    sdf::Console::Instance()->SetQuiet(false);
+    sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+  #endif
+
+  const auto path =
+      sdf::testing::TestFile("sdf", "box.dae");
+
+  sdf::Errors errors;
+  sdf::SDFPtr sdf = sdf::readFile(path, errors);
+  ASSERT_EQ(errors.size(), 1u);
+  EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::PARSING_ERROR);
+  EXPECT_NE(std::string::npos, errors[0].Message().find(
+      "XML does not seem to be an SDFormat or an URDF file."));
+
+  // Check nothing has been printed
+  EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+}
+
+/////////////////////////////////////////////////
+/// Check that malformed SDF files do not throw confusing error
+TEST(Parser, ReadFileMalformedSDFNoRobotError)
+{
+  // Capture sdferr output
+  std::stringstream buffer;
+  auto old = std::cerr.rdbuf(buffer.rdbuf());
+
+#ifdef _WIN32
+  sdf::Console::Instance()->SetQuiet(false);
+#endif
+
+  const auto path =
+      sdf::testing::TestFile("sdf", "bad_syntax_pose.sdf");
+
+  sdf::Errors errors;
+  sdf::SDFPtr sdf = sdf::readFile(path, errors);
+  // Check the old error is not printed anymore
+  EXPECT_EQ(std::string::npos, buffer.str().find(
+      "Could not find the 'robot' element in the xml file"));
+
+  // Revert cerr rdbug so as to not interfere with other tests
+  std::cerr.rdbuf(old);
+#ifdef _WIN32
+  sdf::Console::Instance()->SetQuiet(true);
+#endif
+}
+
 /////////////////////////////////////////////////
 /// Main
 int main(int argc, char **argv)
 {
   // temporarily set HOME
   std::string homeDir;
-  sdf::testing::TestSetHomePath(homeDir);
+  sdf::testing::TestTmpPath(homeDir);
+
+#ifdef _WIN32
+  gz::utils::setenv("HOMEPATH", homeDir);
+#else
+  gz::utils::setenv("HOME", homeDir);
+#endif
+
   sdf::Console::Clear();
 
   ::testing::InitGoogleTest(&argc, argv);
