@@ -13,19 +13,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
-*/
+ */
+
+#include <string>
 
 #include <gtest/gtest.h>
-#include <map>
+
 #include "sdf/sdf.hh"
 
-#include "test_config.h"
-
-const std::string SDF_FIXED_JNT = std::string(PROJECT_SOURCE_PATH)
-  + "/test/integration/fixed_joint_reduction.urdf";
-
-const std::string SDF_FIXED_JNT_NO_LUMPING = std::string(PROJECT_SOURCE_PATH)
-  + "/test/integration/fixed_joint_reduction_disabled.urdf";
+#include "test_config.hh"
 
 /////////////////////////////////////////////////
 bool findJointInModel(const std::string &desired_joint_name, sdf::SDFPtr robot)
@@ -38,8 +34,8 @@ bool findJointInModel(const std::string &desired_joint_name, sdf::SDFPtr robot)
     std::string jointName = joint->Get<std::string>("name");
     if (jointName == desired_joint_name)
     {
-        found = true;
-        break;
+      found = true;
+      break;
     }
   }
 
@@ -49,13 +45,19 @@ bool findJointInModel(const std::string &desired_joint_name, sdf::SDFPtr robot)
 /////////////////////////////////////////////////
 TEST(SDFParser, DisableFixedJointReductionTest)
 {
+  const std::string sdfFixedJntFile =
+      sdf::testing::TestFile("integration", "fixed_joint_reduction.urdf");
+
+  const std::string sdfFixedJntNoLumpingFile = sdf::testing::TestFile(
+      "integration", "fixed_joint_reduction_disabled.urdf");
+
   sdf::SDFPtr robot(new sdf::SDF());
   sdf::init(robot);
-  ASSERT_TRUE(sdf::readFile(SDF_FIXED_JNT, robot));
+  ASSERT_TRUE(sdf::readFile(sdfFixedJntFile, robot));
 
   sdf::SDFPtr robot_disable_lumping(new sdf::SDF());
   sdf::init(robot_disable_lumping);
-  ASSERT_TRUE(sdf::readFile(SDF_FIXED_JNT_NO_LUMPING, robot_disable_lumping));
+  ASSERT_TRUE(sdf::readFile(sdfFixedJntNoLumpingFile, robot_disable_lumping));
 
   ASSERT_FALSE(findJointInModel("joint12a", robot));
   ASSERT_FALSE(findJointInModel("joint23a", robot));
