@@ -22,7 +22,7 @@
 
 #include "sdf/sdf.hh"
 
-#include "test_config.h"
+#include "test_config.hh"
 
 const char SDF_TEST_FILE[] = "fixed_joint_reduction.urdf";
 const char SDF_TEST_FILE_COLLISION[] = "fixed_joint_reduction_collision.urdf";
@@ -762,8 +762,8 @@ TEST(SDFParser, FixedJointReductionJointFrameExtensionTest)
   const sdf::Joint *joint = model->JointByName(ball_joint_name);
   ASSERT_NE(nullptr, joint);
 
-  EXPECT_EQ("link3", joint->ChildLinkName());
-  EXPECT_EQ("base_link", joint->ParentLinkName());
+  EXPECT_EQ("link3", joint->ChildName());
+  EXPECT_EQ("base_link", joint->ParentName());
 }
 
 /////////////////////////////////////////////////
