@@ -21,6 +21,7 @@
 
 #include <gz/math/Pose3.hh>
 #include "sdf/AirPressure.hh"
+#include "sdf/AirSpeed.hh"
 #include "sdf/Altimeter.hh"
 #include "sdf/Camera.hh"
 #include "sdf/Collision.hh"
@@ -39,7 +40,7 @@
 #include "sdf/Types.hh"
 #include "sdf/Visual.hh"
 #include "sdf/World.hh"
-#include "test_config.h"
+#include "test_config.hh"
 
 //////////////////////////////////////////////////
 TEST(DOMLink, NotALink)
@@ -138,6 +139,7 @@ TEST(DOMLink, InertialDoublePendulum)
   EXPECT_DOUBLE_EQ(0.0, inertial.MassMatrix().OffDiagonalMoments().X());
   EXPECT_DOUBLE_EQ(0.0, inertial.MassMatrix().OffDiagonalMoments().Y());
   EXPECT_DOUBLE_EQ(0.0, inertial.MassMatrix().OffDiagonalMoments().Z());
+  EXPECT_FALSE(inertial.FluidAddedMass().has_value());
 
   const sdf::Link *upperLink = model->LinkByIndex(1);
   ASSERT_NE(nullptr, upperLink);
@@ -158,6 +160,7 @@ TEST(DOMLink, InertialDoublePendulum)
   EXPECT_DOUBLE_EQ(0.0, inertialUpper.Pose().Pos().Y());
   EXPECT_DOUBLE_EQ(0.5, inertialUpper.Pose().Pos().Z());
   EXPECT_TRUE(inertial.MassMatrix().IsValid());
+  EXPECT_FALSE(inertial.FluidAddedMass().has_value());
 
   const sdf::Link *lowerLink = model->LinkByIndex(2);
   ASSERT_TRUE(lowerLink != nullptr);
@@ -195,6 +198,45 @@ TEST(DOMLink, InertialComplete)
   EXPECT_DOUBLE_EQ(0.0, inertial.Pose().Pos().Y());
   EXPECT_DOUBLE_EQ(0.02, inertial.Pose().Pos().Z());
   EXPECT_TRUE(inertial.MassMatrix().IsValid());
+
+  ASSERT_TRUE(inertial.FluidAddedMass().has_value());
+  auto addedMass = inertial.FluidAddedMass().value();
+  EXPECT_DOUBLE_EQ(11, addedMass(0, 0));
+  EXPECT_DOUBLE_EQ(12, addedMass(0, 1));
+  EXPECT_DOUBLE_EQ(13, addedMass(0, 2));
+  EXPECT_DOUBLE_EQ(14, addedMass(0, 3));
+  EXPECT_DOUBLE_EQ(15, addedMass(0, 4));
+  EXPECT_DOUBLE_EQ(16, addedMass(0, 5));
+  EXPECT_DOUBLE_EQ(12, addedMass(1, 0));
+  EXPECT_DOUBLE_EQ(22, addedMass(1, 1));
+  EXPECT_DOUBLE_EQ(23, addedMass(1, 2));
+  EXPECT_DOUBLE_EQ(24, addedMass(1, 3));
+  EXPECT_DOUBLE_EQ(25, addedMass(1, 4));
+  EXPECT_DOUBLE_EQ(26, addedMass(1, 5));
+  EXPECT_DOUBLE_EQ(13, addedMass(2, 0));
+  EXPECT_DOUBLE_EQ(23, addedMass(2, 1));
+  EXPECT_DOUBLE_EQ(33, addedMass(2, 2));
+  EXPECT_DOUBLE_EQ(34, addedMass(2, 3));
+  EXPECT_DOUBLE_EQ(35, addedMass(2, 4));
+  EXPECT_DOUBLE_EQ(36, addedMass(2, 5));
+  EXPECT_DOUBLE_EQ(14, addedMass(3, 0));
+  EXPECT_DOUBLE_EQ(24, addedMass(3, 1));
+  EXPECT_DOUBLE_EQ(34, addedMass(3, 2));
+  EXPECT_DOUBLE_EQ(44, addedMass(3, 3));
+  EXPECT_DOUBLE_EQ(45, addedMass(3, 4));
+  EXPECT_DOUBLE_EQ(46, addedMass(3, 5));
+  EXPECT_DOUBLE_EQ(15, addedMass(4, 0));
+  EXPECT_DOUBLE_EQ(25, addedMass(4, 1));
+  EXPECT_DOUBLE_EQ(35, addedMass(4, 2));
+  EXPECT_DOUBLE_EQ(45, addedMass(4, 3));
+  EXPECT_DOUBLE_EQ(55, addedMass(4, 4));
+  EXPECT_DOUBLE_EQ(56, addedMass(4, 5));
+  EXPECT_DOUBLE_EQ(16, addedMass(5, 0));
+  EXPECT_DOUBLE_EQ(26, addedMass(5, 1));
+  EXPECT_DOUBLE_EQ(36, addedMass(5, 2));
+  EXPECT_DOUBLE_EQ(46, addedMass(5, 3));
+  EXPECT_DOUBLE_EQ(56, addedMass(5, 4));
+  EXPECT_DOUBLE_EQ(66, addedMass(5, 5));
 }
 
 //////////////////////////////////////////////////
@@ -243,7 +285,7 @@ TEST(DOMLink, Sensors)
   const sdf::Link *link = model->LinkByIndex(0);
   ASSERT_NE(nullptr, link);
   EXPECT_EQ("link", link->Name());
-  EXPECT_EQ(26u, link->SensorCount());
+  EXPECT_EQ(27u, link->SensorCount());
 
   // Get the altimeter sensor
   const sdf::Sensor *altimeterSensor = link->SensorByIndex(0);
@@ -661,6 +703,20 @@ TEST(DOMLink, Sensors)
   EXPECT_DOUBLE_EQ(5.6, airSensor->PressureNoise().StdDev());
   EXPECT_DOUBLE_EQ(123.4, airSensor->ReferenceAltitude());
 
+  // Get the air_speed sensor
+  const sdf::Sensor *airSpeedSensor = link->SensorByName(
+      "air_speed_sensor");
+  ASSERT_NE(nullptr, airSpeedSensor);
+  EXPECT_EQ("air_speed_sensor", airSpeedSensor->Name());
+  EXPECT_EQ(sdf::SensorType::AIR_SPEED, airSpeedSensor->Type());
+  EXPECT_EQ(gz::math::Pose3d(11, 22, 33, 0, 0, 0),
+      airSpeedSensor->RawPose());
+  EXPECT_FALSE(airSpeedSensor->EnableMetrics());
+  const sdf::AirSpeed *airSpeedSensorSDF = airSpeedSensor->AirSpeedSensor();
+  ASSERT_NE(nullptr, airSpeedSensorSDF);
+  EXPECT_DOUBLE_EQ(0.0, airSpeedSensorSDF->PressureNoise().Mean());
+  EXPECT_DOUBLE_EQ(0.01, airSpeedSensorSDF->PressureNoise().StdDev());
+
   // Get the wide angle camera sensor
   EXPECT_TRUE(link->SensorNameExists("wide_angle_camera_sensor"));
   const sdf::Sensor *wideAngleCameraSensor =
@@ -730,12 +786,12 @@ TEST(DOMLink, LoadLinkPoseRelativeTo)
   EXPECT_EQ("__model__", model->LinkByName("L2a")->PoseRelativeTo());
   EXPECT_EQ("L1", model->LinkByName("L3")->PoseRelativeTo());
 
-  EXPECT_EQ(Pose(1, 0, 0, 0, IGN_PI/2, 0), model->LinkByName("L1")->RawPose());
+  EXPECT_EQ(Pose(1, 0, 0, 0, GZ_PI/2, 0), model->LinkByName("L1")->RawPose());
   EXPECT_EQ(Pose(2, 0, 0, 0, 0, 0), model->LinkByName("L2")->RawPose());
   EXPECT_EQ(Pose(12, 0, 0, 0, 0, 0), model->LinkByName("L2a")->RawPose());
   EXPECT_EQ(Pose(3, 0, 0, 0, 0, 0), model->LinkByName("L3")->RawPose());
 
-  EXPECT_EQ(Pose(1, 0, 0, 0, IGN_PI / 2, 0),
+  EXPECT_EQ(Pose(1, 0, 0, 0, GZ_PI / 2, 0),
             model->LinkByName("L1")->SemanticPose().RawPose());
   EXPECT_EQ(Pose(2, 0, 0, 0, 0, 0),
             model->LinkByName("L2")->SemanticPose().RawPose());
@@ -748,7 +804,7 @@ TEST(DOMLink, LoadLinkPoseRelativeTo)
   Pose pose;
   EXPECT_TRUE(
     model->LinkByName("L1")->SemanticPose().Resolve(pose, "__model__").empty());
-  EXPECT_EQ(Pose(1, 0, 0, 0, IGN_PI/2, 0), pose);
+  EXPECT_EQ(Pose(1, 0, 0, 0, GZ_PI/2, 0), pose);
   EXPECT_TRUE(
     model->LinkByName("L2")->SemanticPose().Resolve(pose, "__model__").empty());
   EXPECT_EQ(Pose(2, 0, 0, 0, 0, 0), pose);
@@ -758,16 +814,16 @@ TEST(DOMLink, LoadLinkPoseRelativeTo)
   EXPECT_EQ(Pose(12, 0, 0, 0, 0, 0), pose);
   EXPECT_TRUE(
     model->LinkByName("L3")->SemanticPose().Resolve(pose, "__model__").empty());
-  EXPECT_EQ(Pose(1, 0, -3, 0, IGN_PI/2, 0), pose);
+  EXPECT_EQ(Pose(1, 0, -3, 0, GZ_PI/2, 0), pose);
   // test other API too
   EXPECT_TRUE(model->LinkByName("L1")->SemanticPose().Resolve(pose).empty());
-  EXPECT_EQ(Pose(1, 0, 0, 0, IGN_PI/2, 0), pose);
+  EXPECT_EQ(Pose(1, 0, 0, 0, GZ_PI/2, 0), pose);
   EXPECT_TRUE(model->LinkByName("L2")->SemanticPose().Resolve(pose).empty());
   EXPECT_EQ(Pose(2, 0, 0, 0, 0, 0), pose);
   EXPECT_TRUE(model->LinkByName("L2a")->SemanticPose().Resolve(pose).empty());
   EXPECT_EQ(Pose(12, 0, 0, 0, 0, 0), pose);
   EXPECT_TRUE(model->LinkByName("L3")->SemanticPose().Resolve(pose).empty());
-  EXPECT_EQ(Pose(1, 0, -3, 0, IGN_PI/2, 0), pose);
+  EXPECT_EQ(Pose(1, 0, -3, 0, GZ_PI/2, 0), pose);
 
   // resolve pose of L1 relative to L3
   // should be inverse of L3's Pose()
@@ -873,3 +929,132 @@ TEST(DOMLink, InvalidInertialPoseRelTo)
   EXPECT_EQ(link->Inertial().Pose(),
       gz::math::Pose3d(0.1, 1, 0.2, 0, 0, -0.52));
 }
+
+/////////////////////////////////////////////////
+TEST(DOMLink, InertialAuto)
+{
+  const std::string testFile = sdf::testing::TestFile("sdf",
+        "inertial_stats_auto.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  auto errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(model, nullptr);
+
+  const sdf::Link *link = model->LinkByName("link_1");
+  ASSERT_NE(link, nullptr);
+
+  // Verify inertial values for link_1 match those in inertial_stats.sdf
+  gz::math::Inertiald inertial = link->Inertial();
+  gz::math::MassMatrix3d massMatrix = inertial.MassMatrix();
+  EXPECT_EQ(gz::math::Pose3d::Zero, inertial.Pose());
+  EXPECT_DOUBLE_EQ(6.0, massMatrix.Mass());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Ixx());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Iyy());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Izz());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixy());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixz());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Iyz());
+
+  // Expect //link/inertial element to not contain inertia, mass, or pose
+  auto linkElem = link->Element();
+  auto inertialElem = linkElem->FindElement("inertial");
+  ASSERT_TRUE(inertialElem != nullptr);
+  EXPECT_FALSE(inertialElem->HasElement("inertia"));
+  EXPECT_FALSE(inertialElem->HasElement("mass"));
+  EXPECT_FALSE(inertialElem->HasElement("pose"));
+}
+
+/////////////////////////////////////////////////
+TEST(DOMLink, InertialAutoExplicitMass)
+{
+  const std::string testFile = sdf::testing::TestFile("sdf",
+        "inertial_stats_auto_mass.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  auto errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(model, nullptr);
+
+  std::vector<std::string> linkNames{"link_1", "link_2", "link_3", "link_4"};
+  for (const std::string &linkName : linkNames)
+  {
+    const sdf::Link *link = model->LinkByName(linkName);
+    ASSERT_NE(link, nullptr);
+
+    // Verify inertial values for link_i match those in inertial_stats.sdf
+    gz::math::Inertiald inertial = link->Inertial();
+    gz::math::MassMatrix3d massMatrix = inertial.MassMatrix();
+    EXPECT_EQ(gz::math::Pose3d::Zero, inertial.Pose());
+    EXPECT_DOUBLE_EQ(6.0, massMatrix.Mass());
+    EXPECT_DOUBLE_EQ(1.0, massMatrix.Ixx());
+    EXPECT_DOUBLE_EQ(1.0, massMatrix.Iyy());
+    EXPECT_DOUBLE_EQ(1.0, massMatrix.Izz());
+    EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixy());
+    EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixz());
+    EXPECT_DOUBLE_EQ(0.0, massMatrix.Iyz());
+  }
+}
+
+/////////////////////////////////////////////////
+TEST(DOMLink, InertialAutoSaveInElement)
+{
+  const std::string testFile = sdf::testing::TestFile("sdf",
+        "inertial_stats_auto.sdf");
+
+  // Load the SDF file with SAVE_CALCULATION_IN_ELEMENT
+  sdf::ParserConfig config;
+  config.SetCalculateInertialConfiguration(
+    sdf::ConfigureResolveAutoInertials::SAVE_CALCULATION_IN_ELEMENT);
+
+  sdf::Root root;
+  auto errors = root.Load(testFile, config);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(model, nullptr);
+
+  const sdf::Link *link = model->LinkByName("link_1");
+  ASSERT_NE(link, nullptr);
+
+  // Verify inertial values for link_1 match those in inertial_stats.sdf
+  gz::math::Inertiald inertial = link->Inertial();
+  gz::math::MassMatrix3d massMatrix = inertial.MassMatrix();
+  EXPECT_EQ(gz::math::Pose3d::Zero, inertial.Pose());
+  EXPECT_DOUBLE_EQ(6.0, massMatrix.Mass());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Ixx());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Iyy());
+  EXPECT_DOUBLE_EQ(1.0, massMatrix.Izz());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixy());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Ixz());
+  EXPECT_DOUBLE_EQ(0.0, massMatrix.Iyz());
+
+  // Expect //link/inertial element to contain inertia values
+  auto linkElem = link->Element();
+  auto inertialElem = linkElem->FindElement("inertial");
+  ASSERT_TRUE(inertialElem != nullptr);
+  EXPECT_TRUE(inertialElem->HasElement("mass"));
+  EXPECT_DOUBLE_EQ(6.0, inertialElem->Get<double>("mass"));
+  EXPECT_EQ(gz::math::Pose3d::Zero,
+    inertialElem->Get<gz::math::Pose3d>("pose"));
+  auto momentOfInertiaElem = inertialElem->FindElement("inertia");
+  ASSERT_TRUE(momentOfInertiaElem != nullptr);
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("ixx"));
+  EXPECT_DOUBLE_EQ(1.0, momentOfInertiaElem->Get<double>("ixx"));
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("iyy"));
+  EXPECT_DOUBLE_EQ(1.0, momentOfInertiaElem->Get<double>("iyy"));
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("izz"));
+  EXPECT_DOUBLE_EQ(1.0, momentOfInertiaElem->Get<double>("izz"));
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("ixy"));
+  EXPECT_DOUBLE_EQ(0.0, momentOfInertiaElem->Get<double>("ixy"));
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("ixz"));
+  EXPECT_DOUBLE_EQ(0.0, momentOfInertiaElem->Get<double>("ixz"));
+  EXPECT_TRUE(momentOfInertiaElem->HasElement("iyz"));
+  EXPECT_DOUBLE_EQ(0.0, momentOfInertiaElem->Get<double>("iyz"));
+}
