@@ -13,14 +13,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
-*/
+ */
+
+#include <sstream>
+#include <string>
 
 #include <gtest/gtest.h>
-#include <boost/filesystem.hpp>
-#include <string>
-#include "sdf/sdf.hh"
 
-#include "test_config.h"
+#include "sdf/sdf.hh"
 
 std::string get_sdf_string()
 {
@@ -30,7 +30,7 @@ std::string get_sdf_string()
     << "<model name='model'>"
     << "  <link name='link'/>"
     << "  <plugin name='example' filename='libexample.so'>"
-    << "    <user attribute='attribute' />"
+    << "    <user attribute='attribute' empty_attribute='' />"
     << "    <value attribute='attribute'>value</value>"
     << "  </plugin>"
     << "</model>"
@@ -42,22 +42,9 @@ std::string get_sdf_string()
 // make sure that plugin attributes get parsed
 TEST(PluginAttribute, ParseAttributes)
 {
-  char *pathCStr = getenv("SDF_PATH");
-  boost::filesystem::path path = PROJECT_SOURCE_PATH;
-  path = path / "sdf" / SDF_VERSION;
-  setenv("SDF_PATH", path.string().c_str(), 1);
-
   sdf::SDFPtr model(new sdf::SDF());
   sdf::init(model);
   ASSERT_TRUE(sdf::readString(get_sdf_string(), model));
-  if (pathCStr)
-  {
-    setenv("SDF_PATH", pathCStr, 1);
-  }
-  else
-  {
-    unsetenv("SDF_PATH");
-  }
 
   sdf::ElementPtr plugin =
     model->Root()->GetElement("model")->GetElement("plugin");
@@ -67,6 +54,8 @@ TEST(PluginAttribute, ParseAttributes)
     EXPECT_TRUE(user->HasAttribute("attribute"));
     EXPECT_EQ(user->GetAttribute("attribute")->GetAsString(),
               std::string("attribute"));
+    EXPECT_TRUE(user->HasAttribute("empty_attribute"));
+    EXPECT_EQ(user->GetAttribute("empty_attribute")->GetAsString(), "");
   }
   {
     sdf::ElementPtr value = plugin->GetElement("value");
