@@ -31,7 +31,7 @@
 #include "sdf/World.hh"
 #include "sdf/parser.hh"
 #include "sdf/Filesystem.hh"
-#include "test_config.h"
+#include "test_config.hh"
 #include "test_utils.hh"
 
 //////////////////////////////////////////////////
@@ -125,7 +125,7 @@ TEST(Pose1_9, PoseExpressionFormats)
   model = world->ModelByIndex(14);
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_euler_rpy_degrees_true", model->Name());
-  EXPECT_EQ(Pose(1, 2, 3, IGN_DTOR(90), IGN_DTOR(180), IGN_DTOR(270)),
+  EXPECT_EQ(Pose(1, 2, 3, GZ_DTOR(90), GZ_DTOR(180), GZ_DTOR(270)),
       model->RawPose());
 
   model = world->ModelByIndex(15);
@@ -146,7 +146,7 @@ TEST(Pose1_9, PoseExpressionFormats)
     const auto link = model->LinkByIndex(0);
     ASSERT_NE(nullptr, link);
     EXPECT_EQ("link_euler_rpy_degrees_true", link->Name());
-    EXPECT_EQ(Pose(1, 2, 3, IGN_DTOR(90), IGN_DTOR(180), IGN_DTOR(270)),
+    EXPECT_EQ(Pose(1, 2, 3, GZ_DTOR(90), GZ_DTOR(180), GZ_DTOR(270)),
               link->Inertial().Pose());
   }
   {
@@ -758,7 +758,9 @@ TEST(Pose1_9, ToStringWithoutAttrib)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3  0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 }
 
@@ -780,7 +782,9 @@ TEST(Pose1_9, ToStringWithDegreesFalse)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3  0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='false'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 }
@@ -803,7 +807,9 @@ TEST(Pose1_9, ToStringWithDegreesTrue)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3  0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='true'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 }
@@ -827,7 +833,9 @@ TEST(Pose1_9, ToStringWithEulerRPY)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3  0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "rotation_format='euler_rpy'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 }
@@ -855,7 +863,9 @@ TEST(Pose1_9, ToStringWithEulerRPYDegreesTrue)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3  0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='true'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "rotation_format='euler_rpy'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
@@ -880,9 +890,11 @@ TEST(Pose1_9, ToStringWithQuatXYZ)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3   0.7071068 0 0 0.7071068"));
 
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
   // The string output has changed as it was parsed from the value, instead of
   // the original string.
-  std::string elemStr = poseElem->ToString("");
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "rotation_format='quat_xyzw'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.707107 0 0 0.707107");
 }
@@ -910,9 +922,11 @@ TEST(Pose1_9, ToStringWithQuatXYZWDegreesFalse)
   ASSERT_NE(nullptr, poseValueParam);
   EXPECT_TRUE(poseValueParam->SetFromString("1 2 3   0.7071068 0 0 0.7071068"));
 
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
   // The string output has changed as it was parsed from the value, instead of
   // the original string.
-  std::string elemStr = poseElem->ToString("");
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='false'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "rotation_format='quat_xyzw'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.707107 0 0 0.707107");
@@ -933,7 +947,9 @@ TEST(Pose1_9, ToStringAfterChangingDegreeAttribute)
   ASSERT_NE(nullptr, valParam);
   ASSERT_TRUE(valParam->SetFromString("1 2 3 0.4 0.5 0.6"));
 
-  std::string elemStr = poseElem->ToString("");
+  sdf::PrintConfig config;
+  config.SetOutPrecision(6);
+  std::string elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 
   // Changing to attribute to degrees, however this does not modify the
@@ -944,13 +960,14 @@ TEST(Pose1_9, ToStringAfterChangingDegreeAttribute)
   ASSERT_TRUE(degreesAttrib->Set<bool>(true));
   EXPECT_TRUE(valParam->Reparse());
 
-  elemStr = poseElem->ToString("");
+  elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='true'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 
   // Changing back to radians
   ASSERT_TRUE(degreesAttrib->Set<bool>(false));
-  elemStr = poseElem->ToString("");
+  EXPECT_TRUE(valParam->Reparse());
+  elemStr = poseElem->ToString("", config);
   EXPECT_PRED2(sdf::testing::contains, elemStr, "degrees='false'");
   EXPECT_PRED2(sdf::testing::contains, elemStr, "0.4 0.5 0.6");
 }
@@ -994,7 +1011,7 @@ TEST(Pose1_9, IncludePoseInModelString)
 
   auto boxModel = model->ModelByName("box");
   ASSERT_NE(nullptr, boxModel);
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
       boxModel->RawPose());
 }
 
@@ -1033,7 +1050,7 @@ TEST(Pose1_9, IncludeEulerRPYPoseInModelString)
 
   auto boxModel = model->ModelByName("box");
   ASSERT_NE(nullptr, boxModel);
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
       boxModel->RawPose());
 }
 
@@ -1097,7 +1114,7 @@ TEST(Pose1_9, IncludePoseInWorld)
   const sdf::Model *model = world->ModelByIndex(0);
   ASSERT_NE(nullptr, model);
   ASSERT_EQ("first_box", model->Name());
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
             model->RawPose());
 
   model = world->ModelByIndex(1);
@@ -1108,18 +1125,18 @@ TEST(Pose1_9, IncludePoseInWorld)
   model = world->ModelByIndex(2);
   ASSERT_NE(nullptr, model);
   ASSERT_EQ("third_box", model->Name());
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
             model->RawPose());
 
   model = world->ModelByIndex(3);
   ASSERT_NE(nullptr, model);
   ASSERT_EQ("forth_box", model->Name());
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
             model->RawPose());
 
   model = world->ModelByIndex(4);
   ASSERT_NE(nullptr, model);
   ASSERT_EQ("fifth_box", model->Name());
-  EXPECT_EQ(Pose(0, 10, 0, IGN_DTOR(90), IGN_DTOR(0), IGN_DTOR(0)),
+  EXPECT_EQ(Pose(0, 10, 0, GZ_DTOR(90), GZ_DTOR(0), GZ_DTOR(0)),
             model->RawPose());
 }
