@@ -21,7 +21,7 @@
 #include "sdf/sdf.hh"
 #include "sdf/parser.hh"
 #include "sdf/ParserConfig.hh"
-#include "test_config.h"
+#include "test_config.hh"
 #include "test_utils.hh"
 
 //////////////////////////////////////////////////
@@ -178,8 +178,8 @@ TEST(URDF2SDF, ZeroMassIntermediateLinkWithFixedParentJoint)
     EXPECT_TRUE(model->JointNameExists("joint2_3"));
     const sdf::Joint *joint = model->JointByName("joint2_3");
     ASSERT_NE(nullptr, joint);
-    EXPECT_EQ(std::string("link1"), joint->ParentLinkName());
-    EXPECT_EQ(std::string("link3"), joint->ChildLinkName());
+    EXPECT_EQ(std::string("link1"), joint->ParentName());
+    EXPECT_EQ(std::string("link3"), joint->ChildName());
 
     // link3
     EXPECT_TRUE(model->LinkNameExists("link3"));
